namespace de.osci.osci12.messageparts {

	using System;
	using javax.crypto;
	using org.apache.commons.logging;
	using de.osci.osci12.common;
	using de.osci.osci12.encryption;
	using de.osci.osci12.extinterfaces;
	using System.Security.Cryptography;
	using System.Collections;

	/// <summary><p>Die Attachment-Klasse reprsentiert einen Anhang einer OSCI-Nachricht.
	/// Attachments werden in Content-Elementen mittels eines href-Attributs referenziert.
	/// Sie besitzen hierfr einen Identifier (refId), der innerhalb der Nachricht, an die das
	/// Attachment gehngt wird, eindeutig sein muss. Da dieser String als Referenz verwendet
	/// wird, empfielt sich die Verwendung von URL-encoding
	/// (s. System.Web.HttpUtility.UrlEncode(String, Encoding)) bzw. URL-decoding.</p>
	/// <p>Ein Attachment kann mit einem eigenem symmetrischen Schlssel versehen werden.
	/// Hierdurch kann in verschiedenen verschlsselten Inhaltsdatencontainern (EncryptedDataOSCI)
	/// dasselbe Attachment referenziert werden.</p>
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="Content">
	/// </seealso>
	/// <seealso cref="ContentContainer"> 
	/// </seealso>
	public class Attachment:MessagePart {

		new private static Log log;
		private static int idNr = - 1;
		public const int STATE_OF_ATTACHMENT_CLIENT = 0;
		public const int STATE_OF_ATTACHMENT_PARSING = 1;
		public const int STATE_OF_ATTACHMENT_ENCRYPTED = 2;
		protected internal SecretKey secretKey;
		public static System.String myDigest = "";
		public Hashtable mimeHeaders;

		public String boundary_string = DialogHandler.boundary;
		public bool base64 = true;

		/// <summary>Die Lnge des Attachments in Bytes.
		/// </summary>
		public long length = - 1;

		/// <summary>InputStream des Attachments.
		/// </summary>
		private System.IO.Stream ins = null;

		OSCIDataSource swapBuffer;		

		/// <summary>Setzt den Status ob das Attachment verschlsselt eingestellt werden soll. 
		/// </summary>
		internal bool encrypt;

		private System.String contentType = "text";
        protected internal Hashtable encryptedDigestValues = new Hashtable();
		internal int stateOfAttachment;

		private void  InitBlock() {			
			stateOfAttachment = STATE_OF_ATTACHMENT_CLIENT;
		}

		/// <summary> Liefert die Lnge des Attachments in Byte.
		/// </summary>
		/// <value> Lnge des Attachments
		/// </value>
		virtual public long Length 
		{
			get 
			{
				if (base64)
					return de.osci.helper.Base64.calcB64Length(this.length);
				else
					return this.length;
			}
		}

		/// <summary> Ruft den InputStream der Daten ab, wenn das Attachment einer
		/// empfangenen Nachricht entnommen wurde, oder legt diesen fest.
		/// </summary>
		/// <value> InputStream der Anhangsdaten, oder null, wenn die Nachricht
		/// nicht empfangen (d.h. mit Hilfe eine Konstruktors erzeugt wurde).
		/// </value>
		virtual public System.IO.Stream Stream {
			get {
				log.debug("Attachment Object: ");

				if (swapBuffer == null) {
					throw new System.SystemException(DialogHandler.text.getString("invalid_stateofobject") + "no OSCI-DataSource Object.");
				}
				System.IO.Stream is_Renamed = null;
				is_Renamed = swapBuffer.InputStream;
				is_Renamed.Seek(0,System.IO.SeekOrigin.Begin);
				
				if (encrypt || (stateOfAttachment == STATE_OF_ATTACHMENT_ENCRYPTED)) {
					if (secretKey != null) {
						
						log.debug("Attachment ist verschlsselt codiert.");
						return new de.osci.helper.SymCipherInputStream(is_Renamed, secretKey, false);
					}
					else {
						return null;
					}
				}
				else {
					log.debug("Daten sind nur BASE64igt");
					return is_Renamed;
				}
			}
		}
		
		/// <summary> Ruft den Content-Type (Http-Content-Type) der Binrdaten ab, oder legt diesen fest.
		/// </summary>
		/// <value> Content der Binrdaten, sollte ein gltiges Mime-Format sein (z.B. 'text/html' oder 'image/gif').
		/// </value>
		virtual public System.String ContentType {
			get {
				return contentType;
			}
			set {
				this.contentType = value;
			}
		}

		/// <summary> Setzt zustzliche MIME-Headereintrge fr den MIME-boundary
		/// Abschnitt des Attachments. Die key- und value-Strings der Hashtable mssen
		/// den MIME-Spezifikationen gengen. Die Header
		/// Content-Transfer-Encoding, Content-ID, Content-Type  und Content-Length werden
		/// von der Implementierung gesetzt und werden hier ignoriert.
		/// </summary>
		/// <value> headers MIME-Header als key-value-Paare
		/// </value>
		virtual public System.Collections.Hashtable MimeHeaders 
		{
			get 
			{
				return mimeHeaders;
			}
			set 
			{
				this.mimeHeaders = value;
				removeHeaders(mimeHeaders);
			}
		}

		private void removeHeaders(Hashtable ht)
		{
			String[] hd = new String[] { "content-transfer-encoding", "content-id", "content-length", "content-type" };

			String[] k = new String[ht.Keys.Count];
			ht.Keys.CopyTo(k, 0);
			for (int i = 0; i < k.Length; i++)
			{
				for (int j = 0; j < hd.Length; j++)
				{
					if (k[i].ToLower().Equals(hd[j]))
						ht.Remove(k[i]);
				}
			}
		}

		/// <summary> Liefert den Hashwert des Attachments.
		/// </summary>
		/// <value> Hashwert des Attachments
		/// </value>
		override public byte[] getDigestValue(String algorithm)
		{
            if (!digestValues.ContainsKey(algorithm))
            {
                digestValues.Add(algorithm, createDigest(algorithm, encrypt));
            }

            return (byte[])digestValues[algorithm];
		}

		/// <summary> Liefert true, wenn es sich um ein verschlsseltes Attachment handelt.
		/// </summary>
		/// <value>verschlsselt -> true, unverschlsselt -> false
		/// </value>
		virtual public bool Encrypted {
			get {
				return encrypt;
			}
		}

		/// <summary> Liefert den Hashwert nach der Verschlsselung fr die Nachrichtensignatur.
		/// </summary>
		/// <value>Hashwert der Nachrichtensignatur
		/// </value>
		virtual public byte[] getEncryptedDigestValue(String digestAlgorithm) 
        {
            if (!encryptedDigestValues.ContainsKey(digestAlgorithm))
                encryptedDigestValues.Add(digestAlgorithm, createDigest(digestAlgorithm, false));
            return (byte[])encryptedDigestValues[digestAlgorithm];
		}
		
		/// <summary> Dieser Konstruktor wird vom Parser aufgerufen.
		/// </summary>
        internal Attachment(System.IO.Stream ins, System.String refId, long length, String transportDigestAlgorithm)
        {
			InitBlock();
			this.ins = ins;
			RefID = refId;
			this.length = length;
			
			if (ins != null) {
                this.setInputStream(ins, false, length, transportDigestAlgorithm);
			}
		}
		
		/// <summary> Erzeugt ein neues Attachment-Objekt aus dem InputStream. Der geheime
		/// Schlssel wird fr die Verschlsselung des Attachments benutzt.
		/// </summary>
		/// <param name="ins">der InputStream, aus dem die Daten gelesen und an die Nachricht
		/// angehngt wird.
		/// </param>
		/// <param name="refId">Identifier des Anhangs, z.B. Dateiname. Dieser Identifier muss innerhalb
		/// der Nachricht, an die das Attachment gehngt wird, eindeutig sein
		/// </param>
		/// <param name="secretKey">Secret-Key mit dem verschlsselt werden soll.
		/// Der bergebene Schlssel muss daher fr diesen Algorithmus anwendbar sein.
		/// Wird dieser Parameter mit 'null' bergeben, wird ein neuer AES-256-Schlssel erzeugt.
		/// </param>
		public Attachment(System.IO.Stream ins, System.String refId, SecretKey secretKey) {
			InitBlock();
			if (ins == null) {
				throw new System.ArgumentException(DialogHandler.text.getString("invalid_firstargument") + " ins");
			}
			if (refId == null) {
				throw new System.ArgumentException(DialogHandler.text.getString("invalid_secondargument") + " refId");
			}
			if (secretKey == null) {
				this.secretKey = de.osci.osci12.encryption.Crypto.createSymKey(Constants.SYMMETRIC_CIPHER_ALGORITHM_AES256);
			}
			else {				
				this.secretKey = secretKey; 
			}
				
			this.ins = ins;
			this.encrypt = true;
			idNr++;
				
			RefID = refId;
			log.debug("RefId Name: " + refId);
			this.makeTempFile(); 
		}
		
		/// <summary> Erzeugt ein neues Attachment-Objekt aus dem InputStream. Das Attachment wird unverschlsselt bertragen.
		/// </summary>
		/// <param name="ins">der InputStream, aus dem die Daten gelesen und an die Nachricht
		/// angehngt wird.
		/// </param>
		/// <param name="refId">Identifier des Anhangs, z.B. Dateiname. Dieser Identifier muss innerhalb
		/// der Nachricht, an die das Attachment gehngt wird, eindeutig sein.
		/// </param>
		public Attachment(System.IO.Stream ins, System.String refId) {
			InitBlock();
			if (ins == null) {
				throw new System.ArgumentException("Der InputStream darf nicht null sein.");
			}
			
			if (refId == null) {
				throw new System.ArgumentException("Die refId muss eingestellt werden.");
			}
			
			this.ins = ins;
			this.encrypt = false;
			idNr++;
				
			RefID = refId;
			log.debug("RefId Name: " + refId);
			this.makeTempFile();
		}

		/// <summary> Diese Methode wird nur vom Parser aufgerufen.
		/// </summary>
		/// <param name="ins">Inputstream des Attachments.
		/// </param>
		/// <param name="encrypted">Status des Attachments (sind die Daten verschlsselt).
		/// </param>
		/// <param name="length">Lnge des Streams.
		/// </param>
        internal virtual void setInputStream(System.IO.Stream ins, bool encrypted, long length, String transportDigestAlgorithm)
        {
			log.debug("entry SetStream.");
			this.length = length;
			this.encrypt = encrypted;
			
			if (ins != null) {
				byte[] bytesIn = new byte[1024];
				swapBuffer = DialogHandler.NewDataBuffer;
					
				System.IO.Stream out_Renamed = swapBuffer.OutputStream;
                HashAlgorithm encMsgDigest = null;
                if (transportDigestAlgorithm != null)
                {
                    encMsgDigest = MoreSupport.createMessageDigest(transportDigestAlgorithm);
                    out_Renamed = new CryptoStream(out_Renamed, encMsgDigest, CryptoStreamMode.Write);
                }
				ins = new System.IO.BufferedStream(ins, 1024);
				int count = 0;

				while ((count = SupportClass.ReadInput(ins, ref bytesIn, 0, bytesIn.Length)) > 0) 
				{
                    out_Renamed.Write(bytesIn, 0, count);
				}
				ins.Close();
				out_Renamed.Close();
//				encMsgDigestStream.Close();

                if (transportDigestAlgorithm != null)
                {
                    encryptedDigestValues.Add(transportDigestAlgorithm, encMsgDigest.Hash);
                }
			}
			else 
			{
				throw new System.ArgumentException("Der Inputstream darf nicht 'null' sein");
			}
		}
		
		public bool hasDigestValue(String digestAlgorithm)
		{
			return digestValues.ContainsKey(digestAlgorithm);
		}

		/// <summary> Diese Methode liet die Daten aus dem InputStream und verschlsselt sie und
		/// ermittelt den Digest des Attachments. Das verschlsselte Ergebnis wird in einen
		/// OSCIDataSource geschrieben.
		/// </summary>
		private void  makeTempFile() {
			if (swapBuffer != null) {
				return ;
			}
			
			log.debug("MakeTempFile wurde aufgerufen.");
			swapBuffer = DialogHandler.NewDataBuffer;
			System.IO.Stream out_Renamed = swapBuffer.OutputStream;
			HashAlgorithm encMsgDigest=null;
            HashAlgorithm msgDigest = null;
			CryptoStream encMsgDigestStream;
			CryptoStream msgDigestStream;
			de.osci.helper.SymCipherOutputStream cipherOut;

			msgDigest = new SHA256Managed();
	
			if (encrypt) {
				encMsgDigest = new SHA256Managed();
				encMsgDigestStream = new CryptoStream(out_Renamed, encMsgDigest, CryptoStreamMode.Write);
				cipherOut = new de.osci.helper.SymCipherOutputStream(encMsgDigestStream, secretKey, true);
				out_Renamed = cipherOut;
			}
			else 
				log.debug("Unverschlsseltes Attachment wird erstellt");				
			
			msgDigestStream = new CryptoStream(out_Renamed, msgDigest, CryptoStreamMode.Write);			
			System.IO.BufferedStream bufferdIn = new System.IO.BufferedStream(ins, 1024);

			int count = 0;
			byte[] bytesIn = new byte[1024];
			length = 0;
			
			while ((count = bufferdIn.Read(bytesIn, 0, bytesIn.Length)) > 0) 
			{
				msgDigestStream.Write(bytesIn, 0, count);
			}
			msgDigestStream.Close();
			bufferdIn.Close();	
		
			digestValues.Add(Constants.DIGEST_ALGORITHM_SHA256, msgDigest.Hash);

			if (encrypt)
				encryptedDigestValues.Add(Constants.DIGEST_ALGORITHM_SHA256, encMsgDigest.Hash);
			else
                encryptedDigestValues.Add(Constants.DIGEST_ALGORITHM_SHA256, digestValues[Constants.DIGEST_ALGORITHM_SHA256]);
    

			length = swapBuffer.Length;
			
			System.IO.Stream is_Renamed = swapBuffer.InputStream;
		}
		
		private byte[] createDigest(String algo, bool enc) {
			System.IO.Stream in_Renamed = null;
			
			in_Renamed = swapBuffer.InputStream;
				
			if (in_Renamed == null) {
				throw new System.IO.IOException(DialogHandler.text.getString("invalid_inputstream"));
			}
			
			in_Renamed.Seek(0,System.IO.SeekOrigin.Begin);

			log.debug("State of Attachment: " + stateOfAttachment + " Verschlsselt: " + encrypt);

//            if ((stateOfAttachment == STATE_OF_ATTACHMENT_CLIENT) || ((stateOfAttachment == STATE_OF_ATTACHMENT_ENCRYPTED) && secretKey != null))
            if (!(stateOfAttachment == STATE_OF_ATTACHMENT_PARSING) && enc)
            {
                System.Security.Cryptography.SymmetricAlgorithm sa = secretKey.GetAlgo();
				in_Renamed = new de.osci.helper.SymCipherInputStream(in_Renamed, secretKey, false);
			}

			HashAlgorithm msgDigest = MoreSupport.createMessageDigest(algo);
			CryptoStream msgDigestStream;

			in_Renamed = new CryptoStream(in_Renamed, msgDigest, CryptoStreamMode.Read);

			byte[] bytes = new byte[1024];
			int count;
				
			while ((count = in_Renamed.Read(bytes, 0, bytes.Length)) > 0)
			{
			}
			in_Renamed.Close();
//			digestValues.Add(algo, msgDigest.Hash);

            return msgDigest.Hash;

		}

		public bool Base64Encoding
		{
			set
			{
				base64 = value;
			}
		}

		public String Boundary
		{
			set
			{
				boundary_string = value;
			}
		}

		/// <summary> Interne Methode, wird von Anwendungen normalerweise nicht aufgerufen.
		/// Sie serialisiert das MIME-Boundary des Attachments.
		/// </summary>
		/// <param name="out_Renamed">Outputstream, in den das Attachment serialisiert wird.
		/// </param>
		/// <exception cref="System.Exception">IOException
		/// </exception>
		public override void  writeXML(System.IO.Stream out_Renamed) 
		{
		    MoreSupport.ToStream(out_Renamed,"\r\n--" + boundary_string +
				"\r\nContent-Type: " + contentType + "\r\n",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed, "Content-Transfer-Encoding: ", Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed, base64 ? "base64" : "8bit", Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed, "\r\nContent-ID: <" + RefID + ">\r\n",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed, "Content-Length: " + Length + "\r\n",Constants.CHAR_ENCODING);
 
			if (mimeHeaders != null)
			{
    			String key;				
				Hashtable mh = (Hashtable)mimeHeaders.Clone();
				removeHeaders(mh);
    			for (IDictionaryEnumerator e = mh.GetEnumerator(); e.MoveNext();)
    			{
    				key = (String)e.Key;
    				MoreSupport.ToStream(out_Renamed, key.Trim() + ": " + ((String)mh[key]).Trim() + "\r\n", Constants.CHAR_ENCODING);    	
    			}
			}
			MoreSupport.ToStream(out_Renamed, "\r\n", Constants.CHAR_ENCODING);

			log.debug("######### SwapBuffer#### " + swapBuffer);
			
			System.IO.Stream in_Renamed = swapBuffer.InputStream;
			in_Renamed.Seek(0,System.IO.SeekOrigin.Begin);
			
			if (base64)
				out_Renamed = new Base64OutputStream(out_Renamed, true);

			int count = 0;
			byte[] bytesIn = new byte[1024];
			
			while ((count = SupportClass.ReadInput(in_Renamed, ref bytesIn, 0, bytesIn.Length)) > - 1) {
				out_Renamed.Write(bytesIn, 0, count);
			}
			if (base64)
				out_Renamed.Flush();
		}
		
		static Attachment() {
			log = LogFactory.getLog(typeof(Attachment));
		}
	}
}