namespace de.osci.osci12.messageparts {

	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.common;
	using de.osci.osci12.extinterfaces;
	using de.osci.osci12.signature;
	using System.Security.Cryptography;
	using dothelpers;

	/// <summary> <p><H4>Content</H4></p>
	/// <p>Die Content-Klasse reprsentiert einen Content-Eintrag in einer OSCI-
	/// Nachricht. Die Content-Eintrge befinden sich in ContentContainer-Eintrgen
	/// und enthalten die eigentlichen Nutzdaten, die in beliebigen Daten,
	/// Refenrenzen auf Attachments oder wiederum in Inhaltsdatencontainern bestehen
	/// knnen.</p>
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class Content:MessagePart {

		new private static Log log;
		new private static int id = - 1;
		private Attachment attachment = null;
		protected OSCIDataSource swapBuffer;
		private ContentContainer coco;
		private int contentType;
		public const int ATTACHMENT_REFERENCE = 0;
		public const int CONTENT_CONTAINER = 1;
		public const int DATA = 2;
		internal String coNS;
		private System.IO.Stream transformedDataStream;
		protected String b64;
		static private int zaehl;

		override public byte[] getDigestValue(String digestAlgorithm) 
        {
            if (digestValues.ContainsKey(digestAlgorithm))
					return (byte[])digestValues[digestAlgorithm];

			if (contentType == DATA) 
            {
			    HashAlgorithm msgDigest = MoreSupport.createMessageDigest(digestAlgorithm);
				if (transformedDataStream == null) 
                {
						// Bibliothek akzeptiert nur Base64-codierte Daten. Workaround fr den Fall,
						// dass dieser Transformer beim Laden zwischengespeicherter Daten verloren gegangen ist (StoredMessage).
						// Signaturen mit Transformern knnen in diesem Fall nur mit Hilfe neuer Content-Instanzen
						// erzeugt werden.
					if (transformers.Count == 0)
						transformers.Add(b64);
				    if (transformers.Count > 1) throw new dothelpers.IllegalStateException(de.osci.osci12.common.DialogHandler.text.getString("no_transformed_data"));
					swapBuffer.InputStream.Position=0;
					transformedDataStream = new Base64InputStream(swapBuffer.InputStream);
				}
				CryptoStream outStream = new CryptoStream(new System.IO.MemoryStream(), msgDigest, CryptoStreamMode.Write);
				int s;
				byte[] tmp = new byte[1024];
				long p=	swapBuffer.InputStream.Position;
				try
				{
				    swapBuffer.InputStream.Position=p;
					while ((s=transformedDataStream.Read(tmp, 0, tmp.Length))>0)
					{
					    outStream.Write(tmp, 0, s);
					}
				}
				catch(Exception x)
				{
				    throw x;
				}
				outStream.Close();
//					tmp = msgDigest.Hash;
				digestValues.Add(digestAlgorithm, msgDigest.Hash);
			}
			else 
            {
			    return base.getDigestValue(digestAlgorithm);
			}

			return (byte[])digestValues[digestAlgorithm];
		}

		/// <summary> Liefert den InputStream der Daten zurck, wenn der Content einer
		/// empfangenen Nachricht entnommen wurde.
		/// </summary>
		/// <value> InputStream der Inhaltsdaten, oder null, wenn die Nachricht
		/// nicht empfangen (d.h. mit Hilfe eine Konstruktors erzeugt wurde).
		/// </value>
		virtual public System.IO.Stream ContentStream 
		{
			get 
			{
				if (swapBuffer != null) 
				{
					swapBuffer.InputStream.Seek(0,System.IO.SeekOrigin.Begin);
					return new Base64InputStream(swapBuffer.InputStream);
				} 
				else 
				{
					return null;
				}
			}
		}

		/// <summary> Liefert die eingestellten Daten des Content zurck.
		/// </summary>
		/// <value> String der Inhaltsdaten, oder null, wenn die Nachricht
		/// nicht empfangen (d.h. mit Hilfe eine Konstruktors erzeugt wurde).
		/// </value>
		virtual public System.String ContentData 
		{
			get 
			{
				if (swapBuffer != null) 
				{
					swapBuffer.InputStream.Seek(0,System.IO.SeekOrigin.Begin);						
					System.IO.Stream in_Renamed = new Base64InputStream(swapBuffer.InputStream);
                    return MoreSupport.NewString(in_Renamed);
				} 
				else 
				{
					return null;
				}
			}
		}

		/// <summary> Liefert das referenzierte Attachment zurck.
		/// </summary>
		/// <value> Attachment-Objekt oder null, wenn das Content-Objekt Nutzdaten
		/// oder einen Inhaltsdatencontainern enthlt.
		/// </value>
		virtual public Attachment Attachment {
			get {
				return attachment;
			}			
		}

		/// <summary> Liefert den Inhaltsdatencontainer zurck.
		/// </summary>
		/// <value> ContentContainer-Objekt oder null, wenn das Content-Objekt
		/// Nutzdaten oder eine Referenz auf ein Attachment enthlt.
		/// </value>
		virtual public ContentContainer ContentContainer {
			get {
				return coco;
			}	
		}

		/// <summary> Gibt die Art des Inhalts des Content-Objektes in Form eines Identifiers
		/// zurck. Mgliche Werte sind ATTACHMENT_REFERENCE, CONTENT_CONTAINER und DATA
		/// </summary>
		/// <value> Inhaltstyp
		/// </value>
		virtual public int ContentType {
			get {
				return contentType;
			}			
		}
		
		/// <summary> Legt ein Content-Objekt an. Die Daten werden aus dem bergebenen
		/// InputStream gelesen.
		/// </summary>
		/// <param name="ins">InputStream
		/// </param>
		public Content(System.IO.Stream ins) {
			b64 = "<"+dsNSPrefix+":Transform Algorithm=\"" + Constants.TRANSFORM_BASE64+"\"></"+dsNSPrefix+":Transform>";
			if (ins == null) 
			{
				throw new System.ArgumentException(DialogHandler.text.getString("invalid_firstargument") + " ins");
			}
			
			id++;
			this.RefID = "content" + id;
			contentType = DATA;
			load(ins);
			transformers.Add(b64);
		}
		
		/// <summary> Legt ein Content-Objekt an. Die Daten werden aus dem bergebenen
		/// InputStream gelesen.
		/// </summary>
		/// <param name="dataSource">InputStream
		/// </param>
		/// <exception cref="System.ArgumentException">Im Fehlerfall
		/// </exception>
		public Content(de.osci.osci12.extinterfaces.OSCIDataSource dataSource) {
			b64 = "<"+dsNSPrefix+":Transform Algorithm=\"" + Constants.TRANSFORM_BASE64+"\"></"+dsNSPrefix+":Transform>";
			log.debug("Konstruktor");
			if (dataSource == null) {
				throw new System.ArgumentException(DialogHandler.text.getString("invalid_firstargument") + " dataSource");
			}
			id++;
			this.RefID = "content" + id;
			swapBuffer = dataSource;
			contentType = DATA;
		}
		
		/// <summary> Legt ein Content-Objekt mit dem Inhalt des bergebenen Strings an.
        /// <b>Hinweis:</b> Die bergabe von Inhaltsdaten als String ist zur 
        /// Bequemlichkeit fr Daten vorgesehen, die empfngerseitig ebenfalls
        /// als String der OSCI-Nachricht entnommen und weiterverarbeitet werden.
        /// Dies ist zu beachten, wenn z.B. XML-Dokumente bergeben werden, die
        /// eine Zeichensatzdeklaration beinhalten und mglicherweise empfngerseitig
        /// als Binrdaten einem Parser bergeben werden. Die OSCI-Bibliothek
        /// bertrgt die Daten UTF-8-codiert.   
		/// </summary>
		/// <param name="data">Inhalt
		/// </param>
		/// <exception cref="System.ArgumentException">Im Fehlerfall
		/// </exception>
		public Content(System.String data) {
			b64 = "<"+dsNSPrefix+":Transform Algorithm=\"" + Constants.TRANSFORM_BASE64+"\"></"+dsNSPrefix+":Transform>";
			if (data == null) 
			{
				throw new System.ArgumentException(DialogHandler.text.getString("invalid_firstargument") + " data");
			}
			id++;
			this.RefID = "content" + id;
			load(new System.IO.MemoryStream(MoreSupport.ToByteArray(data,Constants.CHAR_ENCODING)));
			contentType = DATA;
			transformers.Add(b64);
		}
		
		/// <summary> Legt ein Content-Objekt an, welches eine Referenz auf ein Attachment
		/// enthlt.
		/// </summary>
		/// <param name="attachment">Attachment
		/// </param>
		/// <exception cref="System.ArgumentException">Im Fehlerfall
		/// </exception>
		public Content(Attachment attachment) {
			b64 = "<"+dsNSPrefix+":Transform Algorithm=\"" + Constants.TRANSFORM_BASE64+"\"></"+dsNSPrefix+":Transform>";
			transformers.Add(can);
			if (attachment == null) {
				throw new System.ArgumentException(DialogHandler.text.getString("invalid_firstargument") + " attachment");
			}		
			id++;
			this.attachment = attachment;
			this.RefID = "content" + id;
			contentType = ATTACHMENT_REFERENCE;
		}

		/// <summary>Legt ein Content-Objekt mit dem Inhalt des bergebenen Strings an.
		/// bergebene Transformer-Strings werden in die XML-Signatur eingetragen,
		/// die Strings mssen die gesamten Transformer-Eintrge gem der
		/// XML-Signature-Spezifikation in kanonischer Form enthalten.
		/// Die zu signierenden transformierten Daten mssen in dem als dritten
		/// Parameter bergebenen String enthalten sein.
        /// <b>Man beachte den Hinweis zur bergabe von Inhaltsdaten als String.</b>
		/// </summary>
		/// <param name="data">Inhaltsdaten.
		/// </param>
		/// <param name="transformer">Array der Transformereintrge.
		/// </param>
		/// <param name="transformedData">String mit den transformierten Daten.
		/// </param>
		///@throws IOException bei Lesefehlern
		///@throws OSCIException bei Problemn beim Aufbau des OSCI-Signatureintrags
		///@throws NoSuchAlgorithmException wenn der verwendete Security-Provider
		///den erforderlichen Hash-Algorithmus nicht untersttzt.
		public Content(String data, String[] transformer, String transformedData):this(data){
			b64 = "<"+dsNSPrefix+":Transform Algorithm=\"" + Constants.TRANSFORM_BASE64+"\"></"+dsNSPrefix+":Transform>";
			if ((transformer != null) && (transformer.Length > 0)) 
			{
				transformedDataStream = new System.IO.MemoryStream(MoreSupport.ToByteArray(transformedData));
				for (int i=0; i<transformer.Length; i++)
					this.transformers.Add(transformer[i]);	
//				byte[] unimportant = DigestValue;
			}
		}
		/// <summary>
		/// Legt ein Content-Objekt an. Die Daten werden aus dem bergebenen
		/// InputStream gelesen. bergebene Transformer-Strings werden in die
		/// XML-Signatur eingetragen, die Strings mssen die gesamten Transformer-Eintrge
		/// gem der XML-Signature-Spezifikation in kanonischer Form enthalten.
		/// Die zu signierenden transformierten Daten werden aus dem als dritten
		/// Parameter bergebenen Stream gelesen.
		/// </summary>
		/// <param name="ins">InputStream der Inhaltsdaten
		/// </param>
		/// <param name="transformer">Array der Transformereintrge
		/// </param>
		/// <param name="transformedData">InputStream der transformierten Daten
		/// </param>
		/// <exception cref="OSCIException">bei Problemen beim Aufbau des OSCI-Signatureintrags
		/// </exception>
		public Content(de.osci.osci12.dotcommon.InputStream ins, String[] transformer, de.osci.osci12.dotcommon.InputStream transformedData):this(ins){
			b64 = "<"+dsNSPrefix+":Transform Algorithm=\"" + Constants.TRANSFORM_BASE64+"\"></"+dsNSPrefix+":Transform>";
			if ((transformer != null) && (transformer.Length > 0)) 
			{
				transformedDataStream = transformedData;
				for (int i=0; i<transformer.Length; i++)
					this.transformers.Add(transformer[i]);	
//				byte[] s=this.DigestValue;
			}
		}
		/// <summary> Legt ein Content-Objekt an, welches ein ContentContainer-Objekt enthlt.
		/// </summary>
		/// <param name="contentContainer">Inhaltsdatencontainer
		/// </param>
		public Content(ContentContainer contentContainer) {
			b64 = "<"+dsNSPrefix+":Transform Algorithm=\"" + Constants.TRANSFORM_BASE64+"\"></"+dsNSPrefix+":Transform>";
			transformers.Add(can);
			if (contentContainer == null) {
				throw new System.ArgumentException(DialogHandler.text.getString("invalid_firstargument") + " contentContainer");
			}
			
			id++;
			this.coco = contentContainer;
			this.RefID = "content" + id;
			contentType = CONTENT_CONTAINER;
		}
		
		private void  load(System.IO.Stream input) {
			if (swapBuffer == null) {
				swapBuffer = DialogHandler.NewDataBuffer;
				
				Base64OutputStream b64out = new Base64OutputStream(swapBuffer.OutputStream, false);
				byte[] bytes = new byte[1024];
				int anz = 0;
				
				while ((anz = SupportClass.ReadInput(input, ref bytes, 0, bytes.Length)) > 0) {
					b64out.Write(bytes, 0, anz);
				}
				b64out.Close();
			}
		}

		/// <summary> Interne Methode
		/// </summary>
		/// <returns>String</returns>
		public override System.String ToString() {
			System.IO.MemoryStream out_Renamed = new System.IO.MemoryStream();
			writeXML(out_Renamed);
			return MoreSupport.ToByteArray(out_Renamed.ToString(),Constants.CHAR_ENCODING).ToString();
		}
		
		/// <summary>  Interne Methode, wird von Anwendungen normalerweise nicht aufgerufen.
		/// </summary>
		/// <param name="out_Renamed">out
		/// </param>
		/// <exception href="IOException">IOException
		/// </exception>
		public override void writeXML(System.IO.Stream out_Renamed){
			writeXML(out_Renamed, false);
		}
		internal void  writeXML(System.IO.Stream out_Renamed,bool inner) {
		  MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix + ":Content",Constants.CHAR_ENCODING);
			if (attachment != null) 
			{
				log.debug("SCHREIBE Attachment." + RefID);
				if (!inner)
				{
					if (coNS==null)
						out_Renamed.Write(ns,0,ns.Length);
					else MoreSupport.ToStream(out_Renamed,coNS,Constants.CHAR_ENCODING);
				}
				MoreSupport.ToStream(out_Renamed," Id=\"" + RefID + "\" href=\"cid:" + attachment.RefID + "\"></" + osciNSPrefix + ":Content>",Constants.CHAR_ENCODING);
      		} 
			else if (coco != null) 
				{
				log.debug("Schreibe ContentConatiner." + RefID);
				if (!inner){if (coNS==null)out_Renamed.Write(ns,0,ns.Length);
								else MoreSupport.ToStream(out_Renamed,coNS,Constants.CHAR_ENCODING);}
				MoreSupport.ToStream(out_Renamed," Id=\"" + RefID + "\">",Constants.CHAR_ENCODING);
				coco.writeXML(out_Renamed);
				MoreSupport.ToStream(out_Renamed,"</" + osciNSPrefix + ":Content>",Constants.CHAR_ENCODING);
			} 
			else 
			{
				log.debug("Schreibe Data.");
				MoreSupport.ToStream(out_Renamed,"><" + osciNSPrefix + ":Base64Content",Constants.CHAR_ENCODING);
				if (!inner)if (coNS==null)out_Renamed.Write(ns,0,ns.Length);
				MoreSupport.ToStream(out_Renamed," Id=\"" + RefID + "\">",Constants.CHAR_ENCODING);
				int count;
				byte[] inBytes = new byte[1024];
				swapBuffer.InputStream.Seek(0,System.IO.SeekOrigin.Begin);

				while ((count = SupportClass.ReadInput(swapBuffer.InputStream, ref inBytes, 0, inBytes.Length)) > 0) {
					out_Renamed.Write(inBytes, 0, count);
				}
				out_Renamed.Flush();
				MoreSupport.ToStream(out_Renamed,"</" + osciNSPrefix + ":Base64Content></" + osciNSPrefix + ":Content>",Constants.CHAR_ENCODING);
			}
		}
		
		static Content() {
			log = LogFactory.getLog(typeof(Content));
		}

		/// <summary>Liefert die Transformereintrge in der Signatur.
		/// </summary>
		/// <returns>String-Array mit den Transformereintrgen
		/// </returns>
		public String[] getTransformerForSignature() {
			if (transformers == null || transformers.Count < 2) return null;
			String[] ret = new String[transformers.Count-1];
			for (int i = 1; i < transformers.Count; i++)
				ret[i-1] = (String)transformers[i];
			return ret;

		}

		/// <summary>Bevor eine Signaturprfung an dem ContentContainer-Objekt durchgefhrt
		/// werden kann, welches dieses Content-Objekt enthlt, mssen mit dieser
		/// Methode die transformierten Daten bergeben werden. Dies betrifft nur
		/// Content-Objekte, die unter Anwendung von Transformationen signiert wurden.
		/// </summary>
		/// <param name="transformedData">transformierte Daten
		/// </param>
		/// <seealso cref="setTransformedData(System.IO.Stream)">
		/// </seealso>
		/// <seealso cref="de.osci.osci12.messageparts.ContentContainer.checkSignature">
		/// </seealso>
		public void setTransformedData(String transformedData) {
			try {
				setTransformedData(new System.IO.MemoryStream(MoreSupport.ToByteArray(transformedData)));
			}
			catch (Exception ex) {
				// Kann bei UTF-8 nicht auftreten
			}
		}
		/// <summary>Bevor eine Signaturprfung an dem ContentContainer-Objekt durchgefhrt
		/// werden kann, welches dieses Content-Objekt enthlt, mssen mit dieser
		/// Methode die transformierten Daten bergeben werden. Dies betrifft nur
		/// Content-Objekte, die unter Anwendung von Transformationen signiert wurden.
		/// </summary>
		/// <param name="transformedData">transformierte Daten
		/// </param>
		/// <seealso cref="setTransformedData(String)">
		/// </seealso>
		/// <seealso cref="de.osci.osci12.messageparts.ContentContainer.checkSignature">
		/// </seealso>
		public void setTransformedData(System.IO.Stream transformedData) {
			if (getTransformerForSignature()==null) throw new Exception(DialogHandler.text.getString("no_transformer_state"));
			if (transformedDataStream!=null) throw new Exception(DialogHandler.text.getString("illegal_change_of_transformed_data"));
			transformedDataStream = transformedData;
		}
	}
}