namespace de.osci.osci12.messageparts {

	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.common;
	using de.osci.osci12.roles;
	using de.osci.osci12.signature;
	using de.osci.osci12.encryption;
	using de.osci.helper;
    using System.Security.Cryptography;
    using System.Collections;

	/// <summary> <p><H4>ContentContainer</H4></p>
	/// <p>Die ContentContainer-Klasse stellt einen OSCI-Inhaltsdatenscontainer dar. 
	/// Ein ContentContainer kann einen oder mehrere Content-oder EncryptedData-Objekte enthalten. 
	/// Attachments werden als Contents eingestellt, die eine Referenz auf das Attachment enthalten.</p>
	/// <p>Ein Content-Container wird als eine Einheit signiert und / oder verschlsselt.</p>
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class ContentContainer:MessagePart {

		private void  InitBlock() {
			stateOfObject = STATE_OF_OBJECT_CONSTRUCTION;
			signerList = new ArrayList();
			roles = new ArrayList();
			attachments = new Hashtable();
			contentList = new ArrayList();
			encryptedDataList = new ArrayList();
		}

        public OSCISignature[] Signatures
        {
            get
            {
                return (OSCISignature[])signerList.ToArray(typeof(OSCISignature));
            }
        }

		/// <summary> Liefert die eingestellten Attachment-Objekte des ContentContainer.
		/// </summary>
		/// <value> Array der referenzierten Attachments
		/// </value>
		virtual public Attachment[] Attachments {
			get {
				log.debug("Anzahl der Attachments: " + attachments.Count);
				Attachment[] atts = null;
				if (attachments.Count > 0) {
					atts = new Attachment[attachments.Count];
					System.Collections.IEnumerator enum_Renamed = attachments.GetEnumerator();
					int count = 0;
					while (enum_Renamed.MoveNext()) {
						atts[count] = (Attachment) (((System.Collections.DictionaryEntry)((System.Object)enum_Renamed.Current))).Value;
						count++;
					}
				}
				return atts;
			}
			
		}

		/// <summary> Liefert die eingestellten Rollen-Objekte des ContentContainer, welche fr die
		/// Signatur sowie untergeordnete Verschlsselungen und Signaturen verwendet wurden.
		/// </summary>
		/// <value> Array der Rollenobjekte
		/// </value>
		virtual public Role[] Roles {
			get {
				return (Role[]) this.roles.ToArray(typeof(Role));
			}
		}

		/// <summary> Liefert die Rollenobjekte, von denen die Signaturen angebracht wurden.
		/// </summary>
		/// <value> Array der Rollenobjekte.
		/// </value>
		virtual public Role[] Signers {
			get {
				Role[] roles = new Role[signerList.Count];
				for (int i = 0; i < roles.Length; i++) {
					roles[i] = ((OSCISignature) signerList[i]).signer;
				}
				return roles;
			}
			
		}

		/// <summary> Liefert die im ContentContainer enthaltenen Content-Objekte.
		/// </summary>
		/// <value>Array der enthaltenen Content-Objekt
		/// </value>
		/// <seealso cref="Content">
		/// </seealso>
		virtual public Content[] Contents {
			get {
				return (Content[]) contentList.ToArray(typeof( Content));
			}
		}

		/// <summary> Liefert die im ContentContainer enthaltenen verschlsselten Daten als
		/// EncryptedData-Objekte.
		/// </summary>
		/// <value>Array der enthaltenen EncryptedData-Objekte
		/// </value>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		virtual public EncryptedDataOSCI[] EncryptedData {
			get {
				return (EncryptedDataOSCI[]) encryptedDataList.ToArray(typeof(EncryptedDataOSCI));
			}		
		}
		
		new private static Log log;
		internal static bool STATE_OF_OBJECT_CONSTRUCTION = true;
		internal static bool STATE_OF_OBJECT_PARSING = false;
		internal bool stateOfObject;
		
		/// <summary>
		/// lfdNr fr die ContentContainer
		/// </summary>
		private static int idNr = - 1;

        /// <summary>
        /// lfdNr fr die signed signature properties Ids
        /// </summary>
        int signedSigPropNr = 0;

        /// <summary>
		///  Vector fr die Signer dieses ContentContainers
		/// </summary>
		internal ArrayList signerList;
		
		/// <summary>
		/// Vector fr alle Roles des ContentContainer. 
		/// Beinhalten auch die Rollen Objekte eventueller Encrypted Data Childs
		/// </summary>
		internal ArrayList roles;
		
		/// <summary>
		/// Vector fr die Attachments
		/// </summary>
		internal Hashtable attachments;
		
		/// <summary>
		/// Vector fr die Contents
		/// </summary>
		private ArrayList contentList;
		
		/// <summary>
		/// Vector fr die EncryptedData Objekte
		/// </summary>
		private ArrayList encryptedDataList;

		private de.osci.osci12.extinterfaces.OSCIDataSource signatureData = null;
		
		/// <summary>Legt ein ContentContainer-Objekt an.
		/// </summary>
		public ContentContainer() {
			InitBlock();
			id = typ + (++idNr);
		}
		
		/// <summary> berprft die Signatur zu dem bergebenen Role Objekt. Zur Installation
		/// von ggf. erforderlichen Transformern s. checkAllSignatures().
		/// Bevor eine Signaturprfung an dem ContentContainer-Objekt durchgefhrt
		/// werden kann, mssen denjenigen Content-Objekten, die unter Anwendung von
		/// Transformationen signiert wurden, die transformierten Daten bergeben werden.
		/// Welche Transformationen erforderlich sind, kann (bei Content-Objekten mit
		/// Inhaltsdaten) mit Hilfe der Methode getTransformerForSignature()
		/// abgefragt werden.
		/// </summary>
		/// <param name="signtureRole">Rollen-Objekt mit dem Zertifikat zur Signatur.
		/// </param>
		/// <returns> true, wenn die Prfung positiv ausgefallen ist.
		/// </returns>
        public virtual bool checkSignature(Role signatureRole)
        {
			log.debug("(start) checkSignature (...) ");
            if (checkContainsSigner(signatureRole))
            {
                OSCISignature[] signature = this.findSignatureObject(signatureRole);
				for (int j = 0; j < signature.Length; j++) {
					log.debug("Signature Objekt: " + signature[j]);
					System.Collections.Hashtable newHashes = new System.Collections.Hashtable();
					for (int i = 0; i < contentList.Count; i++) {
						Content co = ((Content) contentList[i]);
						SupportClass.PutElement(newHashes,"#" + co.RefID, co);
					}
					for (int i = 0; i < encryptedDataList.Count; i++) {
						EncryptedDataOSCI encData = (EncryptedDataOSCI) encryptedDataList[i];
						SupportClass.PutElement(newHashes,"#" + encData.RefID, encData);
					}
					System.Collections.IEnumerator enum_Renamed = attachments.GetEnumerator();
					while (enum_Renamed.MoveNext()) {
						Attachment att = (Attachment) ((System.Collections.DictionaryEntry)enum_Renamed.Current).Value;
						SupportClass.PutElement(newHashes,"cid:" +  att.RefID, att);
					}
					OSCISignatureReference[] sigRefs = signature[j].getReferences();

                    if (log.DebugEnabled)
                    {
					    for (int i = 0; i < sigRefs.Length; i++) 
						    log.debug("Reference: " + sigRefs[i].id);
					}

                    log.debug("Anzahl contents und encData: " + newHashes.Count + " OSCISIGReferenzen: " + sigRefs.Length);

					ArrayList checkedList = new ArrayList();
                    MessagePart mp;
                    byte[] digest, newDigest;
				    
					// sind die Hashwerte gleich?
					for (int i = 0; i < sigRefs.Length; i++) {
						System.String id = sigRefs[i].RefID;
						log.debug("ID die kontrolliert wird: " + id);
                        digest = sigRefs[i].digestValue;
                        if (id.Equals("#" + signature[j].signingPropsId))
                        {
                            HashAlgorithm msgDigest = MoreSupport.createMessageDigest(sigRefs[i].DigestMethodAlgorithm);
                            newDigest = msgDigest.ComputeHash(MoreSupport.ToByteArray(signature[j].signingProperties, Constants.CHAR_ENCODING));
                        }
                        else
                        {
                            mp = (MessagePart)newHashes[id];
                            newDigest = mp.getDigestValue(sigRefs[i].DigestMethodAlgorithm);
                        }
						if (newDigest == null) {
							log.debug("Der aktuelle Digest fr die RefID: " + id + " konnte nicht gefunden werden");
							return false;
						}
						if (! Tools.compareByteArrays(digest, newDigest))
						{
							log.error("Der Digest fr die RefID: " + id + " ist falsch! Eingestellter Digeste:" + Crypto.toHex(digest) + " neuer Digest: " + Crypto.toHex(newDigest));
							return false;
						}
						else 
                        {
							log.debug("Der Digest ist richtig.");
						}

                        checkedList.Add(id);
					}

                    String key;
					System.Collections.IEnumerator enum1 = newHashes.Keys.GetEnumerator();
					while (enum1.MoveNext()) 
                    {
						log.debug("Reference: " + enum1.Current);
                        key = (String) enum1.Current;
                        if (!checkedList.Contains(key))
                        {
                            log.error("Unsigniertes Containerelement gefunden: " + id);
                            return false;
                        }
                    }

                    java.security.cert.X509Certificate c = signatureRole.SignatureCertificate;

/*                    if ((c.getKeyUsage() != null) && !c.getKeyUsage()[0] && !c.getKeyUsage()[1])
                    {
                        log.error("Signature certificate has wrong key usage.");
                        return false;
                    }*/
                    //  nun noch das Signed Info testen

                    bool res = MoreSupport.checkSignature(signatureRole.SignatureCertificate, signature[j].SignedInfoBytes, signature[j].signatureValue, signature[j].signatureAlgorithm);
					if (!res)
						return false;
					else 
						log.debug("Die Signaturprfung wurde erfolgreich abgeschlossen.");
				}
			}
			else {
				log.warn("Content-Signatur konnte nicht berprft werden (Falsches Role Objekt).");
				throw new System.ArgumentException("Content-Signatur konnte nicht berprft werden (Falsches Role Objekt).");
			}
			log.debug("(ende) findSignaurObject");
			return true;
		}
		
		/// <summary> Liefert das zum Role Objekt gehrige XML-Signature Objekt.
		/// </summary>
		/// <param name="roleToCheck">Rollenobjekt, fr das die Signatur-Objekt bergeben werden soll.
		/// </param>
		/// <returns> OSCISignature Objekt, oder null sobald kein korospondierendes Objekt exitstiert
		/// </returns>
		private OSCISignature[] findSignatureObject(Role roleToCheck) {
			log.debug("(start) findSignaurObject (...) ");
			System.Collections.IEnumerator enum_Renamed = this.signerList.GetEnumerator();
			log.debug("Anzahl SIgnerList: " + signerList.Count);
			System.Collections.ArrayList sigs = new System.Collections.ArrayList();
			while (enum_Renamed.MoveNext()) {
				OSCISignature signature = (OSCISignature) enum_Renamed.Current;
				if (signature.signer.SignatureCertificate.Equals(roleToCheck.SignatureCertificate)) {
					sigs.Add(signature);
				}
			}
			OSCISignature[] ret = new OSCISignature[sigs.Count];
			for (int i=0; i<ret.Length; i++)
				ret[i] = (OSCISignature)sigs[i];
			return ret;
		}
		
		/// <summary> Fgt ein Signature() XML-Signature Element dem Objekt hinzu (wird vom Parser aufgerufen)
		/// </summary>
		/// <param name="signature">OSCISignature Objekt
		/// </param>
		internal virtual void  addSignature(OSCISignature signature) {
			signerList.Add(signature);
			roles.Add(signature.signer);
		}
		
		/// <summary> berprft, ob das Role Objekt fr die Signaturprfung verwendet werden kann.
		/// </summary>
		/// <param name="roleToCheck">Role Objekt
		/// </param>
		/// <returns> true, sobald das Zertifikat des Role Objektes zur Signaturprfung benutzt werden kann.
		/// </returns>
		private bool checkContainsSigner(Role roleToCheck) {
			log.debug("(start) checkContainsRole (...) ");
			Role[] signer = this.Signers;
			for (int i = 0; i < signer.Length; i++) {
				log.debug("Role Object: " + signer[i].id);
				if (signer[i].SignatureCertificate.Equals(roleToCheck.SignatureCertificate)) {
					return true;
				}
			}
			return false;
		}

    /// <summary> Durchsucht Inhaltsdatensignaturen nach den verwendeten
    /// Algorithmen. Es wird true zurckgegeben, wenn Referenzen der 
    /// XML-Signatur oder die Signatur selbst mit Algorithmen erzeugt
    /// wurden, die zu dem bergebenen Prfzeitpunkt als unsicher 
    /// eingestuft wurden. 
    /// </summary>
    /// <param name="date"> Prfzeitpunkt </param>
    /// <returns> true, wenn unsichere Algorithmen zur Signatur verwendet wurden,
    /// andernfalls false </returns>

    public bool hasWeakSignature(Role signer, DateTimeOffset date)
    {
	  if (checkContainsSigner(signer) == false)
		  throw new Exception("Message is not signed by given role object " + signer.id + ".");

      int keySize = MoreSupport.getKeySize(signer.SignatureCertificate);

      if (signer.SignatureCertificate.GetPublicKey() is Org.BouncyCastle.Crypto.Parameters.ECPublicKeyParameters)
      {
          if (keySize < 224)
          {
              log.info("Signature key (EC) has insufficient key size: " + keySize);
              return true;
          }
      }
      else
      {
          if (keySize < 1024)
          {
              log.info("Signature key has insufficient key size: " + keySize);
              return true;
          }

          if (date == null)
              date = Constants.ACTUAL_DATE;

          if (keySize < 2048 && date.CompareTo(Constants.OUT_DATE_KEYSIZE_1024) >= 0)
          {
              log.info("Signature key has insufficient key size: " + keySize);
              return true;
          }
      }

	  OSCISignature[] signatures = findSignatureObject(signer);

      for (int j = 0; j < signatures.Length; j++)
	  {
        if (Constants.OUT_DATES.ContainsKey(signatures[j].signatureAlgorithm)
            && date.CompareTo((DateTime)Constants.OUT_DATES[signatures[j].signatureAlgorithm]) >= 0)
              return true;

        System.Collections.IEnumerator digMeths = signatures[j].getDigestMethods().Values.GetEnumerator();
        while (digMeths.MoveNext())
        if (Constants.OUT_DATES.ContainsKey(digMeths.Current)
            && date.CompareTo((DateTime)Constants.OUT_DATES[digMeths.Current]) >= 0)
              return true;
      }

      return false;            
    }


		
		/// <summary>berprft alle Signaturen in dem ContentContainer. Die Hinweise zu
		/// transformierten Daten (s. checkSignature(Role)) sind zu beachten.
		/// </summary>
		/// <returns> true, sobald alle Prfungen positiv ausgefallen sind
		/// </returns>
		/// <seealso cref="checkSignature(Role)">
		/// </seealso>
		public virtual bool checkAllSignature() {
			Role[] signer = this.Signers;
			for (int i = 0; i < signer.Length; i++) {
				if (signer.Length == 0)
					throw new OSCISignatureException("no_signature");
				if (!checkSignature(signer[i])) {
					return false;
				}
			}
			
			return true;
		}

        /// <summary> Diese Methode signiert mit dem angegebenen Rollen-Objekt den kompletten
        /// ContentContainer. Es wird der im DialogHandler festgelegte Standardhashalgorithmus 
        /// verwendet (zur Berechnung der Hashwerte der Nachrichtenbestandteile, die in das 
        /// SignedInfo-Element eingetragen werden). Der Signaturalgorithmus wird von der 
        /// verwendeten Signer-Implementierung festgelegt. 
        /// </summary>
        /// <param name="signer">Role-Objekt mit dem Signer-Objekt
        /// </param>
        /// <exception cref="OSCIRoleException">wenn fr das bergebene Rollenobjekt kein
        /// Signer-Objekt gesetzt wurde oder diesem das erforderliche
        /// Signaturzertifikat fehlt.
        /// </exception>
        /// <exception cref="System.Exception">bei Schreib-/Lesefehlern
        /// </exception>
        public void sign(Role signer)
        {
	        sign(signer, DialogHandler.DigestAlgorithm);  
        }

        /// <summary> Diese Methode signiert mit dem angegebenen Rollen-Objekt den kompletten
		/// ContentContainer.
        /// <b>Hinweis:</b> Wenn der ContentContainer einen verschlsselten 
        /// ContentContainer (EncryptedDataOSCI) enthlt, so werden die (ebenfalls 
        /// verschlsselten) Attachments, die ggf. in diesem Container referenziert
        /// sind, nicht mit signiert. Der Grund ist, dass bei einer Signaturprfung
        /// Refenzen auf Attachments im verschlsselten ContentContainer nicht
        /// berprft werden knnen.	
        /// </summary>
		/// <param name="signer">Role-Objekt mit dem Signer-Objekt
		/// </param>
		/// <exception cref="OSCIRoleException">wenn fr das bergebene Rollenobjekt kein
		/// Signer-Objekt gesetzt wurde oder diesem das erforderliche
		/// Signaturzertifikat fehlt.
		/// </exception>
		/// <exception cref="System.Exception">bei Schreib-/Lesefehlern
		/// </exception>
        public virtual void sign(Role signer, String digestAlgorithm)
        {
            sign(signer, digestAlgorithm, null);
        }

        /// <summary> Diese Methode signiert mit dem angegebenen Rollen-Objekt den kompletten
	    /// ContentContainer. Der Signaturzeitpunkt kann im ISO-8601-Format bergeben 
        /// werden. 
        /// <b>Hinweis:</b> Wenn der ContentContainer einen verschlsselten 
        /// ContentContainer (EncryptedDataOSCI) enthlt, so werden die (ebenfalls 
        ///verschlsselten) Attachments, die ggf. in diesem Container referenziert
        ///sind, nicht mit signiert. Der Grund ist, dass bei einer Signaturprfung
        ///Refenzen auf Attachments im verschlsselten ContentContainer nicht
        /// berprft werden knnen.	
        /// <b>Die Verwendung dieser Methode fhrt zur Inkompatibilitt der
        /// erzeugten Nachrichten mit lteren Versionen der OSCI 1.2-Transportbibliothek 
        /// (vor 1.4). Sie sollte nur in Szenarien eingesetzt werden, in denen sichergestellt 
        /// ist, dass alle beteiligten Kommunikationspartner aktuelle Implementierungen 
        /// verwenden.</b>
        /// </summary>
        /// <param name="signer">Role-Objekt mit dem Signer-Objekt
        /// </param>
        /// <param name="digestAlgorithm">Zu verwendender Hashalgorithmus
        /// </param>
        /// <param name="time">Signaturzeitpunkt im ISO 8601-Format
        /// </param>
        /// <exception cref="OSCIRoleException">wenn fr das bergebene Rollenobjekt kein
		/// Signer-Objekt gesetzt wurde oder diesem das erforderliche
		/// Signaturzertifikat fehlt.
		/// </exception>
		/// <exception cref="System.Exception">bei Schreib-/Lesefehlern
		/// </exception>
        public virtual void sign(Role signer, String digestAlgorithm, String time)
        {
			if (!roles.Contains(signer)) 
            {
				roles.Add(signer);
			}
			if (!(signer is Author) && !(signer is Originator)) 
            {
				throw new OSCIRoleException("wrong_role_sign_cont");
			}
			int ret = 0;
			OSCISignature sig = new OSCISignature();
			if (signerList.Count == 0) 
            {
				log.debug("Anzahl der Contents" + contentList.Count);
				Content cnt;
				for (int i = 0; i < contentList.Count; i++) 
                {
					// Contents und EncryptedData mssen dieselben NS-Prefixes verwenden
					cnt = (Content) contentList[i];
					setNSPrefixes(cnt.soapNSPrefix, cnt.osciNSPrefix, cnt.dsNSPrefix, cnt.xencNSPrefix, cnt.xsiNSPrefix);
                    addSignatureReference(sig, cnt, digestAlgorithm);
 				}
                addAttachmentSigRefs(sig, this, digestAlgorithm);
                EncryptedDataOSCI enc;
				for (int i = 0; i < encryptedDataList.Count; i++) {
					enc = (EncryptedDataOSCI) encryptedDataList[i]; 
					setNSPrefixes(enc.soapNSPrefix, enc.osciNSPrefix, enc.dsNSPrefix, enc.xencNSPrefix, enc.xsiNSPrefix);
                    addSignatureReference(sig, enc, digestAlgorithm); 
				}
			}
			else 
            {
				sig.refs = ((OSCISignature) signerList[0]).refs;
                if (((OSCISignature)signerList[0]).signingTime != null)
                    sig.refs.RemoveAt(sig.refs.Count-1);
            }

            if (time != null)
                sig.addSignatureTime(time, this.id + "TS" + (signedSigPropNr++), digestAlgorithm);

			sig.sign(signer);
			signerList.Add(sig);
		}
/*		
		/// <summary> Speichert die Signierten Daten verschlsselt in den eingetragenen OSCIDataSource.
		/// Die Information sind im Object signatureData wiederzufinden.
		/// </summary>
		private void  createSignedData(OSCISignature sig) {
			for (int i = 0; i < contentList.Count; i++) {
                addSignatureReference(sig, ((Content)contentList[i]), digestValues);
				if (((Content) contentList[i]).Attachment != null) {
					addSignatureReference(sig, ((Content) contentList[i]).Attachment, digestValues);
				}
			}
			for (int i = 0; i < encryptedDataList.Count; i++) {
                addSignatureReference(sig, ((EncryptedDataOSCI)encryptedDataList[i]), digestValues);
			}
		}
*/ 
 /**
   * Fgt Attachment-Signaturreferenzen rekursiv hinzu.
   * @param sig
   * @param coco
   * @throws OSCIException
   * @throws IOException
   * @throws NoSuchAlgorithmException
   */
        private void addAttachmentSigRefs(OSCISignature sig, ContentContainer coco, String digestAlgorithm)
  {
	  Content cnt;
	  for (int i=0; i < coco.contentList.Count; i++)
	  {
		  cnt = (Content)coco.contentList[i];
		  if (cnt.ContentType == Content.ATTACHMENT_REFERENCE)
              addSignatureReference(sig, cnt.Attachment, digestAlgorithm);
          else if (cnt.ContentType == Content.CONTENT_CONTAINER)
              addAttachmentSigRefs(sig, cnt.ContentContainer, digestAlgorithm);			  
	  }
  }
		
		private void  addSignatureReference(OSCISignature sig, MessagePart mp, String algo)
{
			sig.refs.Add(new OSCISignatureReference(mp, algo));
		}
		
		/// <summary> Fgt dem ContentContainer ein Content-Objekt hinzu.
		/// </summary>
		/// <param name="content">das hinzuzufgende Content-Objekt
		/// </param>
		/// <seealso cref="Content">
		/// </seealso>
		public virtual void  addContent(Content content) {
			if ((this.signerList.Count > 0) && (stateOfObject == STATE_OF_OBJECT_CONSTRUCTION)) {
				throw new System.SystemException("Content-Container wurde bereits signiert.");
			}
			if (!contentList.Contains(content)) {
				contentList.Add(content);
				if (content.Attachment != null) {
					SupportClass.PutElement(attachments, content.Attachment.RefID, content.Attachment);
				}
				if (content.ContentContainer != null) {
					Role[] roles = content.ContentContainer.Roles;
					for (int i = 0; i < roles.Length; i++) {
						this.roles.Add(roles[i]);
					}
					Attachment[] atts = content.ContentContainer.Attachments;
					if (atts != null) {
						for (int i = 0; i < atts.Length; i++) {
							SupportClass.PutElement(this.attachments, atts[i].RefID, atts[i]);
						}
					}
				}
			}
		}
		
		/// <summary> Entfernt ein Content-Objekt aus dem ContentContainer.
		/// </summary>
		/// <param name="content">das zu entfernende Content-Objekt
		/// </param>
		/// <seealso cref="Content">
		/// </seealso>
		public virtual void  removeContent(Content content) {
			if ((this.signerList.Count > 0) && (stateOfObject == STATE_OF_OBJECT_CONSTRUCTION)) {
				throw new System.SystemException("Content-Container wurde bereits signiert.");
			}
			log.debug("start remove");
			contentList.Remove(content);
			if (content.Attachment != null) {
				SupportClass.HashtableRemove(this.attachments, content.Attachment.RefID);
			}
		}
		
		/// <summary> Fgt dem ContentContainer ein EncryptedData-Objekt hinzu.
		/// </summary>
		/// <param name="encryptedDataElement">das hinzuzufgende EncryptedData-Objekt
		/// </param>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		public virtual void  addEncryptedData(EncryptedDataOSCI encryptedDataElement) {
			if ((this.signerList.Count > 0) && (stateOfObject == STATE_OF_OBJECT_CONSTRUCTION)) {
				throw new System.SystemException("Content-Container wurde bereits signiert.");
			}
			if (!encryptedDataList.Contains(encryptedDataElement)) {
				log.debug("Encrypted-Data Element wird hinzugefgt.");
				encryptedDataList.Add(encryptedDataElement);
			}
			Attachment[] atts = encryptedDataElement.Attachments;
			for (int i = 0; i < atts.Length; i++) {
				if (!attachments.Contains(atts[i])) {
					SupportClass.PutElement(this.attachments, atts[i].RefID, atts[i]);
				}
			}
			Role[] roles = encryptedDataElement.Roles;
			for (int i = 0; i < roles.Length; i++) {
				this.roles.Add(roles[i]);
			}
			log.debug("Anzahl der neuen Roles: " + roles.Length);
		}
		
		/// <summary> Entfernt ein EncryptedData-Objekt aus dem ContentContainer.
		/// Der zweite Parameter gibt an, ob bei EncryptedData-Objekten, die ein
		/// verschlsseltes Attachment referenzieren, dieses aus dem ContentContainer
		/// entfernt wird.
		/// Dies ist von Bedeutung, wenn ein Attachment in mehreren
		/// EncryptedData-Objekten referenziert wird.
		/// </summary>
		/// <param name="encryptedDataElement">das zu entfernende EncryptedData-Objekt
		/// </param>
		/// <param name="removeAttachment"><b>true</b> -> Attachments, welche im EncryptedData-Objekt
		/// referenziert sind, werden ebenfalls aus dem ContentContainer entfernt.
		/// </param>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		public virtual void  removeEncryptedData(EncryptedDataOSCI encryptedDataElement, bool removeAttachment) {
			if ((this.signerList.Count > 0) && (stateOfObject == STATE_OF_OBJECT_CONSTRUCTION)) {
				throw new System.SystemException("Content-Container wurde bereits signiert.");
			}
			this.encryptedDataList.Remove(encryptedDataElement);
			if (removeAttachment) {
				System.Collections.IEnumerator enum_Renamed = encryptedDataElement.attachments.GetEnumerator();
				while (enum_Renamed.MoveNext()) {
					Attachment att = (Attachment) enum_Renamed.Current;
					if (attachments.ContainsKey(att)) {
						SupportClass.HashtableRemove(attachments, att);
					}
				}
			}
		}
		
		/// <summary> Interne Methode, wird von Anwendungen normalerweise nicht aufgerufen.
		/// </summary>
		/// <param name="out_Renamed">Stream, in den geschrieben werden soll.
		/// </param>
		/// <exception cref="System.Exception">IOException im Fehlerfall
		/// </exception>
		public override void writeXML(System.IO.Stream out_Renamed){
			writeXML(out_Renamed, true);
		}
		protected void  writeXML(System.IO.Stream out_Renamed, bool inner) {
			int i;
			MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix + ":ContentContainer",Constants.CHAR_ENCODING);
			if ((out_Renamed is SymCipherOutputStream) || (!(out_Renamed is System.Security.Cryptography.CryptoStream) && !inner))			//pr nov5
						out_Renamed.Write(ns,0,ns.Length);
			if ((RefID!=null) && (RefID.Length>0)) 
				MoreSupport.ToStream(out_Renamed," Id=\"" + this.RefID + "\"",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,">",Constants.CHAR_ENCODING);
  
			if (signerList.Count > 0) {
				for (i = 0; i < signerList.Count; i++) {
					((OSCISignature) signerList[i]).writeXML(out_Renamed);
				}
			}
			for (i = 0; i < contentList.Count; i++) {
				((Content) contentList[i]).writeXML(out_Renamed, inner);
			}
			for (i = 0; i < encryptedDataList.Count; i++) {
				((EncryptedDataOSCI) encryptedDataList[i]).writeXML(out_Renamed, inner);
			}
			MoreSupport.ToStream(out_Renamed,"</" + osciNSPrefix + ":ContentContainer>",Constants.CHAR_ENCODING);
		}
		
		/// <summary> Serialisiert den ContentContainer.
		/// </summary>
		/// <returns> Xml-String des ContentConatiners
		/// </returns>
		public override System.String ToString() {
			System.IO.MemoryStream out_Renamed = new System.IO.MemoryStream();
			writeXML(out_Renamed);
			
			try {
				return MoreSupport.NewString(out_Renamed.ToArray(), Constants.CHAR_ENCODING);
			} catch (System.IO.IOException ex) {
				log.error("Error", ex);
				return null;
			}
		}
		static ContentContainer() {
			log = LogFactory.getLog(typeof(ContentContainer));
		}
	}
}