namespace de.osci.osci12.messageparts {

	using System;
	using java.security.cert;
	using org.apache.commons.logging;
	using de.osci.osci12.common;
	
	/// <summary> <p>Diese Klasse reprsentiert das OSCI-Inspektion-Element. Hier werden
	/// Informationen fr die ausgewerteten Zertifikate der OSCI-Nachricht gehalten. </p>
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class Inspection:MessagePart {

		private const long serialVersionUID = 3258130267062415451L;
		internal String X509SubjectName = null;
		/// <summary>
		/// Liefert die Seriennummer des Zertifikats.
		/// </summary>
		/// <value>Zertifikatsnummer
		/// </value>
		virtual public System.String x509SubjectName {
			get {
				return X509SubjectName;
			}
		}

		virtual public bool IsOnlineChecked {
			get {
				return isOnlineChecked;
			}
		}

		internal System.String X509SerialNumber = null;

		/// <summary>
		/// Liefert die Seriennummer des Zertifikats.
		/// </summary>
		/// <value>Zertifikatsnummer
		/// </value>
		virtual public System.String x509SerialNumber {
			get {
				return X509SerialNumber;
			}
		}

		internal System.String X509IssuerName = null;

		/// <summary>
		/// Liefert den Ausstellernamen den Zertifikats.
		/// </summary>
		/// <value>Ausstellername
		/// </value>
		virtual public System.String x509IssuerName {
			get {
				return X509IssuerName;
			}
		}

		internal Timestamp TimeStamp;

		/// <summary>
		/// Liefert den Zeitstempel der Prfung.
		/// </summary>
		/// <value>Zeitstempel
		/// </value>
		virtual public Timestamp timeStamp {
			get {
				return TimeStamp;
			}	
		}

		internal bool onlineResult;

		/// <summary>
		/// Liefert das Ergebnis der online-Prfung.
		/// </summary>
		/// <value>true -> ok, false -> revoked. Bei nicht durchgefhrter
		/// online-Prfung wird revoked zurckgegeben.
		/// </value>
		virtual public bool OnlineResult {
			get {
				return onlineResult;
			}
		}

		internal System.String[] OnlineCheckNames;

		/// <summary>
		/// Liefert den Namen des online-Prfverfahrens.
		/// </summary>
		/// <value>online-Prfverfahren (OCSP/CRL/LDAP)
		/// </value>
		virtual public System.String[] onlineCheckNames {
			get {
				return OnlineCheckNames;
			}	
		}

		// Name des Checkverfahrens
		internal System.String[] OnlineChecks;

		/// <summary>
		/// Liefert den Inhalt des Prfverfahrens-Eintrags. Dies ist bei OCSP-Prfung
		/// der Base-64-codierte OCSP-Request und bei CRL-Prfung das Datum der CRL-Liste.
		/// </summary>
		/// <value>Inhalt des Eintrags
		/// </value>
		virtual public System.String[] onlineChecks {
			get {
				return OnlineChecks;
			}
		}

		internal string MathResult;

		/// <summary>
		/// Liefert das Ergebnis der mathematischen Zertifikatsprfung.
		/// </summary>
		/// <value>true -> ok, false -> corrupted
		/// </value>
		virtual public string mathResult {
			get {
				return MathResult;
			}
		}

		internal string OfflineResult;

		/// <summary>
		/// Liefert das Ergebnis der offline-Prfung.
		/// </summary>
		/// <value>Ergebnis: true -> valid, false -> invalid
		/// </value>
		virtual public string offlineResult {
			get {
				return OfflineResult;
			}
		}

		internal System.String CertType;

		/// <summary>
		/// Liefert den Typ des Zertifikats.
		/// </summary>
		/// <value>Typ (advanced/qualified/unknown)
		/// </value>
		virtual public System.String certType {
			get {
				return CertType;
			}
		}

		/// <summary>
		/// Liefert die ID-Nummer.
		/// </summary>
		/// <value>ID-Nummer</value>
		virtual public int IdNr {
			get {
				return idNr;
			}	
		}

		new private static Log log;
		public static System.String CERT_TYPE_ADVANCED = "advanced";
		public static System.String CERT_TYPE_QUALIFIED = "qualified";
		public static System.String CERT_TYPE_UNKNOWN = "unknown";
		public static String CERT_TYPE_ACCREDITED= "accredited";
		private static int idNr = - 1;
		private X509Certificate cert;
		internal bool isOnlineChecked = false;
		
		internal Inspection() {
		}
		
		/// <summary> Dieser Konstruktur wird nur bei Offlineprfung benutzt.
		/// </summary>
		/// <param name="cert">Zertifikat fr das diese Prfergebnisse gelten.
		/// </param>
		/// <param name="certType">Art des Zertifikates (advanced, qualified oder unknown).
		/// </param>
		/// <param name="timeStamp">Timestamp Element zu diesem Eintrag.
		/// </param>
		/// <param name="mathResult">Ergbnis der Prfung der Zertifikatssignatur OK oder corrupted (true or false).
		/// </param>
		/// <param name="offlineResult">bei Offlineprfung Ergebnis der Offline-Gltigkeitsprfung des Zertifikates valid oder invalid (true, false)).
		/// </param>
		public Inspection(X509Certificate cert, System.String certType, Timestamp timeStamp, string mathResult, string offlineResult) {
			X509IssuerName =  cert.GetIssuerName();
			X509SerialNumber = cert.GetSerialNumber().ToString(); 
			X509SubjectName = cert.GetName();
			this.TimeStamp = timeStamp;
			this.MathResult = mathResult;
			this.OfflineResult = offlineResult;
			this.CertType = certType;
			this.isOnlineChecked = false;
		}
		
		/// <summary>
		/// </summary>
		/// <param name="cert">Zertifikat fr das diese Prfergebnisse gelten
		/// </param>
		/// <param name="certType">Art des Zertifikates (advanced, qualified oder unknown)
		/// </param>
		/// <param name="timeStamp">Timestamp Element zu diesem Eintrag
		/// </param>
		/// <param name="mathResult">Ergbnis der Prfung der Zertifikatssignatur OK oder corrupted (true or false)
		/// </param>
		/// <param name="offlineResult">bei Offlineprfung Ergebnis der Offline-Gltigkeitsprfung des Zertifikates valid oder invalid (tur, false))
		/// </param>
		/// <param name="onlineResult">Ergebnis der Online Prfung OK oder revoked
		/// </param>
		/// <param name="onlineCheckName">Art der Onlineprfung OCSP, CRL oder LDAP
		/// </param>
		/// <param name="onlineCheck">Ergebnis der CRL oder OCSP. Entweder base64 oder dateTime Objekt
		/// </param>
		public Inspection(X509Certificate cert, System.String certType, Timestamp timeStamp, string mathResult, string offlineResult, bool onlineResult, System.String[] onlineCheckNames, System.String[] onlineChecks) {
			X509IssuerName =  cert.GetIssuerName();
			X509SerialNumber = cert.GetSerialNumber().ToString(); 
			X509SubjectName = cert.GetName();
			this.TimeStamp = timeStamp;
			this.MathResult = mathResult;
			this.OfflineResult = offlineResult;
			this.onlineResult = onlineResult;
			this.OnlineCheckNames = onlineCheckNames;
			this.OnlineChecks = onlineChecks;
			this.isOnlineChecked = true;
		}
		
		public override void writeXML(System.IO.Stream out_Renamed) {
			MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix + ":Inspection>",Constants.CHAR_ENCODING);
			timeStamp.writeXML(out_Renamed);
			MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix + ":X509SubjectName>" + ProcessCardBundle.encode(x509SubjectName) + "</" + osciNSPrefix + ":X509SubjectName>",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix + ":X509IssuerName>" + ProcessCardBundle.encode(x509IssuerName) + "</" + osciNSPrefix + ":X509IssuerName>",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix + ":X509SerialNumber>" + x509SerialNumber + "</" + osciNSPrefix + ":X509SerialNumber>",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix + ":CertType Type=\"" + certType + "\"></" + osciNSPrefix + ":CertType><" + osciNSPrefix + ":MathResult Result=\"",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,this.mathResult.ToString().ToLower() ,Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,"\"></" + osciNSPrefix + ":MathResult><" + osciNSPrefix + ":OfflineResult Result=\""+offlineResult.ToString().ToLower()+"\"></" + osciNSPrefix + ":OfflineResult>",Constants.CHAR_ENCODING);
			if (isOnlineChecked) {
				String res = "revoked";
				if (onlineResult) res = "ok";
				MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix + ":OnlineResult Result=\""+res + "\">"
					,Constants.CHAR_ENCODING);
				for (int i = 0; i < onlineCheckNames.Length; i++) {
					MoreSupport.ToStream(out_Renamed,
						"<" + osciNSPrefix + ":" + onlineCheckNames[i] + ">"
						+ onlineChecks[i] + "</" + osciNSPrefix + ":" + onlineCheckNames[i] + ">"
						,Constants.CHAR_ENCODING);
				}
				MoreSupport.ToStream(out_Renamed,"</" + osciNSPrefix + ":OnlineResult>",Constants.CHAR_ENCODING);
			}
			MoreSupport.ToStream(out_Renamed,"</" + osciNSPrefix + ":Inspection>",Constants.CHAR_ENCODING);
		}
		
		static Inspection() {
			log = LogFactory.getLog(typeof(Inspection));
		}
	}
}