namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.common;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	
	/// <summary><p><H4>Dialogende-Nachrichtenobjekt</H4></p>
	/// Diese Klasse dient der Beendigung eines expliziten Dialogs.
	/// Clients erhalten als Antwort auf diese Nachricht vom Intermedir ein
	/// Nachrichtenobjekt, welches eine Rckmeldung ber den Erfolg der Operation
	/// enthlt. Diese Rckmeldung wird von der send()-Methode ausgewertet, so
	/// da eine Behandlung der Anwortnachricht selbst nicht erforderlich ist.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p>
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.common.DialogHandler">
	/// </seealso>
	public class ExitDialog:OSCIRequest {

		new private static Log log;
		
		/// <summary> Legt ein Nachrichtenobjekt zur Dialogbeendigung an.
		/// </summary>
		/// <param name="dh">DialogHandler-Objekt, welches den expliziten Dialog reprsentiert.
		/// </param>
		/// <seealso cref="de.osci.osci12.common.DialogHandler">
		/// </seealso>
		/// 
		public ExitDialog(DialogHandler dh):base(dh) {
			messageType = EXIT_DIALOG; 
			originator = ((Originator) dh.Client);
			
			dialogHandler.Controlblock.Response = dialogHandler.prevChallenge;
			dialogHandler.Controlblock.Challenge=de.osci.helper.Tools.createRandom(10);
			dialogHandler.Controlblock.SequenceNumber = dialogHandler.Controlblock.SequenceNumber + 1;
		}
		
		internal ExitDialog():base() {
			messageType = EXIT_DIALOG;
		}
		
		/// <summary> Versendet die Nachricht und liefert die Antwortnachricht zurck.
		/// Diese Methode wirft eine Exception, wenn beim Aufbau oder Versand der
		/// Nachricht ein Fehler auftritt. Fehlermeldungen vom Intermedir mssen
		/// dem Feedback der Antwortnachricht entnommen werden.
		/// </summary>
		/// <returns> Antwortnachricht-Objekt
		/// </returns>
		// --> Storestreams:
		public virtual ResponseToExitDialog send() {
			return (ResponseToExitDialog) transmit(null, null);
		}
		// <-- Storestreams:

		public override void  compose() 
		{
			base.compose();
			if (dialogHandler.Controlblock.Challenge == null) 
			{
				throw new System.SystemException("Kein Challenge-Wert in DialogHandler eingestellt.");
			}
			if (dialogHandler.Controlblock.Response == null) {
				throw new System.SystemException("Kein Response-Wert in DialogHandler eingestellt.");
			}
			if (dialogHandler.Controlblock.ConversationID == null) {
				throw new System.SystemException("Keine Conversation-ID in DialogHandler eingestellt.");
			}
			if (dialogHandler.Controlblock.SequenceNumber == - 1) {
				throw new System.SystemException("Kein Squenznummer in DialogHandler eingestellt.");
			}
			body = new Body("<" + osciNSPrefix + ":exitDialog></" + osciNSPrefix + ":exitDialog>");
			
			stateOfMsg |= STATE_COMPOSED;
		}
		
		public override void  writeXML(System.IO.Stream out_Renamed) {
			base.writeXML(out_Renamed);
			// ClientSignatur
			if (signatureHeader != null) {
				signatureHeader.writeXML(out_Renamed);
			}
			// DesiredLanguage
			desiredLanguagesH.writeXML(out_Renamed);
			
			if (nonIntermediaryCertificatesH != null) {
				nonIntermediaryCertificatesH.writeXML(out_Renamed);
			}
			completeMessage(out_Renamed);
		}
		
		static ExitDialog() {
			log = LogFactory.getLog(typeof(ExitDialog));
		}
	}
}