namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.dotcommon; // Storestreams
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	
	/// <summary><p><H4>Laufzettelabholauftrag</H4></p>
	/// Mit dieser Klasse werden Nachrichtenobjekte fr Laufzettelabholauftrge
	/// angelegt. Clients knnen hiermit Laufzettel eingegangener Nachrichten vom Intermedir
	/// abrufen.  Als Antwort auf diese Nachricht erhalten sie vom Intermedir
	/// ein ResponseToFetchProcessCard-Nachrichtenobjekt, welches eine Rckmeldung ber den
	/// Erfolg der Operation und ggf. die gewnschten Laufzettel enthlt.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p>
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.messagetypes.ResponseToFetchProcessCard">
	/// </seealso>
	public class FetchProcessCard:OSCIRequest {

		new private static Log log;
		internal System.String selectionRule;
		private int selectionMode = - 1;
		internal int roleForSelection = -1;
		private long quantityLimit = - 1;
		internal bool selectNoReceptionOnly = false; 

		/// <summary> Ruft die Auswahlregel fr die abzuholende Nachricht ab, oder setzt diese. Der Inhalt des
		/// bergebenen Strings hngt vom gewhlten Auswahlmodus ab und kann
		/// entweder in einer Base64-codierten Message-Id oder einem Datum bestehen.
        /// Mehrere Message-Ids knnen durch "&amp;" getrennt aneinandergereiht werden.
        /// Das Format eines Datums mu dem XML-Schema nach
        /// <a href="http://www.w3.org/TR/xmlschema-2/#dateTime">
        /// http://www.w3.org/TR/xmlschema-2/#dateTime</a> (ISO 8601-Format) entsprechen.
		/// </summary>
		/// <value>Message-Id oder Datum
		/// </value>
		/// <seealso cref="SelectionMode()">
		/// </seealso>
		virtual public System.String SelectionRule {
			get {
				return selectionRule;
			}	
			set {
				this.selectionRule = value;
			}
		}

		// <summary> BEZOGEN AUF getSelectionMode: Liefert den gesetzten Auswahlmodus.
		// </summary>
		// <returns> den Auswahlmodus SELECT_BY_MESSAGE_ID, SELECT_BY_DATE_OF_RECEPTION,
		// SELECT_BY_RECENT_MODIFICATION oder NO_SELECTION_RULE
		// </returns>
		// <seealso cref="SelectionRule(String)">
		// </seealso>
		// <seealso cref="SelectionMode()">
		// </seealso>
		
		/// <summary> Ruft den Auswahlmodus ab, oder setzt diesen. Mgliche Werte sind
		/// SELECT_BY_MESSAGE_ID, SELECT_BY_DATE_OF_RECEPTION,
		/// SELECT_BY_RECENT_MODIFICATION oder NO_SELECTION_RULE (default).
		/// </summary>
		/// <value>Auswahlmodus
		/// </value>
		/// <seealso cref="SelectionRule()">
		/// </seealso>
		virtual public int SelectionMode {
			get {
				return selectionMode;
			}
			set {
				if ((value != SELECT_BY_MESSAGE_ID) && (value != SELECT_BY_DATE_OF_RECEPTION) && (value != SELECT_BY_RECENT_MODIFICATION) && (value != NO_SELECTION_RULE)) {
					throw new System.ArgumentException("Ungltiger Auswahlmodus. " + "Es sind nur die Werte SELECT_BY_MESSAGE_ID, SELECT_BY_DATE_OF_RECEPTION, " + "SELECT_BY_RECENT_MODIFICATION und NO_SELECTION_RULE erlaubt.");
				}
				this.selectionMode = value;
			}
		}

		/// <summary> Ruft die maximale Anzahl zurckzugebender Laufzettel ab, oder setzt diese.
		/// </summary>
		/// <value> gesetzte maximale Anzahl.
		/// </value>
		virtual public long QuantityLimit {
			get {
				return quantityLimit;
			}
			set {
				if (value <= 0) {
					throw new System.ArgumentException("Limit fr die Zahl der Laufzettel mu grer 0 sein.");
				}
				this.quantityLimit = value;
			}
		}
		
		/// <summary> Legt ein Nachrichtenobjekt fr einen Laufzettelabholauftrag an.
		/// </summary>
		/// <param name="dh">DialogHandler-Objekt des Dialogs, innerhalb dessen die Nachricht
		/// versendet werden soll.
		/// </param>
		/// <seealso cref="de.osci.osci12.common.DialogHandler">
		/// </seealso>
		public FetchProcessCard(DialogHandler dh):base(dh) {
			originator = ((Originator) dh.Client);
			messageType = FETCH_PROCESS_CARD;
			dialogHandler.Controlblock.Response = dialogHandler.prevChallenge;
			dialogHandler.Controlblock.Challenge = Tools.createRandom(10);
			dialogHandler.Controlblock.SequenceNumber = dialogHandler.Controlblock.SequenceNumber + 1;
		}
		
		internal FetchProcessCard():base() {
			messageType = FETCH_PROCESS_CARD;
		}
		
		/// <summary> Versendet die Nachricht und liefert die Antwortnachricht zurck.
		/// Diese Methode wirft eine Exception, wenn beim Aufbau oder Versand der
		/// Nachricht ein Fehler auftritt. Fehlermeldungen vom Intermedir mssen
		/// dem Feedback der Antwortnachricht entnommen werden.
		/// </summary>
		/// <returns> das Antwortnachricht-Objekt
		/// </returns>
		// --> Storestreams:
		public virtual ResponseToFetchProcessCard send() {
			return (ResponseToFetchProcessCard) transmit(null, null);
		}

		public virtual ResponseToFetchProcessCard send(System.IO.Stream storeOutput, System.IO.Stream storeInput) {
			return (ResponseToFetchProcessCard) transmit(storeOutput, storeInput);
		}
		// <-- Storestreams:

		/// <summary>Liefert den Identifier fr das Auswahlkriterium, ob nur Laufzettel von
		/// Nachrichten zurckgegeben werden sollen, die an den oder vom Absender des
		/// Laufzettelabholauftrags geschickt wurden.
		/// Mgliche Werte sind
		/// <ul><li>SELECT_ORIGINATOR - fr Nachrichten, die vom Absender dieses
		/// Auftrags gesendet wurden.</li><li>SELECT_ADDRESSEE - fr Nachrichten, die
		/// an den Absender dieses Auftrags gesendet wurden.</li><li>SELECT_ALL -
		/// fr alle Nachrichten (default).</li></ul> <b>Diese Einstellung ist nur in
		/// den Selection-Modes SELECT_BY_DATE_OF_RECEPTION und
		/// SELECT_BY_RECENT_MODIFICATION wirksam.</b>
		/// </summary>
		/// <value>int Code for the Selection Role
		/// </value>
		/// <seealso href="SELECT_ORIGINATOR">
		/// </seealso>
		/// <seealso href="SELECT_ADDRESSEE">
		/// </seealso>
		/// <seealso href="SELECT_ALL">
		/// </seealso>
		/// <seealso cref="SelectionRule">
		/// </seealso>
		/// <seealso cref="SelectionMode">
		/// </seealso>
		public int RoleForSelection {
			get
			{
			  return roleForSelection;
			}
			set{
				if ((value != SELECT_ORIGINATOR) && (value != SELECT_ADDRESSEE) && (value != SELECT_ALL))
					throw new dothelpers.IllegalArgumentException(DialogHandler.text.getString("invalid_firstargument")+value);
				this.roleForSelection = value;
			}
		}

		/// <summary> Legt fest, ob nur Laufzettel von Nachrichten zurckgegeben werden sollen,
		/// fr die keine Empfangsbesttigung vom Empfnger vorliegt.</summary>
		/// <value>noReceptionOnly true -> es werden nur Laufzettel fr nicht zugestellte
		/// Nachrichten zurckgegeben. false -> es werden alle Laufzettel zurckgegeben
		/// (default). <b>Diese Einstellung ist nur in den Selection-Modes
		/// SELECT_BY_DATE_OF_RECEPTION und SELECT_BY_RECENT_MODIFICATION wirksam.</b>
		/// </value>
		/// <seealso cref="SelectionRule">
		/// </seealso>
		/// <seealso cref="SelectionMode">
		/// </seealso>
		/// <seealso cref="RoleForSelection">
		/// </seealso>
		public bool SelectNoReceptionOnly {set{
			this.selectNoReceptionOnly = value;}
		}

		/// <summary> Legt fest, ob nur Laufzettel von Nachrichten zurckgegeben werden sollen,
		/// fr die keine Empfangsbesttigung vom Empfnger vorliegt.
		/// Gibt an, ob Liefert den Identifier fr das Auswahlkriterium, ob nur Laufzettel von
		/// Nachrichten zurckgegeben werden sollen, die an den oder vom Absender des
		/// Laufzettelabholauftrags geschickt wurden.
		/// </summary>
		/// <seealso cref="RoleForSelection">
		/// </seealso>
		public bool isSelectNoReception() {
			return selectNoReceptionOnly;
		}

		public override void  compose() 
		{
			base.compose();
			if (dialogHandler.Controlblock.Challenge == null) 
			{
				throw new System.SystemException("Kein Challenge-Wert in DialogHandler eingestellt.");
			}
			
			if (dialogHandler.Controlblock.Response == null) {
				throw new System.SystemException("Kein Response-Wert in DialogHandler eingestellt.");
			}
			
			if (dialogHandler.Controlblock.ConversationID == null) {
				throw new System.SystemException("Keine Conversation-ID in DialogHandler eingestellt.");
			}
			
			if (dialogHandler.Controlblock.SequenceNumber == - 1) {
				throw new System.SystemException("Kein Squenznummer in DialogHandler eingestellt.");
			}
			
			//System.String selection = "";
			String selectionAttributes = ""; 
    
			if (selectNoReceptionOnly) selectionAttributes += " NoReception=\"true\""; 
    
			if (roleForSelection == SELECT_ADDRESSEE) 
				selectionAttributes += " Role=\"Addressee\""; 
			else if (roleForSelection == SELECT_ORIGINATOR) 
				selectionAttributes += " Role=\"Originator\""; 
    
			System.Text.StringBuilder selection = new System.Text.StringBuilder("<"); 
			selection.Append(osciNSPrefix); 
			selection.Append(":SelectionRule>"); 
			if (selectionMode == SELECT_BY_MESSAGE_ID) 
			{
				String[] msgIds = selectionRule.Split('&');
				for (int i = 0; i < msgIds.Length; i++) {
					selection.Append("<"+osciNSPrefix+":MessageId>");
					selection.Append(de.osci.helper.Base64.encode(SupportClass.ToByteArray(msgIds[i], Constants.CHAR_ENCODING)));
					selection.Append("</"+osciNSPrefix+":MessageId>");
				}
				selection.Append("</");
			} else if (selectionMode == SELECT_BY_DATE_OF_RECEPTION) {
				selection.Append("<"+osciNSPrefix+":ReceptionOfDelivery" + selectionAttributes + ">"  + selectionRule + "</"+osciNSPrefix+":ReceptionOfDelivery></");
			} else if (selectionMode == SELECT_BY_RECENT_MODIFICATION) {
				selection.Append( "<"+osciNSPrefix+":RecentModification" + selectionAttributes + ">" + selectionRule + "</"+osciNSPrefix+":RecentModification></");
			}
			else 
				selection.Remove(0, selection.Length); 
    
			if (selection.Length > 0) 
			{ 
				selection.Append(osciNSPrefix); 
				selection.Append(":SelectionRule>"); 
			} 
			if (quantityLimit >= 0) 
			{
				selection.Append("<"+osciNSPrefix+":Quantity Limit=\"" + quantityLimit + "\"></"+osciNSPrefix+":Quantity>");
			}
			body = new Body("<"+  osciNSPrefix +":fetchProcessCard>" + selection +
				"</"+  osciNSPrefix +":fetchProcessCard>");
			stateOfMsg |= STATE_COMPOSED;
			body.setNSPrefixes(this);
		}
		
		public override void  writeXML(System.IO.Stream out_Renamed) {
			base.writeXML(out_Renamed);
			
			// ClientSignatur
			if (signatureHeader != null) {
				signatureHeader.writeXML(out_Renamed);
			}

			// DesiredLanguage
			desiredLanguagesH.writeXML(out_Renamed);
			
			if (nonIntermediaryCertificatesH != null) {
				nonIntermediaryCertificatesH.writeXML(out_Renamed);
			}
			completeMessage(out_Renamed);
		}
		
		static FetchProcessCard() {
			log = LogFactory.getLog(typeof(FetchProcessCard));
		}
	}
}