namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.dotcommon; // Storestreams
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	using de.osci.osci12.soapheader;

	/// <summary><p><H4>Weiterleitungsauftrag</H4></p>
	/// Mit dieser Klasse werden Nachrichtenobjekte fr Weiterleitungsauftrge
	/// angelegt. Clients erhalten als Antwort auf diese Nachricht vom Intermedir
	/// ein ResponseToForwardDelivery-Nachrichtenobjekt, welches eine Rckmeldung ber den
	/// Erfolg der Operation (getFeedback()) und ggf. den Laufzettel zur gesendeten
	/// Nachricht enthlt.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p>
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.messagetypes.ResponseToForwardDelivery">
	/// </seealso>
	public class ForwardDelivery:OSCIRequest,ContentPackageI {

		new private static Log log;
		private System.String subject;

		/// <summary> Ruft die gewnschte Qualitt des Zeitstempels ab, 
		/// mit dem der Intermedir den Eingang des Auftrags im Laufzettel protokolliert, 
		/// oder setzt diese. 
		/// </summary>
		/// <value><b>true</b>: kryptographischer Zeitstempel von einem
		/// akkreditierten Zeitstempeldienst.
		/// <p><b>false</b>: Einfacher Zeitstempel, default
		/// (lokale Rechnerzeit des Intermedrs).</p>
		/// </value>
		/// <seealso cref="QualityOfTimeStampCreation()">
		/// </seealso>
		virtual public bool QualityOfTimeStampCreation {
			get {
				return qualityOfTimestampTypeCreation.QualityCryptographic;
			}
			set {
				qualityOfTimestampTypeCreation = new QualityOfTimestampH(false, value);
			}
		}

		/// <summary>Ruft die gewnschte Qualitt des Zeitstempels ab, 
		/// mit dem der Intermedir den Empfang der Annahmeantwort im Laufzettel protokolliert,
		/// oder setzt diese.
		/// </summary>
		/// <value><b>true</b>: kryptographischer Zeitstempel von einem
		/// akkreditierten Zeitstempeldienst. 
		/// <p><b>false</b>: Einfacher Zeitstempel, default
		/// (lokale Rechnerzeit des Intermedrs).</p>
		/// </value>
		/// <seealso cref="QualityOfTimeStampReception()">
		/// </seealso>
		virtual public bool QualityOfTimeStampReception {
			get {
				return qualityOfTimestampTypeReception.QualityCryptographic;
			}
			set {
				qualityOfTimestampTypeReception = new QualityOfTimestampH(true, value);
			}
		}

		/// <summary>Ruft die Adresse des Empfngers der Inhaltsdaten ab, oder setzt diese.
		/// </summary>
		/// <value>Adresse
		/// </value>
		virtual public System.String ContentReceiver {
			get {
				return uriReceiver.ToString();
			}
			set {
				this.uriReceiver = value;
			}
		}

		/// <summary>Ruft den Betreff-Eintrag im Laufzettel ab, oder setzt diesen.
		/// </summary>
		/// <value> Betreff der Zustellung
		/// </value>
		virtual public System.String Subject {
			get {
				return subject;
			}
			set {
				this.subject = value;
			}
		}

		/// <summary> Liefert die in die Nachricht eingestellten (unverschlsselten) Inhaltsdaten als ContentContainer-Objekte.
		/// </summary>
		/// <value> enthaltene ContentContainer mit Inhaltsdaten.
		/// </value>
		/// <seealso cref="de.osci.osci12.messageparts.ContentContainer">
		/// </seealso>
		override public ContentContainer[] ContentContainer {
			get {
				return base.ContentContainer;
			}
		}

		/// <summary> Liefert die in die Nachricht eingestellten verschlsselten Inhaltsdaten
		/// als EncryptedData-Objekte.
		/// </summary>
		/// <value> enthaltene EncryptedData-Objekt mit verschlsselten Inhaltsdaten.
		/// </value>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		override public EncryptedDataOSCI[] EncryptedData {
			get {
				return base.EncryptedData;
			}
		}

		/// <summary> Ruft die Message-ID der Nachricht ab, oder setzt diese.
		/// </summary>
		virtual public System.String MessageId {
			get {
				return messageId;
			}
			set {
				this.messageId = value;
			}
		}
		
		/// <summary> Legt ein Nachrichtenobjekt fr einen Weiterleitungsauftrag an.
		/// </summary>
		/// <param name="dh">DialogHandler-Objekt des Dialogs, innerhalb dessen die Nachricht
		/// versendet werden soll.
		/// </param>
		/// <seealso cref="de.osci.osci12.common.DialogHandler">
		/// </seealso>
		public ForwardDelivery(DialogHandler dh, Addressee addressee, System.String uriReceiver, String messageId):base(dh) {
			if (!dialogHandler.explicitDialog)	dialogHandler.resetControlBlock();

			dialogHandler.Controlblock.Response = dialogHandler.prevChallenge;
			dialogHandler.Controlblock.Challenge = Tools.createRandom(10);
			dialogHandler.Controlblock.SequenceNumber = dialogHandler.Controlblock.SequenceNumber + 1;
			
			messageType = FORWARD_DELIVERY;
			originator = ((Originator) dh.Client);
			this.addressee = addressee;
			this.uriReceiver = uriReceiver;
			this.messageId = messageId;

			QualityOfTimeStampCreation=false;
			QualityOfTimeStampReception=false;
		}
		
		/// <summary> Legt ein Nachrichtenobjekt fr einen Weiterleitungsauftrag an.
		/// Dieser Konstruktor wird nur von dem Parser benutzt.
		/// </summary>
		internal ForwardDelivery(DialogHandler dh):base() {
			messageType = FORWARD_DELIVERY;
			DialogHandler = dh;
		}
		
		/// <summary> Versendet die Nachricht und liefert die Antwortnachricht zurck.
		/// Diese Methode wirft eine Exception, wenn beim Aufbau oder Versand der
		/// Nachricht ein Fehler auftritt. Fehlermeldungen vom Intermedir mssen
		/// dem Feedback der Antwortnachricht entnommen werden.
		/// </summary>
		/// <returns> das Antwortnachricht-Objekt
		/// </returns>
		// --> Storestreams:		
		public virtual ResponseToForwardDelivery send() {
			return (ResponseToForwardDelivery) transmit(null, null);
		}
		
		public virtual ResponseToForwardDelivery send(System.IO.Stream storeOutput, System.IO.Stream storeInput) {
			return (ResponseToForwardDelivery) transmit(storeOutput, storeInput);
		}
		// <-- Storestreams:

		/// <summary> Fgt der Nachricht einen Inhaltsdatencontainer hinzu.
        /// <b>Hinweis: </b>ContentContainer-Objekte mit Attachments, die durch Entschlsselung 
        /// von EncryptedDataOSCI-Objekten gewonnen wurden, knnen hier nicht ohne weiteres
        /// hinzugefgt werden. In diesem Fall mssen die Attachments zunchst ausgelesen und 
        /// die Daten in neu angelegten Attachment-Objekten wieder hinzugefgt werden.
		/// </summary>
		/// <param name="container">Inhaltsdatencontainer
		/// </param>
        /// <seealso cref="de.osci.osci12.messageparts.ContentContainer"/>
        /// <seealso cref="de.osci.osci12.messageparts.EncryptedDataOSCI"/>
		public override void  addContentContainer(ContentContainer container) {
			base.addContentContainer(container);
		}
		
		/// <summary> Entfernt einen Inhaltsdatencontainer aus der Nachricht.
		/// </summary>
		/// <param name="container">Inhaltsdatencontainer
		/// </param>
		/// <seealso cref="addContentContainer">
		/// </seealso>
		public override void  removeContentContainer(ContentContainer container) {
			base.removeContentContainer(container);
		}
		
		/// <summary> Fgt der Nachricht einen EncryptedData-Eintrag mit verschlsselten
		/// Inhaltsdaten hinzu.
		/// </summary>
		/// <param name="encData">verschlsselte Daten
		/// </param>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		public override void  addEncryptedData(EncryptedDataOSCI encData) {
			base.addEncryptedData(encData);
		}
		
		/// <summary> Entfernt einen EncryptedData-Eintrag mit verschlsselten Daten
		/// aus der Nachricht.
		/// </summary>
		/// <param name="encData">verschlsselte Daten
		/// </param>
		/// <seealso cref="addEncryptedData">
		/// </seealso>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		public override void  removeEncryptedData(EncryptedDataOSCI encData) {
			base.removeEncryptedData(encData);
		}
		
		public override void  compose() 
		{
			base.compose();
			if (dialogHandler.Controlblock.Challenge == null) 
			{
				throw new System.SystemException("Kein Challenge-Wert in DialogHandler eingestellt.");
			}
			if (dialogHandler.Controlblock.SequenceNumber == - 1) {
				throw new System.SystemException("Kein Squenznummer in DialogHandler eingestellt.");
			}

			importAllCertificates();
			if (nonIntermediaryCertificatesH == null) {
				createNonIntermediaryCertificatesH();
			}
			if (uriReceiver == null) {
				throw new System.SystemException("Kein URI fr ContentReceiver eingestellt.");
			}
			if (messageId == null) {
				throw new System.SystemException("Keine MessageId eingestellt.");
			}
			
			System.String head = "<" + osciNSPrefix + ":ContentReceiver URI=\"" + uriReceiver + 
				"\"></" + osciNSPrefix + ":ContentReceiver><" + osciNSPrefix + ":MessageId>" 
				+ de.osci.helper.Base64.encode(
				messageId,Constants.CHAR_ENCODING) + "</" + osciNSPrefix + ":MessageId>";
			
			if (subject != null) {
				head += ("<" + osciNSPrefix + ":Subject>" + subject + "</" + osciNSPrefix + ":Subject>");
			}
			osciH = new OsciH("forwardDelivery", head);
			body = new Body(ContentContainer, EncryptedData);
			stateOfMsg |= STATE_COMPOSED;
		}
		
		public override void writeXML(System.IO.Stream out_Renamed) {
			log.debug("CreateMsg child");
			base.writeXML(out_Renamed);
			
			// ClientSignatur
			if (signatureHeader != null) {
				signatureHeader.writeXML(out_Renamed);	
			}

			// DesiredLanguage
			desiredLanguagesH.writeXML(out_Renamed);
			qualityOfTimestampTypeCreation.writeXML(out_Renamed);
			qualityOfTimestampTypeReception.writeXML(out_Renamed);
			osciH.writeXML(out_Renamed);
			
			if (nonIntermediaryCertificatesH != null) {
				nonIntermediaryCertificatesH.writeXML(out_Renamed);
			}
			completeMessage(out_Renamed);
		}
		
		static ForwardDelivery() {
			log = LogFactory.getLog(typeof(ForwardDelivery));
		}
	}
}