namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;
	using de.osci.osci12.common;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	using de.osci.osci12.soapheader;
    using java.security.cert;

	/// <summary> Diese Klasse ist die Superklasse aller OSCI-Nachrichten-Objekte.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public abstract class OSCIMessage 
	{
//		private static ISO8601DateTimeFormat isodate = new ISO8601DateTimeFormat();
        private String digestAlgorithm;

		private void  InitBlock() 
		{
			otherAutors = new System.Collections.Hashtable();
			otherReaders = new System.Collections.Hashtable();
			contentContainer = new System.Collections.Hashtable();
			encryptedData = new System.Collections.Hashtable();
			attachments = new System.Collections.Hashtable();
			xmlns = new System.Collections.Hashtable();
            digestAlgorithm = DialogHandler.DigestAlgorithm;
        }

		/// <summary>Liefert Status der Nachricht (signiert/unsigniert)
		/// </summary>
		/// <returns>signierte Nachricht -> true</returns>
		/// <seealso cref="de.osci.osci12.common.DialogHandler.CheckSignatures()">
		/// </seealso>
		public bool isSigned() {
			return (signatureHeader != null);
		}

        /// <summary>Durchsucht die Nachrichtensignatur nach den verwendeten
        /// Algorithmen. Es wird true zurckgegeben, wenn Referenzen der 
        /// XML-Signatur oder die Signatur selbst mit Algorithmen erzeugt
        /// wurden, die zu dem bergebenen Prfzeitpunkt als unsicher 
        /// eingestuft wurden.
        /// </summary>
        /// <param name="date"> Prfzeitpunkt </param>
        /// <returns>true, wenn unsichere Algorithmen zur Signatur verwendet wurden,
        /// andernfalls false</returns>
        public bool hasWeakSignature(DateTimeOffset date)
        {
            if (signatureHeader == null)
    	        throw new Exception("Message is not signed.");
    
            if (signerCert == null)
    	        throw new Exception("Role object referenced in signature not found.");

            int keySize = MoreSupport.getKeySize(signerCert);

            if (signerCert.GetPublicKey() is Org.BouncyCastle.Crypto.Parameters.ECPublicKeyParameters)
            {
                if (keySize < 224)
                {
                    log.info("Signature key (EC) has insufficient key size: " + keySize);
                    return true;
                }
            }
            else
            {
                if (keySize < 1024)
                {
                    log.info("Signature key has insufficient key size: " + keySize);
                    return true;
                }

                if (date == null)
                    date = Constants.ACTUAL_DATE;

                if (keySize < 2048 && date.CompareTo(Constants.OUT_DATE_KEYSIZE_1024) >= 0)
                {
                    log.info("Signature key has insufficient key size: " + keySize);
                    return true;
                }
            }
  
            if (Constants.OUT_DATES.ContainsKey(signatureHeader.signatureAlgorithm)
                && date.CompareTo((DateTime)Constants.OUT_DATES[signatureHeader.signatureAlgorithm]) >= 0)
                return true;
            System.Collections.IEnumerator digMeths = signatureHeader.getDigestMethods().Values.GetEnumerator();
            while (digMeths.MoveNext())
                if (Constants.OUT_DATES.ContainsKey(digMeths.Current)
                    && date.CompareTo((DateTime)Constants.OUT_DATES[digMeths.Current]) >= 0)
                        return true;
  
            return false;            
        }


		virtual public ContentContainer[] ContentContainer {
			get {
				ContentContainer [] cc=new ContentContainer [contentContainer.Count];
				contentContainer.Values.CopyTo(cc,0);
				return cc;
			}
		}

		/// <summary> Verschlsselt die Nachricht auf Nachrichtenebene
		/// </summary>
		/// An dieser Stelle wird fr die Generierung des XML-Files ein "value" ausgelesen, ein "return" nicht!
		/// <value> Verschlsselte Transportdaten
		/// </value>
		virtual public EncryptedDataOSCI[] EncryptedData {
			get {
				EncryptedDataOSCI [] ed=new EncryptedDataOSCI [encryptedData.Count];
				encryptedData.Values.CopyTo(ed,0);
				return ed;
			}	
		}

		virtual public Attachment[] Attachments {
			get {
				Attachment [] at=new Attachment [attachments.Count];
				attachments.Values.CopyTo(at,0);
				return at;
			}	
		}

		internal DialogHandler DialogHandler;

		/// <summary> Liefert den DialogHandler des Nachrichtenobjektes.
		/// </summary>
		/// <value> Der DialogHandler
		/// </value>
		virtual public DialogHandler dialogHandler {
			get {
				return DialogHandler;
			}			
		}

		/// <summary>  Liefert den Identifier fr den Nachrichtentyp (ACCEPT_DELIVERY,
		/// EXIT_DIALOG...). Fr Instanzen von OSCIMessage ist der Typ
		/// TYPE_UNDEFINED.
		/// </summary>
		/// <value> Der Messagetyp-Identifier
		/// </value>
		virtual public int MessageType {
			get {
				return messageType;
			}	
		}

		internal Addressee addressee;
		
		/// <summary> Liefert das Addressee-Rollenobjekt des Nachrichtenobjektes.
		/// </summary>
		/// <value> Der Addressee
		/// </value>
		virtual public Addressee Addressee {
			get {
				return addressee;
			}			
		}

		internal Originator originator;
	
		/// <summary> Liefert das Originator-Rollenobjekt des Nachrichtenobjektes.
		/// </summary>
		/// <value> Der Originator
		/// </value>
		virtual public Originator Originator {
			get {
				return originator;
			}	
		}
		
		/// <summary> Liefert alle Author-Rollenobjekte, die fr die Signatur von Inhaltsdaten
		/// in der Nachricht verwendet wurden oder die mit der Methode addRole(Role)
		/// der Nachricht hinzugefgt wurden.
		/// </summary>
		/// <value> Array von Author-Objekten
		/// </value>
		/// <seealso cref="addRole(Role)">
		/// </seealso>
		/// <seealso cref="de.osci.osci12.messageparts.ContentContainer.sign(Role)">
		/// </seealso>
		virtual public Author[] OtherAuthors {
			get {
				Author [] au=new Author [otherAutors.Count];
				otherAutors.Values.CopyTo(au,0);
				return au;
			}	
		}

		/// <summary> Liefert alle Reader-Rollenobjekte, fr die Inhaltsdaten
		/// in der Nachricht verschlsselt wurden oder die mit der Methode addRole(Role)
		/// der Nachricht hinzugefgt wurden.
		/// </summary>
		/// <value> Array von Reader-Objekten
		/// </value>
		/// <seealso cref="addRole(Role)">
		/// </seealso>
		/// <seealso cref="de.osci.osci12.messageparts.EncryptedDataOSCI.encrypt(Role)">
		/// </seealso>
		virtual public de.osci.osci12.roles.Reader[] OtherReaders {
			get {
				Reader [] re=new Reader [otherReaders.Count];
				otherReaders.Values.CopyTo(re,0);
				return re;
			}			
		}

		private static Log log;
		public System.Collections.Hashtable hashableMsgPart;

		/// <summary>Konstante, die einen undefinierten Nachrichtentyp anzeigt. 
		/// </summary>
		public const int TYPE_UNDEFINED = 0;

		/// <summary>Konstante, die einen Annahmeauftrag anzeigt. 
		/// </summary>
		public const int ACCEPT_DELIVERY = 0x01;

		/// <summary>Konstante, die einen Dialogendeauftrag anzeigt. 
		/// </summary>
		public const int EXIT_DIALOG = 0x02;

		/// <summary>Konstante, die einen Abbholauftrag anzeigt. 
		/// </summary>
		public const int FETCH_DELIVERY = 0x03;

		/// <summary>Konstante, die einen Laufzettelabholauftrag anzeigt. 
		/// </summary>
		public const int FETCH_PROCESS_CARD = 0x04;

		/// <summary>Konstante, die einen Weiterleitungsauftrag anzeigt. 
		/// </summary>
		public const int FORWARD_DELIVERY = 0x05;

		/// <summary>Konstante, die einen MessageId-Anforderungsauftrag anzeigt. 
		/// </summary>
		public const int GET_MESSAGE_ID = 0x06;

		/// <summary>Konstante, die einen Dialoginitialisierungsauftrag anzeigt. 
		/// </summary>
		public const int INIT_DIALOG = 0x07;

		/// <summary>Konstante, die einen Abwicklungsauftrag anzeigt. 
		/// </summary>
		public const int MEDIATE_DELIVERY = 0x08;

		/// <summary>Konstante, die einen Bearbeitungsauftrag anzeigt. 
		/// </summary>
		public const int PROCESS_DELIVERY = 0x09;

		/// <summary>Konstante, die einen Zustellungsauftrag anzeigt. 
		/// </summary>
		public const int STORE_DELIVERY = 0x0A;

		/// <summary>Konstante, die eine Annahmeantwort anzeigt. 
		/// </summary>
		public const int RESPONSE_TO_ACCEPT_DELIVERY = 0x10;

		/// <summary>Konstante, die eine Dialogendeantwort anzeigt. 
		/// </summary>
		public const int RESPONSE_TO_EXIT_DIALOG = 0x20;

		/// <summary>Konstante, die eine Abbholantwort anzeigt. 
		/// </summary>
		public const int RESPONSE_TO_FETCH_DELIVERY = 0x30;

		/// <summary>Konstante, die eine Laufzettelabholantwort anzeigt. 
		/// </summary>
		public const int RESPONSE_TO_FETCH_PROCESS_CARD = 0x40;

		/// <summary>Konstante, die eine Weiterleitungsantwort anzeigt. 
		/// </summary>
		public const int RESPONSE_TO_FORWARD_DELIVERY = 0x50;

		/// <summary>Konstante, die ein MessageId-Anforderungsantwort anzeigt. 
		/// </summary>
		public const int RESPONSE_TO_GET_MESSAGE_ID = 0x60;

		/// <summary>Konstante, die eine Dialoginitialisierungsantwort anzeigt. 
		/// </summary>
		public const int RESPONSE_TO_INIT_DIALOG = 0x70;

		/// <summary>Konstante, die eine Abwicklungsantwort anzeigt. 
		/// </summary>
		public const int RESPONSE_TO_MEDIATE_DELIVERY = 0x80;

		/// <summary>Konstante, die eine Bearbeitungsantwort anzeigt. 
		/// </summary>
		public const int RESPONSE_TO_PROCESS_DELIVERY = 0x90;

		/// <summary>Konstante, die eine Zustellungsantwort anzeigt. 
		/// </summary>
		public const int RESPONSE_TO_STORE_DELIVERY = 0xA0;
		/// <summary>Konstante, die eine Rckmeldung auf Nachrichtenebene (SOAP-Fault) anzeigt.
		/// </summary>
		public const int SOAP_FAULT_MESSAGE = 0xB0;

		/// <summary>Konstante, die eine verschlsselte SOAP-Nachricht anzeigt. 
		/// </summary>
		public const int SOAP_MESSAGE_ENCRYPTED = 0x100;

		private de.osci.osci12.extinterfaces.OSCIDataSource encryptedMsg = null;
		private EncryptedDataOSCI transportEncryptedData = null;

		/// <summary>Kein Auswahlmodus gesetzt.
		/// </summary>
		public const int NO_SELECTION_RULE = - 1;

		/// <summary>Auswahlmodus fr Nachrichten/Laufzettel nach Message-Id. 
		/// </summary>
		public const int SELECT_BY_MESSAGE_ID = 0;

		/// <summary>Auswahlmodus fr Nachrichten/Laufzettel nach Empfangsdatum. 
		/// </summary>
		public const int SELECT_BY_DATE_OF_RECEPTION = 1;

		/// <summary>Auswahlmodus fr Nachrichten/Laufzettel nach Datum der letzen Modifikation. 
		/// </summary>
		public const int SELECT_BY_RECENT_MODIFICATION = 2;

		/// <summary>Auswahlmodus fr alle Laufzettel (default). </summary>
		public const int SELECT_ALL = -1;
		/// <summary> Auswahlmodus fr Laufzettel von Nachrichten an den Absender eines Laufzettelabholauftrags.</summary>
		public const int SELECT_ADDRESSEE = 0;
		/// <summary>Auswahlmodus fr Laufzettel von Nachrichten vom Absender eines Laufzettelabholauftrags.</summary>
		public const int SELECT_ORIGINATOR = 1;
		protected internal System.String messageId;

		/// <summary>Content-Id der Nachricht 
		/// </summary>
		public static System.String contentID = "osci@message";

		internal System.Collections.Hashtable otherAutors;
		internal System.Collections.Hashtable otherReaders;

		/// <summary>Enthlt die ContentContainer der Nachricht 
		/// </summary>
		public System.Collections.Hashtable contentContainer;

		/// <summary>Enthlt die EncryptedData-Objekte der Nachricht 
		/// </summary>
		public System.Collections.Hashtable encryptedData;

		/// <summary>Enthlt die Attachment-Objekte der Nachricht 
		/// </summary>
		public System.Collections.Hashtable attachments;

		protected internal XMLReader xmlReader = null;
		protected internal System.String currentElement;
		protected internal System.Collections.Hashtable xmlns;
		protected internal int messageType = 0;
		
		/// <summary>Signatureintrag im Header (Client- oder Suppliersignatur). 
		/// </summary>
		public OSCISignature signatureHeader;

		/// <summary>DesiredLanguages-Header 
		/// </summary>
		public DesiredLanguagesH desiredLanguagesH; // Kommt zwar nur in Requests vor, kann so aber in sign() abgefragt werden.

		/// <summary>QualityOfTimestampCreation-Headereintrag 
		/// </summary>
		public QualityOfTimestampH qualityOfTimestampTypeCreation;

		/// <summary>QualityOfTimestampReception-Headereintrag
		/// </summary>
		public QualityOfTimestampH qualityOfTimestampTypeReception;

		public OsciH osciH;

		/// <summary>NonIntermediaryCertificates-Headereintrag
		/// </summary>
		public NonIntermediaryCertificatesH nonIntermediaryCertificatesH;

		/// <summary>IntermediaryCertificates-Headereintrag
		/// </summary>
		public IntermediaryCertificatesH intermediaryCertificatesH;
		public ControlBlockH controlBlock;

		internal Body body = null;
        internal X509Certificate signerCert;
        
		
		// hier sollten schon einmal die States definiert werden
		internal int stateOfMsg = 0;
		protected internal const int STATE_COMPOSED = 0x01;
		protected internal const int STATE_SIGNED = 0x02;
		protected internal const int STATE_PARSED = 0x04;
		public String soapNSPrefix = "soap";
		public String 	osciNSPrefix = "osci";
		public String dsNSPrefix = "ds";
		public String xencNSPrefix = "xenc";
		public String xsiNSPrefix = "xsi";	
		private DefaultHandler parentHandler;
		public String ns = " xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\" xmlns:osci=\"http://www.osci.de/2002/04/osci\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xenc=\"http://www.w3.org/2001/04/xmlenc#\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"";
		
		public String boundary_string;
		protected bool base64 = true;

		protected internal System.Collections.ArrayList customHeaders =new System.Collections.ArrayList();
		internal string bodyId = "body";		
		/// <summary> Konstruktor for the OSCIMessage object
		/// </summary>
		protected internal OSCIMessage() {
			InitBlock();
		}
		
		/// <summary> Konstruktor fr ein OSCIMessage-Objekt
		/// </summary>
		/// <param name="dh">
		/// </param>
		protected internal OSCIMessage(DialogHandler dh) {
			InitBlock();
			DialogHandler = dh;
		}
		
		/// <todo>Checken, ob dies noch ntig ist, wenn die Rollen erst beim Aufbau der Nachricht zusammengesucht werden.</todo>
		/// <summary> <p>Diese Methode ermglicht es Anwendungen, zustzliche Zertifikate in den
		/// NonIntermediaryCertificates-Header einzustellen, die dann vom Intermedir mit
		/// geprft werden. Die Zertifikate werden in Form von Reader- oder Author-Objekten
		/// bergeben, die die entsprechenden Zertifikate enthalten mssen.</p>
		/// Die Methoden ContentContainer.sign(Role) und EncryptedData.encrypt(Role)
		/// fgen die bergebenen Rollenobjekte der Nachricht automatisch hinzu, so da
		/// diese Methode in der Regel nicht bentigt wird.
		/// </summary>
		/// <param name="role">Hinzuzfgendes Reader- oder Author-Objekt
		/// <exception cref="System.Exception">Throws IllegalArgumentException, when ...</exception> 
		/// </param>
		/// <seealso cref="de.osci.osci12.messageparts.ContentContainer.sign(Role)">
		/// </seealso>
		/// <seealso cref="de.osci.osci12.messageparts.EncryptedDataOSCI.encrypt(Role)">
		/// </seealso>
		public virtual void  addRole(Role role) {		
			if (role is Originator) {
				originator = (Originator) role;
			} else if (role is Addressee) {
				addressee = (Addressee) role;
			} else {
				if ((role is Author) && (!otherAutors.ContainsValue(role))) { { 
			  if (otherAutors.ContainsKey(role.id)) { 
				  // Role.id wird nur noch intern verwendet 
				  role.cipherRefId = role.CipherCertificateId; 
				  role.signatureRefId = role.SignatureCertificateId;
				  role.id += '0'; 
			  } 
					SupportClass.PutElement(otherAutors, role.id, role);
		  } 
       
				}
				else if ((role is de.osci.osci12.roles.Reader) && (!otherReaders.ContainsValue(role))) {
					if (otherReaders.ContainsKey(role.id)) {
						// Role.id wird nur noch intern verwendet
						role.cipherRefId = role.CipherCertificateId;
						role.id += '0';
					}
					SupportClass.PutElement(otherReaders, role.id, role);
				}
			}
		}

		public virtual void  addContentContainer(ContentContainer container) {
			log.debug("ContentConatiner wird hinzugefgt.");
			// bernehmen der Role Objekte
			SupportClass.PutElement(contentContainer, container.RefID, container);
			
			Role[] roles = container.Roles;
			
			for (int i = 0; i < roles.Length; i++) {
				log.debug("Ein weiteres Role-Objekt wird der Nachricht hinzugefgt.");
				
				if (roles[i] is Originator) 
                {
					if ((this.originator != null) && (this.originator != roles[i])) 
                    {
						throw new OSCIRoleException("Es wurde versucht einen weiteren anderen Originator der Nachricht zuzufgen.");
					}
					
					log.debug("Eine Originator wird hinzugefgt.");
					this.originator = (Originator) roles[i];
				} 
                else if (roles[i] is Addressee) 
                {
					if ((this.addressee != null) && (this.addressee != roles[i])) 
                    {
						throw new OSCIRoleException("Es wurde versucht einen weiteren anderen Addressee der Nachricht zuzufgen.");
                    }					
					this.addressee = (Addressee) roles[i];
					log.debug("Eine Addressee wird hinzugefgt.");
					break;
				}
				
				if (roles[i] is Author) 
                {
                    otherAutors[roles[i].id] = roles[i];
				}
				
				if (roles[i] is de.osci.osci12.roles.Reader) 
                {
					otherReaders[roles[i].id]=roles[i];
				}
			}
			
			// bernehme der Attachments
			Attachment[] atts = container.Attachments;
			if (atts != null && atts.Length > 0) {
				for (int i = 0; i < atts.Length; i++) {
					this.addAttachment(atts[i]);
				}
			}
		}
		
		public virtual void  removeContentContainer(ContentContainer container) {
			SupportClass.HashtableRemove(contentContainer, container.RefID);
		}

        /**
          * Durchsucht <b>die unverschlsselten</b> Inhaltsdaten nach dem ContentContainer
          * mit der bergebenen RefID.     
          * @param refID zu suchende RefID
          * @return den zugehrigen ContentContainer oder null, wenn die Referenz
          * nicht gefunden wurde.
          */
        public ContentContainer getContentContainerByRefID(String refID)
  {
	  ContentContainer[] containers = ContentContainer;
	  MessagePart ret = null;
	  for (int i = 0; i < containers.Length; i++) 
	  {
		  if ((ret = searchMessagePart(containers[i], refID)) != null)
		  {
			  if (ret is ContentContainer)
				  return (ContentContainer)ret;
			  else
			  {
				  log.info("RefID " + refID + " does not refer to a ContentContainer.");
				  return null;
			  }
		  }			  
	  }
	  log.info("RefID " + refID + " not found.");
	  return null;
  }

        /**
         * Durchsucht <b>die unverschlsselten</b> ContentContainer nach dem Content
         * mit der bergebenen RefID.     
         * @param refID zu suchende RefID
         * @return den zugehrigen Content oder null, wenn die Referenz
         * nicht gefunden wurde.
         */
        public Content getContentByRefID(String refID)
  {
	  ContentContainer[] containers = ContentContainer;
	  MessagePart ret = null;
	  for (int i = 0; i < containers.Length; i++) 
	  {
		  if ((ret = searchMessagePart(containers[i], refID)) != null)
		  {
			  if (ret is Content)
				  return (Content)ret;
			  else
			  {
				  log.info("RefID " + refID + " does not refer to a Content.");
				  return null;
			  }
		  }			  
	  }
	  log.info("RefID " + refID + " not found.");
	  return null;
  }
  private MessagePart searchMessagePart(ContentContainer cnt, String refID)
  {
      if (cnt.RefID.Equals(refID))
          return cnt;

      Content[] contents = cnt.Contents;
      for (int i = 0; i < contents.Length; i++)
      {
          if (contents[i].RefID.Equals(refID))
              return contents[i];
          if (contents[i].ContentType == Content.CONTENT_CONTAINER)
          {
              MessagePart mp = searchMessagePart(contents[i].ContentContainer, refID);
              if (mp != null)
                  return mp;
          }
      }
      return null;
  }
  public virtual void addEncryptedData(EncryptedDataOSCI encData)
        {
			if ((encData.RefID == null) || encData.RefID.Equals("")) {
				throw new System.NullReferenceException("EncryptedDataOSCI-Objekt besitzt keine oder leere RefID.");
			}
			
			SupportClass.PutElement(encryptedData, encData.RefID, encData);
			
			// bernehmen der Role Objekte
			Role[] roles = encData.Roles;
			
			for (int i = 0; i < roles.Length; i++) {
				if (roles[i] is Originator) {
					if ((this.originator != null) && (this.originator != roles[i])) {
						throw new OSCIRoleException("Es wurde versucht einen weiteren anderen Originator der Nachricht zuzufgen.");
					}
					
					this.originator = (Originator) roles[i];
				}
				else if (roles[i] is Addressee) {
					if ((this.addressee != null) && (this.addressee != roles[i])) {
						throw new OSCIRoleException("Es wurde versucht einen weiteren anderen Addressee der Nachricht zuzufgen.");
					}
					
					this.addressee = (Addressee) roles[i];
				}
				
				if (roles[i] is Author) 
				{
					otherAutors[roles[i].id]=roles[i];
				}
				
				if (roles[i] is de.osci.osci12.roles.Reader) 
				{
					otherReaders[roles[i].id]=roles[i];
				}
			}
			
			// bernehmen der Attachments
			Attachment[] atts = encData.Attachments;
			
			for (int i = 0; i < atts.Length; i++) {
				this.addAttachment(atts[i]);
			}
		}
		
		public virtual void  removeEncryptedData(EncryptedDataOSCI encData) {
			SupportClass.HashtableRemove(encryptedData, encData.RefID);
		}
		
		/// <summary> Interne Methode. Attachments werden automatisch einer Nachricht hinzugefgt,
		/// wenn der referenzierende Content eingestellt wird. Diese Methode sollte
		/// von Anwendungen daher normalerweise nicht aufgerufen werden.
		/// </summary>
		/// <param name="attachment">Attachment
		/// </param>
		/// <seealso cref="de.osci.osci12.messageparts.ContentContainer.addContent(Content)">
		/// </seealso>
		internal virtual void  addAttachment(Attachment attachment) {
			SupportClass.PutElement(attachments, attachment.RefID, attachment);
			if (boundary_string != null)
				attachment.boundary_string = boundary_string;
		}
		
		/// <summary> Interne Methode. Attachments werden automatisch aus einer Nachricht entfernt,
		/// wenn der referenzierende Content entfernt wird. Diese Methode sollte
		/// von Anwendungen daher normalerweise nicht aufgerufen werden.
		/// </summary>
		/// <param name="attachment">Attachment
		/// </param>
		/// <seealso cref="de.osci.osci12.messageparts.ContentContainer.removeContent(Content)">
		/// </seealso>
		internal virtual void  removeAttachment(Attachment attachment) {
			SupportClass.HashtableRemove(attachments, attachment.RefID);
		}
				
  private bool cmpRoles(Role org, Role next)
  {
    java.security.cert.X509Certificate orgCert = null;
    java.security.cert.X509Certificate nextCert = null;

	try
	{
		if (org.hasCipherCertificate())
		{
			if (next.hasCipherCertificate() && !org.CipherCertificate.Equals(next.CipherCertificate))
				return false;
		}
		else if (next.hasCipherCertificate())
			org.CipherCertificate = next.CipherCertificate;

		if (org.hasSignatureCertificate())
		{
			if (next.hasSignatureCertificate() && !org.SignatureCertificate.Equals(next.SignatureCertificate))
				return false;
		}
		else if (next.hasSignatureCertificate())
			org.SignatureCertificate = next.SignatureCertificate;

		return true;
    }
    catch (OSCIRoleException ex)
    {
      return false;
    }
  }

  private void importCertificates(Role[] roles)
  {
    for (int i = 0; i < roles.Length; i++)
    {
      if ((roles[i] is Originator) && (originator != null) &&
              (!cmpRoles(originator, roles[i])))
        throw new OSCIRoleException(DialogHandler.text.getString("incompatible_role_error") +
                                    " Originator");
      else if ((roles[i] is Addressee) && (addressee != null) &&
                   !cmpRoles(addressee, roles[i]))
        throw new OSCIRoleException(DialogHandler.text.getString("incompatible_role_error") +
                                    " Addressee");
      else if (roles[i] is de.osci.osci12.roles.Reader)
			otherReaders[roles[i].id] = roles[i];
      else if (roles[i] is Author)
			otherAutors[roles[i].id] = roles[i];
	 }
  }

  protected void collectCertificatesFromContentContainer(ContentContainer coco)
  {
    importCertificates(coco.Roles);

    Content[] cnt = coco.Contents;

    for (int i = 0; i < cnt.Length; i++)
    {
      if (cnt[i].ContentType == Content.CONTENT_CONTAINER)
        collectCertificatesFromContentContainer(cnt[i].ContentContainer);
    }

    EncryptedDataOSCI[] enc = coco.EncryptedData;

    for (int i = 0; i < enc.Length; i++)
    {
      importCertificates(enc[i].Roles);
    }
  }

  protected void importAllCertificates()
  {
    System.Collections.IEnumerator it = contentContainer.Values.GetEnumerator();

    while (it.MoveNext())
      collectCertificatesFromContentContainer((ContentContainer) it.Current);

    it = encryptedData.Values.GetEnumerator();

    while (it.MoveNext())
      importCertificates(((EncryptedDataOSCI) it.Current).Roles);
  }

		public virtual void  compose() 
		{
			if (boundary_string == null)
				boundary_string = DialogHandler.boundary + "_" + de.osci.helper.Tools.createRandom(24);

			System.Collections.IEnumerator e = attachments.Values.GetEnumerator();
			while (e.MoveNext())
			{
				((Attachment)e.Current).boundary_string = boundary_string;
			}
		}
		
		protected internal virtual void  sign(Role role) 
		{
            if (!(role is Intermed) && !(role is Originator) && !(role is Addressee)) 
			{
				throw new OSCIRoleException("Falsches Rollenobjekt (" + role.GetType() + ").\nuere Signaturen knnen nur von Originator-, Intermediaer- oder Addressee-Objekte angebracht werden.", null);
			}
			
			if ((stateOfMsg & STATE_COMPOSED) == 0) 
				compose();

			System.String name = "";
			
			if (this is OSCIRequest) {
				name = "<"+osciNSPrefix+":ClientSignature Id=\"clientsignature\" " + soapNSPrefix + ":actor=\"http://schemas.xmlsoap.org/soap/actor/next\" " + soapNSPrefix + ":mustUnderstand=\"1\">";
			}
			else {
				name = "<"+osciNSPrefix+":SupplierSignature Id=\"suppliersignature\" " + soapNSPrefix + ":actor=\"http://schemas.xmlsoap.org/soap/actor/next\" " + soapNSPrefix + ":mustUnderstand=\"1\">";
			}
			signatureHeader = MessagePartsEntry.osciSignature(name);
			
			de.osci.osci12.common.DialogHandler generatedAux = this.DialogHandler;
			signatureHeader.refs.Add(MessagePartsEntry.osciSignatureReference(dialogHandler.Controlblock, digestAlgorithm));
			
			if (desiredLanguagesH != null) {
                signatureHeader.refs.Add(MessagePartsEntry.osciSignatureReference(desiredLanguagesH, digestAlgorithm));
			}
			if (qualityOfTimestampTypeCreation != null) {
                signatureHeader.refs.Add(MessagePartsEntry.osciSignatureReference(qualityOfTimestampTypeCreation, digestAlgorithm));
			}
			if (qualityOfTimestampTypeReception != null) {
                signatureHeader.refs.Add(MessagePartsEntry.osciSignatureReference(qualityOfTimestampTypeReception, digestAlgorithm));
			}
			if (osciH != null) {
                signatureHeader.refs.Add(MessagePartsEntry.osciSignatureReference(osciH, digestAlgorithm));
			}

			for (int i = 0; i < customHeaders.Count; i++)
                signatureHeader.refs.Add(MessagePartsEntry.osciSignatureReference(((CustomHeader)customHeaders[i]), digestAlgorithm));

			if (attachments.Count > 0) {
				System.Collections.IEnumerator atts = attachments.GetEnumerator();
				Attachment att;
				byte[] tmp;
				
				while (atts.MoveNext()) {
					// Workaround fr das Problem, dass Attchments im Header verschlsselt signiert werden.
					att = (Attachment) ((System.Collections.DictionaryEntry)(((System.Object)((atts.Current))))).Value;
					tmp = null;
					if (att.hasDigestValue(digestAlgorithm))
                    {
                        tmp = att.getDigestValue(digestAlgorithm);
                        att.digestValues.Remove(digestAlgorithm);
                    }
                    att.digestValues.Add(digestAlgorithm, att.getEncryptedDigestValue(digestAlgorithm));
					signatureHeader.refs.Add(MessagePartsEntry.osciSignatureReference(att, digestAlgorithm));
                    att.digestValues.Remove(digestAlgorithm);
                    if (tmp != null)
                    {
                        att.digestValues.Add(digestAlgorithm, tmp);
                    }
				}
			}
			
			if (role is Intermed) {
				if (intermediaryCertificatesH == null) {
					intermediaryCertificatesH = new IntermediaryCertificatesH();
				}
				intermediaryCertificatesH.SignatureCertificateIntermediary = (Intermed) role;
			}
			else {
				if (nonIntermediaryCertificatesH == null) {
					nonIntermediaryCertificatesH = new NonIntermediaryCertificatesH();
				}
				if (role is Originator) {
					nonIntermediaryCertificatesH.SignatureCertificateOriginator = (Originator) role;
				}
				else {
					nonIntermediaryCertificatesH.SignatureCertificateAddressee = (Addressee) role;
				}
			}
			
			if (intermediaryCertificatesH != null) {
				signatureHeader.refs.Add(MessagePartsEntry.osciSignatureReference(intermediaryCertificatesH, digestAlgorithm));
			}
			if (nonIntermediaryCertificatesH != null) {
				signatureHeader.refs.Add(MessagePartsEntry.osciSignatureReference(nonIntermediaryCertificatesH, digestAlgorithm));
			}
			signatureHeader.refs.Add(MessagePartsEntry.osciSignatureReference(body, digestAlgorithm));   
			signatureHeader.sign(role);
		}
		
		// Diese Methode fgt alle in den Role-Objekte auffindbaren Zertifikate zum non-Intermed Header hinzu.
		public virtual void  createNonIntermediaryCertificatesH() {
			if ((stateOfMsg & STATE_PARSED) != 0) {
				return ;
			}
			
			bool empty = true;
			
			if (nonIntermediaryCertificatesH == null) {
				nonIntermediaryCertificatesH = new NonIntermediaryCertificatesH();
			}
			else {
				empty = false;
			}
			if (originator != null) 
			{
				if (originator.hasSignatureCertificate())
				{
					nonIntermediaryCertificatesH.SignatureCertificateOriginator = originator;
					empty = false;
				}

				if (originator.hasCipherCertificate())
				{
					nonIntermediaryCertificatesH.CipherCertificateOriginator = originator;
					empty = false;
				}
			}
			
			System.Collections.ArrayList v;
			
			if ((v = searchCertificates(false, OtherAuthors)).Count > 0) {
				empty = false;
			}
			Author [] au=new Author [v.Count];
			for (int i =0; i < au.Length; i++)
				au[i] = (Author)v[i];
			nonIntermediaryCertificatesH.CipherCertificatesOtherAuthors = au;
			
			if (addressee != null) 
			{
				if (addressee.hasCipherCertificate())
				{
					nonIntermediaryCertificatesH.CipherCertificateAddressee = addressee;
					empty = false;
				}
			}
			
			if ((v = searchCertificates(false, OtherReaders)).Count > 0) {
				empty = false;
			}
			Reader [] rd=new Reader[v.Count];
			for (int i =0; i < rd.Length; i++)
				rd[i] = (Reader)v[i];
			nonIntermediaryCertificatesH.CipherCertificatesOtherReaders = rd;
			
			if ((v = searchCertificates(true, OtherAuthors)).Count > 0) {
				empty = false;
			}
			au=new Author [v.Count];
			for (int i =0; i < au.Length; i++)
				au[i] = (Author)v[i];
			nonIntermediaryCertificatesH.SignatureCertificatesOtherAuthors = au;
			
			if (empty) {
				nonIntermediaryCertificatesH = null;
			}
		}
		
		private System.Collections.ArrayList searchCertificates(bool signature, Role[] roles) {
			System.Collections.ArrayList v = new System.Collections.ArrayList();
			java.security.cert.X509Certificate r;
			for (int i = 0; i < roles.Length; i++) 
			{
				if ((signature && roles[i].hasSignatureCertificate())
					|| (!signature && roles[i].hasCipherCertificate()))
						v.Add(roles[i]);
			}
			return v;
		}
			
		/// <summary> Diese Methode liefert ein Role Objekt passend zu der bergebenen RefID
		/// </summary>
		/// <param name="uri">URI</param>
		/// <returns> Role Objekt oder Null
		/// </returns>
		public Role getRoleForRefID(String uri) {
			Role role;
			System.Collections.IDictionaryEnumerator enu=otherAutors.GetEnumerator();
			foreach (object en in otherAutors.Keys)
			{
				role = (Role) otherAutors[en];
				if (checkRole(uri, role))return role;
			}
            foreach (object en in otherReaders.Keys)
			{
				role = (Role) otherReaders[en];
				if (checkRole(uri, role))return role;
			}

            if (checkRole(uri, Originator)) return Originator;
            if (checkRole(uri, Addressee)) return Addressee;
            if (checkRole(uri, dialogHandler.Supplier)) return dialogHandler.Supplier;
            if (checkRole(uri, dialogHandler.Client)) return dialogHandler.Client;
			return null;
		}
		private bool checkRole(String uri, Role role) {
			try 
            {
				if (uri.Equals(role.CipherCertificateId))
					return true;
				else if (uri.Equals(role.SignatureCertificateId))
					return true;
			}
			catch (NullReferenceException ex) 
            {
			}

			return false;
		}

		internal virtual OSCIMessage parse(System.IO.Stream in_Renamed) {
			parseMimePart(in_Renamed);
			
			return saxIt(in_Renamed);
		}
		
		/// <summary>  - Speicherung der Nachricht - create Table of HashIt (zur berprfung
		/// der Signatur) Soap:envelope Attlist new OSCIMsg(DH) ParseHeader()
		/// +(findDialog(conversationID)) checkControlBlock(String
		/// conversationID,String SessionID,String sequenzNR) +(checkMsgID(String
		/// MsgID)) +(checkDialogCert(X509)) parseBody decrypt(Content) hashif
		/// - Speicherung der Nachricht - create Table of HashIt (zur berprfung
		/// der Signatur) Soap:envelope Attlist new OSCIMsg(DH) ParseHeader()
		/// +(findDialog(conversationID)) checkControlBlock(String
		/// conversationID,String SessionID,String sequenzNR) +(checkMsgID(String
		/// MsgID)) +(checkDialogCert(X509)) parseBody decrypt(Content) hashif -
		/// Speicherung der Nachricht - create Table of HashIt (zur berprfung der
		/// Signatur) Soap:envelope Attlist new OSCIMsg(DH) ParseHeader()
		/// +(findDialog(conversationID)) checkControlBlock(String
		/// conversationID,String SessionID,String sequenzNR) +(checkMsgID(String
		/// MsgID)) +(checkDialogCert(X509)) parseBody decrypt(Content) hashif
		/// auffinden der Mime-informationen
		/// </summary>
		/// <param name="in_Renamed">
		/// </param>
		private void  parseMimePart(System.IO.Stream in_Renamed) {
		}
		
		private OSCIMessage saxIt(System.IO.Stream in_Renamed) {
			return null;
		}
		
		/// <summary> gltiger Soap Request, gltiges Soap mit Attachments (OSCI Transport IO)
		/// </summary>
		/// <param name="in_Renamed">
		/// </param>
		private void  checkOuterSoap(System.IO.Stream in_Renamed) {
		}
		
		private void  checkMsg() {
		}

        /// <summary> Setzt den Hash-Algorithmus fr die Signatur der Nachrichten
        /// (Voreinstellung aus DialogHandler).</summary>
        /// <param name="_digestAlgorithm">Hashalgorithmus-XML-Attribut</param>
        public void setDigestAlgorithm(String _digestAlgorithm)
        {
            if (isSigned())
                throw new Exception("Message is already signed.");
            this.digestAlgorithm = _digestAlgorithm;
        }
        
        /// <summary>  speichert die Transportnachricht (komplett verschlsselt) sowie die OSCI
		/// Nachricht nach dem decrypten des Transportumschlages
		/// </summary>
		/// <param name="in_Renamed">
		/// </param>
		/// <param name="out_Renamed">
		/// </param>
		/// <param name="nameOfStream">
		/// </param>
		private void  storeMsg(System.IO.Stream in_Renamed, System.IO.Stream out_Renamed, System.String nameOfStream) {
		}
		
		protected internal virtual void  completeMessage(System.IO.Stream out_Renamed) {
			writeCustomSOAPHeaders(out_Renamed);
			MoreSupport.ToStream(out_Renamed,"</" + soapNSPrefix + ":Header>",Constants.CHAR_ENCODING);
			body.writeXML(out_Renamed);
			MoreSupport.ToStream(out_Renamed,"</" + soapNSPrefix + ":Envelope>",Constants.CHAR_ENCODING);
			
			for (System.Collections.IEnumerator e = attachments.GetEnumerator(); e.MoveNext(); ) {
				((Attachment) ((System.Collections.DictionaryEntry)(((System.Object)((e.Current))))).Value).writeXML(out_Renamed);
			}
			
			MoreSupport.ToStream(out_Renamed,"\r\n--" + boundary_string + "--\r\n",Constants.CHAR_ENCODING);
			out_Renamed.Flush();
		}

		protected void writeCustomSOAPHeaders(System.IO.Stream out0) {
			if (customHeaders.Count > 0) {
				for (int i = 0; i < customHeaders.Count; i++) 
				{ 
					MoreSupport.ToStream(out0,((CustomHeader) customHeaders[i]).getData(),Constants.CHAR_ENCODING);
				}
			}
		}

		/// <summary> Mit dieser Methode knnen beliebige Eintrge dem SOAP-Header der
		/// Nachricht hinzugefgt werden. Die bergebenen Strings mssen vollstndige
		/// XML-Tags sein. Das unterste Element muss ein id-Attribut besitzen, welches
		/// innerhalb der Nachricht eindeutig sein muss. Die Bibliothek verwendet
		/// fr die id-Attribute die Namen der Eintrge (ohne Namespace) in
		/// Kleinschreibung, es sollten daher beispielsweise nicht "desiredlanguages",
		/// "clientsignature" oder "body" verwendet werden.
		/// Bei zu signierenden Nachrichten muss der Tag auerdem in kanonischer Form bergeben
		/// werden. Der uerste Tag muss alle Namespace-Deklarationen der OSCI-Nachricht enthalten.
		/// </summary>
		/// <param name="xml">XML-Tag</param>
		/// <seealso cref="getCustomHeaders()">
		/// </seealso>
		public void addCustomHeader(String xml) {
			customHeaders.Add(new CustomHeader(xml));
		}

		/// <summary>Liefert vorhandene SOAP-Header-Eintrge.
		/// </summary>
		/// <returns>Array der SOAP-Header-Eintrge
		/// </returns>
		/// <seealso cref="addCustomHeader(String)">
		/// </seealso>
		public String[] getCustomHeaders() {
			String[] headers = new String[customHeaders.Count];

			for (int i = 0; i < headers.Length; i++)
				headers[i] = ((CustomHeader) customHeaders[i]).getData();

			return headers;
		}
		
		/// <summary> Helfermethode zum Formatieren eines Datums in das fr OSCI erforderliche
		/// ISO-8601-Format.</summary> 
		/// <param name="xml">date Datum-Objekt</param>
		/// <returns> formatierten Datum-String
		/// </returns>
		public static String formatISO8601(DateTime date) {
            return new ISO8601DateTimeFormat().format(date);
		}

		/// <summary> Helfermethode zum Parsen eines Datumsstrings im ISO-8601-Format.</summary> 
		/// <param name="xml">Datum-String</param>
		/// <returns>date Datum-Objekt
		/// </returns>
		public static DateTime parseISO8601(String date) {
            return new ISO8601DateTimeFormat().parse(date);
		}

		public bool Base64Encoding
		{
			set
			{
				base64 = value;
			}
			get
			{
				return base64;
			}
		}

		public virtual void  writeXML(System.IO.Stream out_Renamed) {
			if ((stateOfMsg & STATE_COMPOSED) == 0) {
				compose();
			}
			MoreSupport.ToStream(out_Renamed,"MIME-Version: 1.0\r\nContent-Type: Multipart/Related; boundary=" + boundary_string + "; type=text/xml\r\n",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,"\r\n--" + boundary_string + "\r\nContent-Type: text/xml; charset=UTF-8\r\n",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,"Content-Transfer-Encoding: 8bit\r\nContent-ID: <" + contentID + ">\r\n\r\n",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n\r\n",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,"<" + soapNSPrefix + ":Envelope xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\" xmlns:osci=\"http://www.osci.de/2002/04/osci\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xenc=\"http://www.w3.org/2001/04/xmlenc#\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://schemas.xmlsoap.org/soap/envelope/ soap",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,(GetType().FullName.Substring(GetType().FullName.LastIndexOf((System.Char) '.') + 1) + ".xsd http://www.w3.org/2000/09/xmldsig# oscisig.xsd http://www.w3.org/2001/04/xmlenc# oscienc.xsd\"><" + soapNSPrefix + ":Header>"),Constants.CHAR_ENCODING);
			dialogHandler.Controlblock.writeXML(out_Renamed);
		}
				
		public override System.String ToString() {
			System.IO.MemoryStream out_Renamed = new System.IO.MemoryStream();
			try {
				writeXML(out_Renamed);
				out_Renamed.Close();
				return MoreSupport.NewString(out_Renamed.ToArray(), Constants.CHAR_ENCODING);
			}
			catch (System.Exception ex) {
				SupportClass.WriteStackTrace(ex, Console.Error);
				return null;
			}
		}

		static OSCIMessage() {
			log = LogFactory.getLog(typeof(OSCIMessage));
		}
	}
}