namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.extinterfaces;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	using de.osci.osci12.soapheader;

	/// <summary><p><H4>Bearbeitungsauftrag</H4></p>
	/// Diese Klasse reprsentiert Nachrichtenobjekte fr Bearbeitungsauftrge.
	/// Der Intermedir erzeugt nach dem Erhalt eines Abwicklungsauftrags eine
	/// Instanz dieser Klasse und sendet die Nachricht an den Empfnger (hier als Supplier).
	/// Als Antwort auf diese Nachricht mu der Empfnger ein
	/// ResponseToProcessDelivery-Nachrichtenobjekt mit einer Rckmeldung (Feedback)
	/// aufbauen und an den Intermedir zurcksenden.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.messagetypes.ResponseToProcessDelivery">
	/// </seealso>
	
	public class ProcessDelivery:OSCIRequest,ContentPackageI {

		/// <summary> Diese Methode liefert den Laufzettel der Zustellung zurck oder null,
		/// wenn bei der Verarbeitung der Nachricht ein Fehler aufgetereten ist.
		/// Die Informationen im Laufzettel knnen auch direkt ber die einzelnen
		/// getX()-Methoden ausgewertet werden.
		/// </summary>
		/// <value> Liefert den Laufzettel als ProcessCardBundle-Objekt (im Fehlerfall null).
		/// </value>
		/// <seealso cref="TimestampCreation()">
		/// </seealso>
		/// <seealso cref="TimestampForwarding()">
		/// </seealso>
		/// <seealso cref="Inspections()">
		/// </seealso>
		/// <seealso cref="Subject()">
		/// </seealso>
		/// <seealso cref="RecentModification()">
		/// </seealso>
		/// <seealso cref="MessageId()">
		/// </seealso>
		virtual public ProcessCardBundle ProcessCardBundle {
			get {
				return processCardBundle;
			}			
		}

		/// <summary> Liefert die in die Nachricht eingestellten (unverschlsselten) Inhaltsdaten als ContentContainer-Objekte.
		/// </summary>
		/// <value> enthaltene ContentContainer mit Inhaltsdaten.
		/// </value>
		/// <seealso cref="de.osci.osci12.messageparts.ContentContainer">
		/// </seealso>
		override public ContentContainer[] ContentContainer {
			get {
				return base.ContentContainer;
			}			
		}

		/// <summary> Liefert die in die Nachricht eingestellten verschlsselten Inhaltsdaten
		/// als EncryptedData-Objekte.
		/// </summary>
		/// <value> enthaltene EncryptedData-Objekt mit verschlsselten Inhaltsdaten.
		/// </value>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		override public EncryptedDataOSCI[] EncryptedData {
			get {
				return base.EncryptedData;
			}	
		}

		/// <summary> Liefert den im Laufzettel enthaltenen Zeitstempel vom Zeitpunkt des Eingangs
		/// des Abwicklungsauftrags beim Intermedir.
		/// </summary>
		/// <value> Zeitstempel der Einreichung beim Intermedir
		/// </value>
		/// <seealso cref="ProcessCardBundle()">
		/// </seealso>
		virtual public Timestamp TimestampCreation {
			get {
				return processCardBundle.Creation;
			}			
		}

		/// <summary> Liefert den im Laufzettel enthaltenen Zeitstempel vom Zeitpunkt des
		/// vollstndigen Aufbaus des Bearbeitungsauftrags vom Intermedir fr den Empfnger.
		/// </summary>
		/// <value> Zeitstempel der Erstellung des Bearbeitungsauftrags durch den Intermedir
		/// </value>
		/// <seealso cref="ProcessCardBundle()">
		/// </seealso>
		virtual public Timestamp TimestampForwarding {
			get {
				return processCardBundle.Forwarding;
			}	
		}

		/// <summary> Liefert die Ergebnisse der Zertifikatsprfungen in Form von Inspection-Objekten,
		/// die im ProcessCardBundle-Objekt enthalten sind.
		/// </summary>
		/// <value>Prfergebnisse
		/// </value>
		/// <seealso cref="ProcessCardBundle()">
		/// </seealso>
		virtual public Inspection[] Inspections {
			get {
				return processCardBundle.Inspections;
			}			
		}

		/// <summary> Liefert den im Laufzettel enthaltenen Betreff-Eintrag.
		/// </summary>
		/// <value> Betreff der Zustellung.
		/// </value>
		/// <seealso cref="ProcessCardBundle()">
		/// </seealso>
		virtual public System.String Subject {
			get {
				return processCardBundle.Subject;
			}	
		}

		/// <summary> Liefert das Datum der letzten nderung des Laufzettels. Das Format
		/// entspricht dem XML-Schema nach http://www.w3.org/TR/xmlschema-2/#dateTime
		/// </summary>
		/// <value> Datum der letzten nderung.
		/// </value>
		/// <seealso cref="ProcessCardBundle()">
		/// </seealso>
		virtual public System.String RecentModification {
			get {
				return processCardBundle.recentModification;
			}	
		}

		/// <summary> Liefert die Message-ID der Nachricht.
		/// </summary>
		/// <value> Message-ID
		/// </value>
		/// <seealso cref="ProcessCardBundle()"> 
		/// </seealso>
		virtual public System.String MessageId {
			get {
				if (processCardBundle != null) {
					return processCardBundle.MessageId;
				} else {
					return null;
				}
			}	
		}

		/// <summary> Diese Methode registriert (statisch) eine OSCIDataSource-Implementierung,
		/// die fr die Speicherung aller eingehenden Nachrichten dieses Typs
		/// verwendet wird. Beim Empfang einer Nachricht wird vom registrierten
		/// OSCIDataSource-Objekt eine neue Instanz geholt (OSCIDataSource.newInstance())
		/// und die Nachricht (der eingehende Bytestrom) in deren OutputStream geschrieben.
		/// </summary>
		/// <value>InputDataSourceImpl
		/// </value>
		/// <seealso cref="de.osci.osci12.extinterfaces.OSCIDataSource">
		/// </seealso>
		public static OSCIDataSource InputDataSourceImpl {
			set {
				inputDataSourceImpl = value;
			}	
		}

		/// <summary> Liefert die Instanz des registrierten OSCIDataSource-Objektes, welches
		/// fr die Speicherung der Nachricht beim Empfang verwendet wurde.
		/// Die Methode liefert null, wenn keine OSCIDataSource-Implementierung
		/// registriert wurde.
		/// </summary>
		/// <value> Instanz von OSCIDataSource
		/// </value>
		virtual public OSCIDataSource InputDataSource {
			get {
				return inputDataSource;
			}	
		}

		new private static Log log;
		internal ProcessCardBundle processCardBundle;
		
		private static OSCIDataSource inputDataSourceImpl;
		private OSCIDataSource inputDataSource;
		
		internal ProcessDelivery(MediateDelivery md):base(new DialogHandler((Intermed) md.dialogHandler.Supplier, md.addressee, (de.osci.osci12.extinterfaces.TransportI) null)) {
			messageType = PROCESS_DELIVERY;
			uriReceiver = md.uriReceiver;
			desiredLanguagesH = new DesiredLanguagesH(md.desiredLanguagesH.LanguageList);
			addressee = md.addressee;
			originator = md.originator;
			
			ContentContainer[] con = md.ContentContainer;
			
			for (int i = 0; i < con.Length; i++) {
				addContentContainer(con[i]);
			}
			EncryptedDataOSCI[] enc = md.EncryptedData;
			for (int i = 0; i < enc.Length; i++) {
				addEncryptedData(enc[i]);
			}
			Attachment[] att = md.Attachments;
			for (int i = 0; i < att.Length; i++) {
				addAttachment(att[i]);
			}
			for (int i = 0; i < md.OtherAuthors.Length; i++)
				otherAutors[md.OtherAuthors[i].id]=md.OtherAuthors[i];
			for (int i = 0; i < md.OtherReaders.Length; i++) 
				otherReaders[md.OtherReaders[i].id]=md.OtherReaders[i];
			dialogHandler.Encryption=md.dialogHandler.Encryption;
			dialogHandler.Controlblock.Response = null;
			dialogHandler.Controlblock.Challenge = Tools.createRandom(10);
			dialogHandler.Controlblock.ConversationID = null;
			dialogHandler.Controlblock.SequenceNumber = - 1;
		}
		
		public ProcessDelivery(DialogHandler dh):base(dh) {
			addressee = ((Addressee) dh.Supplier);
			messageType = PROCESS_DELIVERY;
		}
		
		public ProcessDelivery() {
			messageType = PROCESS_DELIVERY;
		}
				
		/// <summary> Fgt der Nachricht einen Inhaltsdatencontainer hinzu.
        /// </summary>
		/// <param name="container">Inhaltsdatencontainer
		/// </param>
		/// <seealso cref="de.osci.osci12.messageparts.ContentContainer">
		/// </seealso>
		public override void  addContentContainer(ContentContainer container) {
			base.addContentContainer(container);
		}
		
		/// <summary> Entfernt einen Inhaltsdatencontainer aus der Nachricht.
		/// </summary>
		/// <param name="container">Inhaltsdatencontainer
		/// </param>
		/// <seealso cref="addContentContainer">
		/// </seealso>
		public override void  removeContentContainer(ContentContainer container) {
			base.removeContentContainer(container);
		}
		
		/// <summary> Fgt der Nachricht einen EncryptedData-Eintrag mit verschlsselten
		/// Inhaltsdaten hinzu.
		/// </summary>
		/// <param name="encData">verschlsselte Daten
		/// </param>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		public override void  addEncryptedData(EncryptedDataOSCI encData) {
			base.addEncryptedData(encData);
		}
		
		/// <summary> Entfernt einen EncryptedData-Eintrag mit verschlsselten Daten
		/// aus der Nachricht.
		/// </summary>
		/// <param name="encData">verschlsselte Daten
		/// </param>
		/// <seealso cref="addEncryptedData">
		/// </seealso>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		public override void  removeEncryptedData(EncryptedDataOSCI encData) {
			base.removeEncryptedData(encData);
		}
				
		/**
		 * Liefert die vom Intermedir mitgesendete Message-ID fr die Antwortnachricht.
		 * @return Message-ID fr Antwortnachricht oder null, wenn keine Protokollierung
		 * verlangt ist.
		 */
		public String GetMessageIdResponse() {
			return messageId;
		}
		public override void  compose() 
		{
			base.compose();
			if (processCardBundle != null) 
			{
				String pdh = "<" + osciNSPrefix + ":MessageIdResponse>"
					+ de.osci.helper.Base64.encode(MoreSupport.ToByteArray(messageId,Constants.CHAR_ENCODING))
					+ "</" + osciNSPrefix + ":MessageIdResponse>" + processCardBundle.writeToString();

				osciH = new OsciH("processDelivery" , "" + pdh);
			} else {
				osciH = new OsciH("processDelivery", "");
			}

			if (dialogHandler.Client.hasCipherCertificate()) {				
				// Test, ob vorhanden
				if (intermediaryCertificatesH == null) {
					intermediaryCertificatesH = new IntermediaryCertificatesH();
				}
				intermediaryCertificatesH.CipherCertificateIntermediary = (Intermed) dialogHandler.Client;

			}
			
			createNonIntermediaryCertificatesH();
			
			body = new Body(ContentContainer, EncryptedData);
			stateOfMsg |= STATE_COMPOSED;
		}
		
		public override void  writeXML(System.IO.Stream out_Renamed) {
			base.writeXML(out_Renamed);

			// ClientSignatur
			if (signatureHeader != null) {
				signatureHeader.writeXML(out_Renamed);
			}

			// DesiredLanguage
			desiredLanguagesH.writeXML(out_Renamed);
			osciH.writeXML(out_Renamed);
			if (intermediaryCertificatesH != null) {
				intermediaryCertificatesH.writeXML(out_Renamed);
			}
			if (nonIntermediaryCertificatesH != null) {
				nonIntermediaryCertificatesH.writeXML(out_Renamed);
			}
			completeMessage(out_Renamed);
		}
		
		static ProcessDelivery() {
			log = LogFactory.getLog(typeof(ProcessDelivery));
		}
	}
}