 namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	using java.security.cert; 
    

  


	/// <summary><p><H4>Laufzettelabholauftrag-Nachrichtenobjekt</H4></p>
	/// Diese Klasse reprsentiert die Antwort des Intermedirs auf einen
	/// Laufzettelabholauftrag.
	/// Clients erhalten vom Intermedir eine Instanz dieser Klasse, die eine Rckmeldung
	/// ber den Erfolg der Operation (getFeedback()) sowie ggf. die
	/// angeforderten Laufzettel enthlt.
	 /// 
	 /// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	 /// <p>Erstellt von Governikus GmbH & Co.  KG

	 /// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	 /// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	 /// unter der URL www.osci.de abgerufen oder bei der
	 /// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	 /// 28195 Bremen schriftlich angefordert werden.</p>
	 /// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.messagetypes.FetchProcessCard">
	/// </seealso>

	public class ResponseToFetchProcessCard:OSCIResponseTo
	{
		private void  InitBlock()
		{
			processCardBundles = new ProcessCardBundle[0];
		}

		/// <summary> Liefert die angeforderten Laufzettel als Array von ProcessCardBundle-Objekten.
		/// </summary>
		/// <value> Laufzettel
		/// </value>
		/// <seealso cref="de.osci.osci12.messageparts.ProcessCardBundle">
		/// </seealso>
		virtual public ProcessCardBundle[] ProcessCardBundles
		{
			get
			{
				return processCardBundles;
			}
		}

		internal System.String SelectionRule;

		/// <summary> Liefert die gesetzte Auswahlregel. Der Inhalt des zruckgegebenen
		/// Strings hngt vom gesetzten Auswahlmodus ab und kann
		/// entweder in einer oder mehrerer Message-Id oder einem Datum bestehen
		/// Das Format eines Datums entspricht dem XML-Schema nach
		/// <a href="http://www.w3.org/TR/xmlschema-2/#dateTime">
		/// http://www.w3.org/TR/xmlschema-2/#dateTime</a>.
		/// Wurde keine Regel gesetzt, wird als default null zurckgegeben.
		/// Mehrere Message-Ids werden aneinandergereiht durch "&amp;" getrennt zurckgegeben.
		/// </summary>
		/// <value> die Auswahlregel (Message-Id oder Datum)
		/// </value>
		/// <seealso cref="SelectionRule">
		/// </seealso>
		virtual public System.String selectionRule
		{
			get
			{
				return SelectionRule;
			}		
		}

		internal int SelectionMode = - 1;

		/// <summary> Liefert den gesetzten Auswahlmodus.
		/// </summary>
		/// <value> Auswahlmodus SELECT_BY_MESSAGE_ID, SELECT_BY_DATE_OF_RECEPTION,
		/// SELECT_BY_RECENT_MODIFICATION oder NO_SELECTION_RULE
		/// </value>
		/// <seealso cref="SelectionMode">
		/// </seealso>
		///<getset></getset> 
		virtual public int selectionMode
		{
			get
			{
				return SelectionMode;
			}	
		}

		private long Quantity_Limit = - 1;

		/// <summary> Liefert die maximale Anzahl angeforderter Laufzettel.
		/// </summary>
		/// <value> maximale Anzahl
		/// </value>
		/// <getset></getset>
		virtual public long quantityLimit
		{
			get
			{
				return Quantity_Limit;
			}	
		}

		virtual internal long QuantityLimit
		{
			set
			{
				if (value <= 0)
				{
					throw new System.ArgumentException("Limit fr die Zahl der Laufzettel mu grer 0 sein.");
				}
				
				this.Quantity_Limit = value;
			}
			get
			{
				return Quantity_Limit;
			}
			
		}
		/// <summary>Liefert den Identifier fr das Auswahlkriterium, ob nur Laufzettel von
		/// Nachrichten zurckgegeben werden sollen, die an den oder vom Absender des
		/// Laufzettelabholauftrags geschickt wurden.
		/// Mgliche Werte sind
		/// <ul><li>SELECT_ORIGINATOR - fr Nachrichten, die vom Absender dieses
		/// Auftrags gesendet wurden.</li><li>SELECT_ADDRESSEE - fr Nachrichten, die
		/// an den Absender dieses Auftrags gesendet wurden.</li><li>SELECT_ALL -
		/// fr alle Nachrichten (default).</li></ul> <b>Diese Einstellung ist nur in
		/// den Selection-Modes SELECT_BY_DATE_OF_RECEPTION und
		/// SELECT_BY_RECENT_MODIFICATION wirksam.</b>
		/// </summary>
		/// <value>int Code for the Selection Role
		/// </value>
		internal int roleForSelection = - 1;
		public int RoleForSelection {
			get {
				return roleForSelection;
			}
		}
    
		/** 
		 * Gibt an, ob Liefert den Identifier fr das Auswahlkriterium, ob nur Laufzettel von 
		 * Nachrichten zurckgegeben werden sollen, die an den oder vom Absender des 
		 * Laufzettelabholauftrags geschickt wurden. 
		 * 
		 * @see FetchProcessCard#setRoleForSelection(int) 
		 * @return Modus 
		 */ 
		internal bool selectNoReceptionOnly = false;
		public bool SelectNoReceptionOnly { get{
				 return selectNoReceptionOnly; }
		} 

		new private static Log log;
		internal ProcessCardBundle[] processCardBundles;
		
		internal ResponseToFetchProcessCard(DialogHandler dh):base(dh)
		{
			InitBlock();
			messageType = RESPONSE_TO_FETCH_PROCESS_CARD;
			originator = ((Originator) dialogHandler.Client);
		}
		
		internal ResponseToFetchProcessCard(FetchProcessCard fpc):base(fpc.dialogHandler)
		{
			InitBlock();
			originator = ((Originator) dialogHandler.Client);
			messageType = RESPONSE_TO_FETCH_PROCESS_CARD;
			SelectionMode = fpc.SelectionMode;
			SelectionRule = fpc.SelectionRule;
			roleForSelection=fpc.RoleForSelection;
			selectNoReceptionOnly=fpc.isSelectNoReception();
			
			if (fpc.QuantityLimit > 0)
			{
				QuantityLimit = fpc.QuantityLimit;
			}
			dialogHandler.Controlblock.Response = dialogHandler.prevChallenge;
			
			dialogHandler.Controlblock.Challenge = Tools.createRandom(10);
		}
		
		public override void  compose()
		{
			base.compose();
			System.Text.StringBuilder sb = new System.Text.StringBuilder("<"+osciNSPrefix+":responseToFetchProcessCard>");
			sb.Append(writeFeedBack());
			sb.Append("<" + osciNSPrefix + ":fetchProcessCard>");
			String selectionAttributes = "";
			if (selectNoReceptionOnly) selectionAttributes += " NoReception=\"true\""; 
    
			if (roleForSelection == SELECT_ADDRESSEE) 
				selectionAttributes += " Role=\"Addressee\""; 
			else if (roleForSelection == SELECT_ORIGINATOR) 
				selectionAttributes += " Role=\"Originator\""; 
			System.Text.StringBuilder selection = new System.Text.StringBuilder("<");
			selection.Append(osciNSPrefix);
			selection.Append(":SelectionRule>");
			if (selectionMode == SELECT_BY_MESSAGE_ID) 
			{
				String[] msgIds = selectionRule.Split('&');
				for (int i = 0; i < msgIds.Length; i++) {
					selection.Append("<");
					selection.Append(osciNSPrefix);
					selection.Append(":MessageId>");
					selection.Append(de.osci.helper.Base64.encode(MoreSupport.ToByteArray(msgIds[i],Constants.CHAR_ENCODING)));
					selection.Append("</");
					selection.Append(osciNSPrefix);
					selection.Append(":MessageId>");
				}
				selection.Append("</");
			}
			else if (selectionMode == SELECT_BY_DATE_OF_RECEPTION)
			{
				selection.Append("<");
				selection.Append(osciNSPrefix);
				selection.Append(":ReceptionOfDelivery" + selectionAttributes + ">" + selectionRule +
					"</");
				selection.Append(osciNSPrefix);
				selection.Append(":ReceptionOfDelivery></");
			}
			else if (selectionMode == SELECT_BY_RECENT_MODIFICATION)
			{
				selection.Append("<");
				selection.Append(osciNSPrefix);
				selection.Append(":RecentModification" + selectionAttributes + ">" + selectionRule + "</");
				selection.Append(osciNSPrefix);
				selection.Append(":RecentModification></");
			}
			if (quantityLimit >= 0)
			{
				selection.Append("<"); 
				selection.Append(osciNSPrefix); 
				selection.Append(":Quantity Limit=\"" + quantityLimit + "\"></"); 
				selection.Append(osciNSPrefix); 
				selection.Append(":Quantity>"); 
			}
			for (int i = 0; i < processCardBundles.Length; i++)
			{
				System.IO.MemoryStream out_Renamed = new System.IO.MemoryStream();
				processCardBundles[i].writeXML(out_Renamed);
				char[] tmpChar;
				byte[] tmpByte;
				tmpByte = out_Renamed.GetBuffer();
				tmpChar = new char[out_Renamed.Length];
				System.Array.Copy(tmpByte, 0, tmpChar, 0, tmpChar.Length);
				sb.Append(new System.String(tmpChar));
			}
			sb.Append("</" + osciNSPrefix + ":responseToFetchProcessCard>");
			body = new Body(sb.ToString());
			stateOfMsg |= STATE_COMPOSED;
		}
		
		public override void  writeXML(System.IO.Stream out_Renamed)
		{
			base.writeXML(out_Renamed);
			
			// ClientSignatur
			if (signatureHeader != null)
			{
				signatureHeader.writeXML(out_Renamed);
			}
			if (intermediaryCertificatesH != null)
			{
				intermediaryCertificatesH.writeXML(out_Renamed);
			}
			completeMessage(out_Renamed);
		}
		static ResponseToFetchProcessCard()
		{
			log = LogFactory.getLog(typeof(ResponseToFetchProcessCard));
		}
	}
}