namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.extinterfaces;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	using de.osci.osci12.soapheader;

	/// <summary><p><H4>Weiterleitungsantwort-Nachrichtenobjekt</H4></p>
	/// Dieses Klasse reprsentiert die Antwort des Intermedirs auf einen
	/// Weiterleitungsauftrag.
	/// Clients erhalten vom Intermedir eine Instanz dieser Klasse, die eine Rckmeldung
	/// ber den Erfolg der Operation (getFeedback()) sowie ggf. den zugehrigen
	/// Laufzettel enthlt.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.messagetypes.ForwardDelivery">
	/// </seealso>
	public class ResponseToForwardDelivery:OSCIResponseTo
	{
		public ProcessCardBundle ProcessCardBundle;

		/// <summary> Diese Methode liefert den Laufzettel der Zustellung zurck oder null,
		/// wenn bei der Verarbeitung der Nachricht ein Fehler aufgetereten ist.
		/// Die Informationen im Laufzettel knnen auch direkt ber die einzelnen
		/// getX()-Methoden ausgewertet werden.
		/// </summary>
		/// <value> Laufzettel als ProcessCardBundle-Objekt, im Fehlerfall null
		/// </value>
		/// <seealso cref="TimestampCreation()">
		/// </seealso>
		/// <seealso cref="TimestampForwarding()">
		/// </seealso>
		/// <seealso cref="TimestampReception()">
		/// </seealso>
		/// <seealso cref="Inspections()">
		/// </seealso>
		/// <seealso cref="Subject()">
		/// </seealso>
		/// <seealso cref="RecentModification()">
		/// </seealso>
		/// <seealso cref="MessageId()">
		/// </seealso>
		virtual public ProcessCardBundle processCardBundle {
			get {
				return ProcessCardBundle;
			}
		}

		/// <summary> Liefert den im Laufzettel enthaltenen Zeitstempel vom Zeitpunkt des Eingangs
		/// des Weiterleitungsauftrags beim Intermedir.
		/// </summary>
		/// <value> Zeitstempel der Einreichung beim Intermedir
		/// </value>
		/// <seealso cref="ProcessCardBundle">
		/// </seealso>
		virtual public Timestamp TimestampCreation {
			get {
				return processCardBundle.Creation;
			}	
		}

		/// <summary> Liefert den im Laufzettel enthaltenen Zeitstempel vom Zeitpunkt des
		/// vollstndigen Aufbaus des Annahmeauftrags vom Intermedir fr den Empfnger.
		/// </summary>
		/// <value> Zeitstempel der Erstellung des Annahmeauftrags durch den Intermedir
		/// </value>
		/// <seealso cref="ProcessCardBundle">
		/// </seealso>
		virtual public Timestamp TimestampForwarding {
			get {
				return processCardBundle.Forwarding;
			}	
		}

		/// <summary> Liefert den im Laufzettel enthaltenen Zeitstempel vom Zeitpunkt des
		/// Eingangs einer positiven Annahmeantwort vom Empfnger beim Intermedir.
		/// </summary>
		/// <value> Zeitstempel der Registrierung einer Empfangsbesttigung (Annahmeantwort)
		/// durch den Intermedir
		/// </value>
		/// <seealso cref="ProcessCardBundle">
		/// </seealso>
		virtual public Timestamp TimestampReception {
			get {
				return processCardBundle.Reception;
			}	
		}

		/// <summary> Liefert die Ergebnisse der Zertifikatsprfungen in Form von Inspection-Objekten,
		/// die im ProcessCardBundle-Objekt enthalten sind.
		/// </summary>
		/// <value> Prfergebnisse
		/// </value>
		/// <seealso cref="ProcessCardBundle">
		/// </seealso>
		virtual public Inspection[] Inspections {
			get {
				return processCardBundle.Inspections;
			}	
		}

		/// <summary> Liefert den im Laufzettel enthaltenen Betreff-Eintrag.
		/// </summary>
		/// <value> Betreff der Zustellung
		/// </value>
		/// <seealso cref="ProcessCardBundle">
		/// </seealso>
		virtual public System.String Subject {
			//  void setInspections(Inspection[] inspections) {this.inspections = inspections;}
			get {
				return processCardBundle.subject;
			}
		}

		/// <summary> Liefert das Datum der letzten nderung des Laufzettels. Das Format
		/// entspricht dem XML-Schema nach http://www.w3.org/TR/xmlschema-2/#dateTime
		/// </summary>
		/// <value> Datum der letzten nderung.
		/// </value>
		/// <seealso cref="ProcessCardBundle">
		/// </seealso>
		virtual public System.String RecentModification {
			get {
				return processCardBundle.recentModification;
			}
		}

		/// <summary> Liefert die Message-ID der Nachricht.
		/// </summary>
		/// <value> Message-ID
		/// </value>
		/// <seealso cref="ProcessCardBundle"> 
		/// </seealso>
		virtual public System.String MessageId {
			get {
				return processCardBundle.MessageId;
			}	
		}

		/// <summary> Diese Methode registriert (statisch) eine OSCIDataSource-Implementierung,
		/// die fr die Speicherung aller eingehenden Nachrichten dieses Typs
		/// verwendet wird. Beim Empfang einer Nachricht wird vom registrierten
		/// OSCIDataSource-Objekt eine neue Instanz geholt (OSCIDataSource.newInstance())
		/// und die Nachricht (der eingehende Bytestrom) in deren OutputStream geschrieben.
		/// </summary>
		/// <value></value>
		/// <seealso cref="de.osci.osci12.extinterfaces.OSCIDataSource">
		/// </seealso>
		public static OSCIDataSource InputDataSourceImpl {
			set {
				inputDataSourceImpl = value;
			}
		}

		/// <summary> Liefert die Instanz des registrierten OSCIDataSource-Objektes, welches
		/// fr die Speicherung der Nachricht beim Empfang verwendet wurde.
		/// Die Methode liefert null, wenn keine OSCIDataSource-Implementierung
		/// registriert wurde.
		/// </summary>
		/// <value> Instanz von OSCIDataSource
		/// </value>
		virtual public OSCIDataSource InputDataSource {
			get {
				return inputDataSource;
			}	
		}

		new private static Log log;
		private static OSCIDataSource inputDataSourceImpl;
		private OSCIDataSource inputDataSource;
		
		/// <summary> Dieser Konstruktor ist fr den Intermedir
		/// </summary>
		/// <param name="accDel">
		/// </param>
		/// <param name="rspAccDel">
		/// </param>
		internal ResponseToForwardDelivery(AcceptDelivery accDel, ResponseToAcceptDelivery rspAccDel):base(accDel.fwdDH)
		{
			dialogHandler.Controlblock.Response = dialogHandler.prevChallenge;
			dialogHandler.Controlblock.Challenge = Tools.createRandom(10);
			if (dialogHandler.Controlblock.SequenceNumber == - 1)
			{
				dialogHandler.Controlblock.SequenceNumber = 0;
			}
			this.feedBack = rspAccDel.feedBack;
			body = new Body("");
			 body.setNSPrefixes(this);
		}
		
		internal ResponseToForwardDelivery(DialogHandler dh):base(dh)
		{
			messageType = RESPONSE_TO_FORWARD_DELIVERY;
			/// <summary> @todo Supplier oder Client??
			/// </summary>
			body = new Body("");
			 body.setNSPrefixes(this);
		}
		
		/// <summary> Liefert die erstellte OSCI-Nachricht als Stream zur bertragung an den Empfnger.
		/// </summary>
		/// <param name="out_Renamed">der Outputstream in den serialisiert werden soll
		/// </param>
		public virtual void  getMessageAsStream(System.IO.Stream out_Renamed)
		{
			this.writeXML(out_Renamed);
		}
		
		public override void  compose()
		{
			base.compose();
			if (processCardBundle == null)
			{
				osciH = new OsciH("responseToForwardDelivery", writeFeedBack());
			}
			else
			{
				System.IO.MemoryStream out_Renamed = new System.IO.MemoryStream();
				processCardBundle.writeXML(out_Renamed);
				char[] tmpChar;
				byte[] tmpByte;
				tmpByte = out_Renamed.GetBuffer();
				tmpChar = new char[out_Renamed.Length];
				System.Array.Copy(tmpByte, 0, tmpChar, 0, tmpChar.Length);
				osciH = new OsciH("responseToForwardDelivery", writeFeedBack() + new System.String(tmpChar));
			}
			stateOfMsg |= STATE_COMPOSED;
		}
			
		public override void  writeXML(System.IO.Stream out_Renamed)
		{
			log.debug("CreateMsg child");
			base.writeXML(out_Renamed);
			// ClientSignatur
			if (signatureHeader != null)
			{
				signatureHeader.writeXML(out_Renamed);
			}
			osciH.writeXML(out_Renamed);
			if (intermediaryCertificatesH != null)
			{
				intermediaryCertificatesH.writeXML(out_Renamed);
			}
			completeMessage(out_Renamed);
		}
		static ResponseToForwardDelivery()
		{
			log = LogFactory.getLog(typeof(ResponseToForwardDelivery));
		}
	}
}