namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.dotcommon;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	using de.osci.osci12.soapheader;
	
	/// <summary><p><H4>Bearbeitungsantwort</H4></p>
	/// Mit dieser Klasse werden Nachrichtenobjekte fr Bearbeitungsantworten
	/// angelegt. Ein passiver Client, der als Supplier fungiert, mu nach Empfang
	/// eines Bearbeitungsauftrags eine Instanz dieser Klasse aufbauen und an den Intermedir
	/// zurcksenden. Die Nachricht enthlt eine Rckmeldung ber
	/// den Empfang der Nachricht (Feedback) sowie ggf. verschlsselte bzw.
	/// unverschlsselte Inhaltsdaten.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.messagetypes.ProcessDelivery">
	/// </seealso>
	///<konvert>ContentPackageI zz nicht ntig</konvert>	
	public class ResponseToProcessDelivery:OSCIResponseTo,ContentPackageI {

		new private static Log log;
		private System.String subject;

		/// <summary> Liefert den im Laufzettel enthaltenen Betreff-Eintrag.
		/// </summary>
		/// <returns> den Betreff der Zustellung
		/// </returns>
		/// <summary>  Setzt den Betreff-Eintrag im Laufzettel
		/// </summary>
		/// <param name="subject"> der Betreff
		/// </param>
		virtual public System.String Subject {
			get {
				return subject;
			}
			set {
				this.subject = value;
			}
		}

		/// <summary> Setzt die Rckmeldungen (Fehler und Warnungen) auf Auftragsebene
		/// </summary>
		/// <param name="code">Array mit Fehlercodes
		/// </param>
		/// <summary> Liefert die Qualitt des Zeitstempels, mit dem der Intermedir den
		/// Eingang des Auftrags im Laufzettel protokolliert.
		/// </summary>
		/// <returns> Qualitt des Zeitstempels: <b>true</b> - kryptographischer Zeitstempel von einem
		/// akkreditierten Zeitstempeldienst.<br><b>false</b> - Einfacher Zeitstempel
		/// (lokale Rechnerzeit des Intermedrs).
		/// </returns>
		/// <seealso cref="">#setQualityOfTimeStampCreation(boolean)
		/// </seealso>
		
		/// <summary> Setzt die gewnschte Qualitt des Zeitstempels, mit dem der Intermedir
		/// den Eingang des Auftrags im Laufzettel protokolliert.
		/// </summary>
		/// <param name="cryptographic"><b>true</b>: kryptographischer Zeitstempel von einem
		/// akkreditierten Zeitstempeldienst.<b>false</b>: Einfacher Zeitstempel
		/// (lokale Rechnerzeit des Intermedrs).
		/// </param>
		/// <seealso cref="QualityOfTimeStampCreation()">#getQualityOfTimeStampCreation()
		/// </seealso>
		virtual public bool QualityOfTimeStampCreation {
			get {
				return qualityOfTimestampTypeCreation.QualityCryptographic;
			}
			set {
				qualityOfTimestampTypeCreation = new QualityOfTimestampH(false, value);
			}
		}

		/// <summary> Liefert die geforderte Qualitt des Zeitstempels, mit dem der Intermedir den
		/// Empfang der Annahmeantwort im Laufzettel protokolliert.
		/// </summary>
		/// <returns> Qualitt des Zeitstempels: <b>true</b> - kryptographischer Zeitstempel von einem
		/// akkreditierten Zeitstempeldienst.<br><b>false</b> - Einfacher Zeitstempel
		/// (lokale Rechnerzeit des Intermedrs).
		/// </returns>
		/// <seealso cref="">#setQualityOfTimeStampReception(boolean)
		/// </seealso>
		
		/// <summary> Setzt die gewnschte Qualitt des Zeitstempels, mit dem der Intermedir die
		/// Empfangbesttigung der Zustellung durch den Empfnger im Laufzettel protokolliert.
		/// Die Empfangsbesttigung besteht in einem weiteren Auftrag, den der Empfnger
		/// nach Erhalt der Bearbeitungsantwort innerhalb desselben expliziten Dialogs
		/// an den Intermedir schickt.
		/// </summary>
		/// <param name="cryptographic"><b>true</b>: kryptographischer Zeitstempel von einem
		/// akkreditierten Zeitstempeldienst.<b>false</b>: Einfacher Zeitstempel
		/// (lokale Rechnerzeit des Intermedrs).
		/// </param>
		/// <seealso cref="QualityOfTimeStampReception()">#getQualityOfTimeStampReception()
		/// </seealso>
		virtual public bool QualityOfTimeStampReception {
			get {
				return qualityOfTimestampTypeReception.QualityCryptographic;
			}
			set {
				qualityOfTimestampTypeReception = new QualityOfTimestampH(true, value);
			}
		}

		/// <summary> Liefert ein Array mit den Inhaltsdatencontainern der Nachricht.
		/// </summary>
		/// <seealso cref="addContentContainer">
		/// </seealso>
		override public ContentContainer[] ContentContainer {
			get {
				return base.ContentContainer;
			}
		}

		/// <summary> Liefert die in die Nachricht eingestellten verschlsselten Inhaltsdaten
		/// als EncryptedData-Objekte.
		/// </summary>
		/// <returns> enthaltene EncryptedData-Objekt mit verschlsselten Inhaltsdaten.
		/// </returns>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		override public EncryptedDataOSCI[] EncryptedData {
			get {
				return base.EncryptedData;
			}
		}

		/// <summary> Liefert die Message-ID der Nachricht.
		/// </summary>
		/// <returns> die Message-ID
		/// </returns>
		
		/// <summary> Setzt die Message-ID der Nachricht.
		/// </summary>
		virtual public System.String MessageId {
			get {
				return messageId;
			}
			
			set {
				this.messageId = value;
			}
		}
		
		internal ResponseToProcessDelivery(DialogHandler dh):base(dh) {
			messageType = RESPONSE_TO_PROCESS_DELIVERY;
		}
		
		/// <summary> Legt ein Nachrichtenobjekt fr eine Bearbeitungsantwort an.
		/// </summary>
		/// <param name="dh">DialogHandler-Objekt des Dialogs, innerhalb dessen die Nachricht
		/// versendet werden soll.
		/// </param>
		/// <seealso cref="de.osci.osci12.common.DialogHandler">
		/// </seealso>
		public ResponseToProcessDelivery(ProcessDelivery procDel):base(procDel.dialogHandler) {
			messageType = RESPONSE_TO_PROCESS_DELIVERY;
			// Weil in diesem Szenario die Rollen wechseln (der Orginator der Anfrage wird zum Addressee der Antwort
			// und umgekehrt) werden die Rollenobjekte umgebaut. Damit passen die Rollenobjekte des DialogHandlers nicht mehr so richtig,
			// aber in einem impliziten Dialog kann man das wohl riskieren.
			de.osci.osci12.extinterfaces.crypto.Signer signer = null;
			de.osci.osci12.extinterfaces.crypto.Decrypter decrypter = null;
			 if (dialogHandler.Supplier.hasSignaturePrivateKey())
				signer = dialogHandler.Supplier.Signer;
			
			if (dialogHandler.Supplier.hasCipherPrivateKey())
				decrypter = dialogHandler.Supplier.Decrypter;
			
			originator = new Originator(signer, decrypter);

			java.security.cert.X509Certificate signerCert = null;
			java.security.cert.X509Certificate cipherCert = null;
			
			if (procDel.Originator.hasSignatureCertificate()) 
				signerCert = procDel.Originator.SignatureCertificate;
			
			if (procDel.Originator.hasCipherCertificate())
				cipherCert = procDel.Originator.CipherCertificate;
			
			addressee = new Addressee(signerCert, cipherCert);
			
			dialogHandler.Controlblock.Response = dialogHandler.prevChallenge;
			dialogHandler.Controlblock.Challenge = null;
			dialogHandler.Controlblock.ConversationID = null;
			dialogHandler.Controlblock.SequenceNumber = - 1;
		}
		
		/// <summary> Fgt der Nachricht einen Inhaltsdatencontainer hinzu.
        /// <b>Hinweis: </b>ContentContainer-Objekte mit Attachments, die durch Entschlsselung 
        /// von EncryptedDataOSCI-Objekten gewonnen wurden, knnen hier nicht ohne weiteres
        /// hinzugefgt werden. In diesem Fall mssen die Attachments zunchst ausgelesen und 
        /// die Daten in neu angelegten Attachment-Objekten wieder hinzugefgt werden.
        /// </summary>
		/// <param name="container">Inhaltsdatencontainer
		/// </param>
		/// <seealso cref="de.osci.osci12.messageparts.ContentContainer">
		/// </seealso>
		public override void  addContentContainer(ContentContainer container) {
			base.addContentContainer(container);
		}
		
		/// <summary> Entfernt einen Inhaltsdatencontainer aus der Nachricht.
		/// </summary>
		/// <param name="container">Inhaltsdatencontainer
		/// </param>
		/// <seealso cref="addContentContainer">
		/// </seealso>
		public override void  removeContentContainer(ContentContainer container) {
			base.removeContentContainer(container);
		}
		
		/// <summary> Fgt der Nachricht einen EncryptedData-Eintrag mit verschlsselten
		/// Inhaltsdaten hinzu.
		/// </summary>
		/// <param name="encData">verschlsselte Daten
		/// </param>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		public override void  addEncryptedData(EncryptedDataOSCI encData) {
			base.addEncryptedData(encData);
		}
		
		/// <summary> Entfernt einen EncryptedData-Eintrag mit verschlsselten Daten
		/// aus der Nachricht.
		/// </summary>
		/// <param name="encData">verschlsselte Daten
		/// </param>
		/// <seealso cref="addEncryptedData">
		/// </seealso>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		public override void  removeEncryptedData(EncryptedDataOSCI encData) 
        {
			base.removeEncryptedData(encData);
		}

        /// <summary> Setzt die feedback-Eintrge.
        /// </summary>
        public void setFeedback(String[] code) 
        {
			base.setFeedback(code);
		}


        /// <summary> Setzt die feedback-Eintrge mit frei whlbaren Texten und Sprachen.
        /// Der erste Index des Arrays entspricht dem Index des Entry-Elementes.
        /// Beim zweiten Index bezeichnet
        /// 0- das Sprachkrzel
        /// 1- den Code
        /// 2- den Text
        /// </summary>
        override public System.String[][] Feedback
        {
            set
            {
                base.Feedback = value;
            }
        }



		
		/// <summary> Bringt eine Supplier-Signatur an.
		/// @throws IOException bei Schreib-/Leseproblemen
		/// @throws OSCIRoleException wenn dem Rollenobjekt, das als Client fungiert
		/// kein Signer-Objekt zugeordnet wurde.
		/// @throws java.security.SignatureException bei Signatur-Problemen
		/// @throws de.osci.osci12.common.OSCICancelledException bei Abbruch durch den
		/// Benutzer
		/// </summary>
		public override void  sign() 
        {
			base.sign(originator);
		}
		
		public override void  compose() {
			System.String head = writeFeedBack();
			if (messageId != null) {
				head += "<" + osciNSPrefix + ":MessageId>" + de.osci.helper.Base64.encode(MoreSupport.ToByteArray(messageId,Constants.CHAR_ENCODING)) + "</" + osciNSPrefix + ":MessageId>";
				if (subject != null) {
					head += "<" + osciNSPrefix + ":Subject>" + subject + "</" + osciNSPrefix + ":Subject>";
				}
			}
			else { 
				qualityOfTimestampTypeCreation = null; 
				qualityOfTimestampTypeReception = null; 
			} 

			osciH = new OsciH("responseToProcessDelivery", head);
			createNonIntermediaryCertificatesH();
			body = new Body(ContentContainer, EncryptedData);
			
			stateOfMsg |= STATE_COMPOSED;
		}
		
		public override void  writeXML(System.IO.Stream out_Renamed) {
			base.writeXML(out_Renamed);
			// ClientSignatur
			if (signatureHeader != null) {
				signatureHeader.writeXML(out_Renamed);
			}
			
			if (qualityOfTimestampTypeCreation != null) {
				qualityOfTimestampTypeCreation.writeXML(out_Renamed);
			}
			if (qualityOfTimestampTypeReception != null) {
				qualityOfTimestampTypeReception.writeXML(out_Renamed);
			}
			osciH.writeXML(out_Renamed);
			
			if (nonIntermediaryCertificatesH != null) {
				nonIntermediaryCertificatesH.writeXML(out_Renamed);
			}
			completeMessage(out_Renamed);
		}
		
		/// <summary> Serialisiert und schreibt die Nachricht - ggf. verschlsselt - in den bergebenen Stream.
		/// Die ausgehende Nachricht kann zu Debug- oder Archivierungszwecken (in jedem Fall
		/// unverschlsselt) in den zweiten bergebenen Stream geschrieben werden.
		/// Dieser Parameter kann null sein.
		/// @param out Stream, in den die Antwortnachricht geschrieben werden soll
		/// @param storeOutput Stream, in dem die (unverschlsselte) Antwortnachricht
		/// gespeichert werden soll
		/// Algorithmus nicht untersttzt wird
		/// </summary>
		public virtual void  writeToStream(System.IO.Stream out_Renamed, System.IO.Stream storeOutput) {
			if (dialogHandler.Encryption) {
				new SOAPMessageEncrypted(this, storeOutput).writeXML(out_Renamed);
			} else 
				if (storeOutput != null) {
				StoreOutputStream sos = new StoreOutputStream(out_Renamed, storeOutput);
				writeXML(sos);
				sos.Close();
			}
			else 
				writeXML(out_Renamed);
		}

		static ResponseToProcessDelivery() {
			log = LogFactory.getLog(typeof(ResponseToProcessDelivery));
		}
	}
}