namespace de.osci.osci12.messagetypes {

	using System;
	using Tools = de.osci.helper.Tools;
	using de.osci.osci12;
	using de.osci.osci12.common;
	using OSCIDataSource = de.osci.osci12.extinterfaces.OSCIDataSource;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	using de.osci.osci12.soapheader;
	using org.apache.commons.logging;

	/// <summary> <p>Eine Instanz dieser Klasse wird beim Einlesen einer serialisierten
	/// OSCI-Nachricht beliebigen Typs (Nachrichten mit Inhaltsdaten) angelegt.</p>
	/// Die Klasse dient folgenden Zwecken:<ul><li>Abspeichern und Wiedereinlesen von
	/// Nachrichten mit Inhaltsdaten</li><li>Austausch von Inhaltsdaten zwischen
	/// Autoren und Sendern bzw. Empfnger und Lesern.</li></ul>
	/// <p>
	/// Leser knnen ihre Inhaltsdaten in Nachrichten beliebigen Typs ablegen, diese
	/// speichern und z.B. als Datei weiterreichen. Absender knnen eine solche Datei
	/// mit der Methode loadMessage(InputStream) laden, die Inhaltdatencontainer
	/// entnehmen und anderen Nachrichten hinzufgen.</p>
	/// <p>
	/// Ein generelles Problem ist, dass in verschlsselten Inhaltsdaten einer
	/// OSCI-Nachricht die Informationen ber die enthaltenen Referenzen auf
	/// Zertifikate und Attachments ohne Entschlsselung nicht verfgbar sind.
	/// Im Zweifel mssen daher alle Zertifikate (z.B. mit den Methoden
	/// OSCIMessage.getOtherAuthors(), OSCIMessage.getOtherReaders() und
	/// OSCIMessage.addRole(Role)) und Attachments (Methoden
	/// exportAttachment(OSCIMessage, Attachment) und exportAttachments(OSCIMessage))
	/// entnommen und der neuen Nachricht hinzugefgt werden.</p>
	/// <p>
	/// Anwendungen sollten dies bercksichtigen und fr den Inhaltsdatenaustausch
	/// mglichst mehrere einzelne Nachrichten statt einer komplexen verwenden.
	/// Besonders problematisch ist in diesem Zusammenhang die Signatur durch
	/// Originator- bzw. Verschlsselung fr Addressee-Rollenobjekte, weil diese
	/// i.d.R. nicht in eine neue Nachricht bernommen werden knnen. Hier sollten
	/// grundstzlich Author- und Reader-Objekte verwendet werden.</p>
	/// <p>
	/// Weiter ist zu beachten, dass es beim Zusammensetzen einer neuen Nachricht aus
	/// Inhaltdatencontainern, die anderen Nachrichten entnommen wurden, zu Konflikten
	/// mit den Ref-Ids der Content-Eintrge kommen kann. Da die Bibliothek wegen der
	/// ggf. vorhandenen Signatur diese Ids nicht selbst anpassen kann, sollten
	/// Anwendungen eindeutige Ref-Ids setzen. Diese knnen z.B. aus Message-Ids
	/// und laufenden Nummern oder Zertfikats-Ids (z.B. IssuerDN und SerialNumber)
	/// und Datum/Uhrzeit generiert werden.</p>
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class StoredMessage:OSCIResponseTo {

		new private static Log log;
		internal ProcessCardBundle[] processCardBundles;
		internal ProcessCardBundle processCardBundleReply;
		internal System.String subject;
		internal System.Uri uriReceiver;
		internal System.String selectionRule;
		internal int selectionMode = - 1;
		internal long quantityLimit = - 1;

		/// <summary> Diese Methode liefert den Laufzettel der Nachricht zurck.
		/// Im Falle einer Abwicklungsantwort wird der Laufzettel des Requests
		/// zurckgegeben.
		/// </summary>
		/// <value> Laufzettel des Auftrags als ProcessCardBundle-Objekt oder null,
		/// wenn der Nachrichtentyp keinen Laufzettel enthlt.
		/// </value>
		/// <seealso cref="ProcessCardBundleReply">
		/// </seealso>
		/// <seealso cref="de.osci.osci12.messagetypes.ResponseToMediateDelivery.ProcessCardBundleRequest()">
		/// </seealso>
		virtual public ProcessCardBundle ProcessCardBundle {
			get {
				if (processCardBundles == null) return null;
				return processCardBundles[0];
			}	
		}

		/// <summary> Diese Methode liefert die Laufzettel einer Laufzettelabholantwort zurck.
		/// </summary>
		/// <value> Laufzettel des Auftrags als ProcessCardBundle-Objekt.
		/// </value>
		/// <seealso cref="ProcessCardBundle">
		/// </seealso>
		/// <seealso cref="de.osci.osci12.messagetypes.ResponseToFetchProcessCard.ProcessCardBundles()">
		/// </seealso>
		/// <exception cref="System.Exception">Im Fehlerfall
		/// </exception>
		virtual public ProcessCardBundle[] ProcessCardBundles {
			get {
				if (messageType != OSCIMessage.RESPONSE_TO_FETCH_PROCESS_CARD)
					throw new System.Exception("Funktion wird nicht untersttzt. (ProcessCardBundles)");
				
				return processCardBundles;
			}	
		}

		/// <summary> Diese Methode liefert den Antwort-Laufzettel einer Abwicklungsantwort zurck.
		/// </summary>
		/// <value> Laufzettel der Nachricht als ProcessCardBundle-Objekt.
		/// </value>
		/// <seealso cref="ProcessCardBundle">
		/// </seealso>
		/// <exception cref="System.Exception">Im Fehlerfall
		/// </exception>
		virtual public ProcessCardBundle ProcessCardBundleReply {
			get {
				if (messageType != OSCIMessage.RESPONSE_TO_MEDIATE_DELIVERY)
					throw new System.Exception("Funktion wird nicht untersttzt. (ProcessCardBundleReply)");
				
				return processCardBundleReply;
			}			
		}

		/// <summary> Liefert die Message-Id der Nachricht.
		/// Im Falle einer Abwicklungsantwort wird die Message-Id des Requests
		/// zurckgegeben.
		/// </summary>
		/// <value> Message-ID
		/// </value>
		virtual public System.String MessageId {
			get {
				return messageId;
			}			
		}

		/// <summary> Liefert das Intermediresobjekt oder null, wenn keine Zertifikate in der
		/// Nachricht enthalten sind.
		/// </summary>
		/// <value> Intermedir
		/// </value>
		virtual public Intermed Intermediary {
			get {
				if (dialogHandler.Supplier is Intermed)
					return (Intermed) dialogHandler.Supplier;
				else
					return (Intermed) dialogHandler.Client;
			}	
		}

		/// <summary> Liefert die in die Nachricht eingestellten (unverschlsselten) Inhaltsdaten als 
		/// ContentContainer-Objekte.
		/// </summary>
		/// <value> enthaltene ContentContainer mit Inhaltsdaten.
		/// </value>
		/// <seealso cref="de.osci.osci12.messageparts.ContentContainer">
		/// </seealso>
		override public ContentContainer[] ContentContainer {
			get {
				return base.ContentContainer;
			}
		}

		/// <summary> Liefert die in die Nachricht eingestellten verschlsselten Inhaltsdaten
		/// als EncryptedData-Objekte.
		/// </summary>
		/// <value> enthaltene EncryptedData-Objekt mit verschlsselten Inhaltsdaten.
		/// </value>
		/// <seealso cref="EncryptedDataOSCI">
		/// </seealso>
		override public EncryptedDataOSCI[] EncryptedData {
			get {
				return base.EncryptedData;
			}	
		}

		/// <summary> Liefert den Betreff der Nachricht oder null, wenn kein Betreff in der
		/// Nachricht enthalten ist.
		/// </summary>
		/// <value> Betreff
		/// </value>
		virtual public System.String Subject {
			get {
				return subject;
			}	
		}

		/// <summary> Liefert die URI des Nachrichtenempfngers oder null, wenn keine Empfnger-URI
		/// in der Nachricht enthalten ist.
		/// </summary>
		/// <value> URI
		/// </value>
		virtual public System.Uri UriReceiver {
			get {
				return uriReceiver;
			}	
		}
		
		/// <summary> Liest eine (unverschlsselte) Nachricht aus dem bergebenen Stream. Die in
		/// der Nachricht enthaltenen Inhalte knnen dem zurckgegebenen
		/// Nachrichtenobjekt entnommen werden.
		/// </summary>
		/// <param name="input">zu lesender Stream.
		/// </param>
		/// <returns> StoredMessage-Instanz
		/// </returns>
		/// <seealso cref="storeMessage(OSCIMessage, System.IO.Stream)">
		/// </seealso>
		public static StoredMessage loadMessage(System.IO.Stream input) {
			return new StoredMessageParser().parseStream(input);
		}

		/// <summary> Serialisiert die bergebene Nachricht und schreibt die Daten in den bergebenen Stream.
		/// </summary>
		/// <param name="msg">zu speichernde Nachricht; z.Zt. knnen StoredMessage-Objekte selbst nicht
		/// gespeichert werden.
		/// </param>
		/// <param name="output">Ausgabestream
		/// </param>
		/// <seealso cref="loadMessage(System.IO.Stream)">
		/// </seealso>
		/// <exception cref="System.Exception">Im Fehlerfall
		/// </exception>
		public static void storeMessage(OSCIMessage msg, System.IO.Stream output) {
			if (msg is StoredMessage)
				throw new System.Exception("Funktion wird nicht untersttzt. (storeMessage)");
			msg.writeXML(output);
		}

        /// <summary>Liest eine (unverschlsselte) Nachricht aus dem bergebenen Stream. Die in
        /// der Nachricht enthaltenen Inhalte knnen dem zurckgegebenen
        /// Nachrichtenobjekt entnommen werden. Diese Methode prft auerdem 
        /// eine ggf. vorhandene Nachrichtensignatur. Liefert die Prfung ein 
        /// negatives Ergebnis, wird eine OSCIException (Code 9601) geworfen.
        /// Die Methode prft keine Inhaltsdatensignaturen. Ebenso wird
        /// keine Exception ausgelst, wenn die Nachricht unsigniert ist.
        /// </summary>
        /// <param name="input">zu lesender Stream
        /// </param>
        /// <param name="output">Ausgabestream
        /// </param>
        /// <returns> StoredMessage-Instanz
        /// </returns>
        /// <seealso cref="loadMessage(System.IO.Stream)">
        /// </seealso>
        /// <exception cref="System.IO.IOException"> bei Schreibfehlern
        /// </exception>
        /// <exception cref="OSCIException"> bei Problemen beim Aufbau der Nachricht oder ungltiger Nachrichtensignatur
        /// </exception>
        /// <exception cref="OSCIException"> bei Problemen beim Aufbau der Nachricht oder ungltiger Nachrichtensignatur
        /// </exception>
        
        public static StoredMessage loadMessageCheckingSignature(System.IO.Stream input)
        {      
            StoredMessageParser smp =new StoredMessageParser(); 
	        StoredMessage sm = smp.parseStream(input);
	        if (sm.isSigned())
	        {
		        if (sm.messageType <= 0x10)
			        sm.dialogHandler.supplier.SignatureCertificate=sm.dialogHandler.client.SignatureCertificate;
	  
		        if (!smp.checkMsgHashes(sm))
			        throw new OSCIException("9601");
	        }		  
	        return sm;
        }

        /// <summary>Diese Methode exportiert alle Attachments der Nachricht in eine andere
        /// OSCI-Nachricht beliebigen Typs.
        /// </summary>
        /// <param name="destinationMessage">Zielnachricht
        /// </param>
        public void exportAttachments(OSCIMessage destinationMessage)
        {
            Attachment[] atts = Attachments;

            for (int i = 0; i < atts.Length; i++)
                destinationMessage.addAttachment(atts[i]);
        }

        /// <summary>Diese Methode exportiert ein Attachment der Nachricht in eine andere
        /// OSCI-Nachricht beliebigen Typs.
        /// </summary>
        /// <param name="destinationMessage">Zielnachricht
        /// </param>
        /// <param name="att">zu exportierendes Attachment
        /// </param>
        public void exportAttachment(OSCIMessage destinationMessage, Attachment att)
        {
            Attachment[] atts = Attachments;

            for (int i = 0; i < atts.Length; i++)
            {
                if (att.Equals(atts[i]))
                {
                    destinationMessage.addAttachment(atts[i]);
                    return;
                }
            }
        }
		
		/// <summary> Liefert die Qualitt des Zeitstempels, mit dem der Intermedir den
		/// Eingang des Auftrags im Laufzettel protokolliert.
		/// </summary>
		/// <value> Qualitt des Zeitstempels:
		/// <p> <b>true</b> - kryptographischer Zeitstempel von einem
		/// akkreditierten Zeitstempeldienst.</p>
		/// <p> <b>false</b> - Einfacher Zeitstempel
		/// (lokale Rechnerzeit des Intermedrs).</p>
		/// </value>
		/// <seealso cref="QualityOfTimeStampReception()">
		/// </seealso>
		/// <exception cref="System.Exception">Im Fehlerfall
		/// </exception>
		virtual public bool QualityOfTimeStampCreation {
			get {
				if ((messageType != OSCIMessage.STORE_DELIVERY) && (messageType != OSCIMessage.FORWARD_DELIVERY) && (messageType != OSCIMessage.MEDIATE_DELIVERY) && (messageType != OSCIMessage.RESPONSE_TO_PROCESS_DELIVERY))
					throw new System.Exception("Funktion wird nicht untersttzt. (QualityOfTimeStampCreation)");
				return qualityOfTimestampTypeCreation.QualityCryptographic;
			}
		}

		/// <summary> Liefert die geforderte Qualitt des Zeitstempels, mit dem der Intermedir den
		/// Empfang der Annahmeantwort im Laufzettel protokolliert.
		/// </summary>
		/// <value> Qualitt des Zeitstempels: 
		/// <p> <b>true</b> - kryptographischer Zeitstempel von einem
		/// akkreditierten Zeitstempeldienst.</p>
		/// <p> <b>false</b> - Einfacher Zeitstempel
		/// (lokale Rechnerzeit des Intermedrs).</p>
		/// </value>
		/// <seealso cref="QualityOfTimeStampCreation()">
		/// </seealso>
		/// <exception cref="System.Exception">Im Fehlerfall
		/// </exception>
		virtual public bool QualityOfTimeStampReception {
			get {
				if ((messageType != OSCIMessage.STORE_DELIVERY) && (messageType != OSCIMessage.FORWARD_DELIVERY) && (messageType != OSCIMessage.MEDIATE_DELIVERY) && (messageType != OSCIMessage.RESPONSE_TO_PROCESS_DELIVERY))
					throw new System.Exception("Funktion wird nicht untersttzt. (QualityOfTimeStampReception)");
				return qualityOfTimestampTypeReception.QualityCryptographic;
			}	
		}

		/// <summary> Liefert den gesetzten Auswahlmodus fr Nachrichten oder Laufzettel.
		/// </summary>
		/// <value> Auswahlmodus SELECT_BY_MESSAGE_ID, SELECT_BY_DATE_OF_RECEPTION
		/// oder NO_SELECTION_RULE
		/// </value>
		/// <seealso cref="SelectionRule()">
		/// </seealso>
		/// <exception cref="System.Exception">Im Fehlerfall
		/// </exception>
		virtual public int SelectionMode {
			get {
				if ((messageType != OSCIMessage.FETCH_DELIVERY) && (messageType != OSCIMessage.FETCH_PROCESS_CARD) && (messageType != OSCIMessage.RESPONSE_TO_FETCH_DELIVERY) && (messageType != OSCIMessage.RESPONSE_TO_FETCH_PROCESS_CARD))
					throw new System.Exception("Funktion wird nicht untersttzt. (SelectionMode)");
				return selectionMode;
			}	
		}

		/// <summary> Liefert die gesetzte Auswahlregel fr Nachrichten oder Laufzettel.
		/// Wurde keine Regel gesetzt, wird als default null zurckgegeben.
		/// </summary>
		/// <value> Auswahlregel (Message-Id oder Datum)
		/// </value>
		/// <seealso cref="SelectionMode()">
		/// </seealso>
		/// <exception cref="System.Exception">Im Fehlerfall
		/// </exception>
		virtual public System.String SelectionRule {
			get {
				if ((messageType != OSCIMessage.FETCH_DELIVERY) && (messageType != OSCIMessage.FETCH_PROCESS_CARD) && (messageType != OSCIMessage.RESPONSE_TO_FETCH_DELIVERY) && (messageType != OSCIMessage.RESPONSE_TO_FETCH_PROCESS_CARD))
					throw new System.Exception("Funktion wird nicht untersttzt. (SelectionRule)");
				return selectionRule;
			}	
		}

		/// <summary> Liefert die maximale Anzahl zurckzugebender Laufzettel.
		/// </summary>
		/// <value> gesetzte maximale Anzahl.
		/// </value>
		/// <exception cref="System.Exception">Im Fehlerfall
		/// </exception>
		virtual public long QuantityLimit {
			get {
				if ((messageType != OSCIMessage.FETCH_PROCESS_CARD) && (messageType != OSCIMessage.RESPONSE_TO_FETCH_PROCESS_CARD))
					throw new System.Exception("Funktion wird nicht untersttzt. (QuantityLimit)");
				return quantityLimit;
			}	
		}
		
		internal StoredMessage(int messageType) {
			this.messageType = messageType;
		}
		
		static StoredMessage() {
			log = LogFactory.getLog(typeof(StoredMessage));
		}
	}
}