namespace de.osci.osci12.samples.impl.crypto 
{

    using System;
    using System.IO;
    using de.osci.osci12.extinterfaces.crypto;
	using de.osci.osci12.dotcommon;
	using System.Security.Cryptography;
	using java.security.cert;

    using Org.BouncyCastle.Crypto;
    using Org.BouncyCastle.Crypto.Encodings;
    using Org.BouncyCastle.Crypto.Parameters;
    using Org.BouncyCastle.Math;
    using Org.BouncyCastle.Security;
    using Org.BouncyCastle.Pkcs;
    using Org.BouncyCastle.X509;
    using Org.BouncyCastle.Utilities;
    using Org.BouncyCastle.Crypto.Engines;
 
    using org.apache.commons.logging;

	/// <summary><p>Diese Klasse ist eine Beispiel-Implementierung der abstrakten Decrypter-Klasse.
	/// Fr die Verwendung wird ein PKCS#12-Keystore in Form einer *.p12-Datei bentigt.
	/// Die Implementierung ist fr Testzwecke bestimmt, sie greift auf den ersten
	/// verfgbaren Alias zu. Die PIN fr dessen Privatschlssel muss die gleiche sein
	/// wie die des Keystores.</p>
	/// <p>Diese einfache Implementierung hlt die PIN des Keystores als Character-Array
	/// im Arbeitsspeicher, sie wird als String bergeben. Es wird Anwendern empfohlen,
	/// eigene Implementierungen zu schreiben, die die PIN in der Methode decrypt(...)
	/// abfragen und nach Gebrauch wieder lschen oder anderweitig fr ein sicheres
	/// Pin-Cashing zu sorgen.</p>
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.extinterfaces.crypto.Decrypter">
	/// </seealso>
	public class PKCS12Decrypter : de.osci.osci12.extinterfaces.crypto.Decrypter 
    {
		// private static Log log = LogFactory.getLog(PKCS12Decrypter.class);
        private static Log log;
        private java.security.cert.X509Certificate cert;
        private AsymmetricKeyParameter key;

        static PKCS12Decrypter()
        {
            log = LogFactory.getLog(typeof(PKCS12Decrypter));
		}

		/// <summary> Liefert den Namen des Herstellers.
		/// </summary>
		/// <value> Herstellername
		/// </value>
		public override String Vendor 
        { 
			get 
            {
				return "BOS";
			}
		}

		/// <summary> Liefert die Versionsnummer.
		/// </summary>
		/// <value> Versionsnummer
		/// </value>
		public override String Version 
        {
			get 
            { 
				return "1.6";
			}
		}

		/// <summary> Liefert das Verschlsselungszertifikat.
		/// </summary>
		/// <value> Verschlsselungszertifikat
		/// </value>
		public override java.security.cert.X509Certificate Certificate 
        {
			get 
            {
				return cert;
			}
		}

		public PKCS12Decrypter(String p12_fileName, String pin) 
		{

            FileStream fs = new FileStream(p12_fileName, FileMode.Open, FileAccess.Read);
            Pkcs12Store p12 = new Pkcs12Store(fs, pin.ToCharArray());
            System.Collections.IEnumerable ie = p12.Aliases;

            foreach (String alias in ie)
                if (p12.IsKeyEntry(alias))
                {
                    key = p12.GetKey(alias).Key;
                    cert = new java.security.cert.X509Certificate(p12.GetCertificate(alias).Certificate.GetEncoded());
                    break;
                }
		}

        /// <summary> Entschlsselt das bergebene Byte-Array gem RSAES-PKCS1-v1_5.
		/// </summary>
		/// <param name="data"> die zu entschlsselnden Daten
		/// </param>
		/// <returns> die verschlsselten Daten
		/// </returns>
        public override byte[] decrypt(byte[] data)
        {
            return de.osci.osci12.encryption.Crypto.doRSADecryption(key, data, null, null);
        }

        /// <summary> Entschlsselt das bergebene Byte-Array gem RSAES-OAEP oder RSAES-PKCS1-v1_5.
        /// </summary>
        /// <param name="data"> die zu entschlsselnden Daten
        /// </param>
        /// <param name="mfgAlgorithm"> der vwendete Mask-Generation-Function (null fr RSAES-PKCS1-v1_5)
        /// </param>
        /// <param name="digestAlgorithm">der verwendete Hashalgorithmus (wird ignoriert fr RSAES-PKCS1-v1_5)
        /// </param>
        /// <returns> die zu entschlsselnden Daten
        /// </returns>
        public override byte[] decrypt(byte[] data, String mfgAlgorithm, String digestAlgorithm)
        {
            return de.osci.osci12.encryption.Crypto.doRSADecryption(key, data, mfgAlgorithm, digestAlgorithm);
        }
    }
}