namespace de.osci.osci12.samples.impl.crypto {
	
using System;
using de.osci.osci12.common;
using de.osci.osci12.extinterfaces.crypto;
using de.osci.osci12.dotcommon;
using org.apache.commons.logging;
using System.Security.Cryptography;
//using System.Security.Cryptography.X509Certificates;
using java.security.cert;
using System.IO;
using System.Collections;

using Org.BouncyCastle.Crypto.Signers;
using Org.BouncyCastle.Crypto.Engines;
using Org.BouncyCastle.Crypto;
using Org.BouncyCastle.Pkcs;
using Org.BouncyCastle.Math;
using Org.BouncyCastle.Crypto.Generators;
using Org.BouncyCastle.Crypto.Parameters;
using Org.BouncyCastle.Security;
using Org.BouncyCastle.X509;


	/// <summary><p>Diese Klasse ist eine Beispiel-Implementierung der abstrakten Signer-Klasse.
	/// Fr die Verwendung wird ein PKCS#12-Keystore in Form einer *.p12-Datei bentigt.
	/// Die Implementierung ist fr Testzwecke bestimmt, sie greift auf den ersten
	/// verfgbaren Alias zu. Die PIN fr dessen Privatschlssel muss die gleiche sein
	/// wie die des Keystores.</p>
	/// <p>Diese einfache Implementierung hlt die PIN des Keystores als Character-Array
	/// im Arbeitsspeicher, sie wird als String bergeben. Es wird Anwendern empfohlen,
	/// eigene Implementierungen zu schreiben, die die PIN in der Methode sign(...)
	/// abfragen und nach Gebrauch wieder lschen oder anderweitig fr ein sicheres
	/// Pin-Cashing zu sorgen.</p> 
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.extinterfaces.crypto.Signer">
	/// </seealso>
    public class PKCS12Signer : de.osci.osci12.extinterfaces.crypto.Signer
    {
        private static Log log = LogFactory.getLog(typeof(PKCS12Signer));
        private java.security.cert.X509Certificate cert;
        private bool usePSSforRSAkey;
        private AsymmetricKeyParameter pKey;


        /// <summary> Liefert den Namen des Herstellers.
        /// </summary>
        /// <value> Herstellername
        /// </value>
        public override String Vendor
        {
            get
            {
                return "BOS";
            }
        }

        /// <summary> Liefert die Versionsnummer.
        /// </summary>
        /// <value> Versionsnummer
        /// </value>
        public override String Version
        {
            get
            {
                return "1.5";
            }
        }

        /// <summary> Liefert das Signaturzertifikat.
        /// </summary>
        /// <value> Verschlsselungszertifikat
        /// </value>
        public override java.security.cert.X509Certificate Certificate
        {
            get
            {
                return cert;
            }
        }

        /// <summary>
        /// Dieser Konstruktor bentigt den Pfad zu einer PKCS#12-Privatschlsseldatei (*.p12),
        /// die dazugehrige PIN sowie den gewnschten Signaturalgorithmus. Es wird der erste
        /// in der Schlsseldatei gefundene Schlssel verwendet.
        /// </summary>
        /// <param name="p12_fileName">Pfad zur PKCS#12-Privatschlsseldatei (*.p12)</param>
        /// <param name="pin">PIN</param>
        public PKCS12Signer(String p12_fileName, String pin) : this(p12_fileName, pin, false)
        {            
        }

        /// <summary>
        /// Dieser Konstruktor bentigt den Pfad zu einer PKCS#12-Privatschlsseldatei (*.p12),
        /// die dazugehrige PIN sowie den gewnschten Signaturalgorithmus. Es wird der erste
        /// in der Schlsseldatei gefundene Schlssel verwendet. Falls derangegebene Algorithmus
        /// nicht zu dem gefundenen Privatschlssel passt (ECDSA/RSA), wird der Algorithmus
        /// unter Verwendung des gewhlten Hash-Algorithmus entsprechend angepasst.
        /// </summary>
        /// <param name="p12_fileName">Pfad zur PKCS#12-Privatschlsseldatei (*.p12)</param>
        /// <param name="pin">PIN</param>
        /// <param name="signatureAlgorithm">Signaturalgorithmus</param>
        public PKCS12Signer(String p12_fileName, String pin, bool _usePSSforRSAkey)
        {
            System.IO.Stream pfile = new FileStream(p12_fileName, FileMode.Open);
            Pkcs12Store ks = new Pkcs12Store(pfile, pin.ToCharArray());
            pfile.Close();
            IEnumerator al = ks.Aliases.GetEnumerator();
            while (al.MoveNext())
            {
                if (ks.IsKeyEntry((String)al.Current))
                {
                    pKey = ks.GetKey((String)al.Current).Key;
                    cert = new java.security.cert.X509Certificate(ks.GetCertificate((String)al.Current).Certificate.GetEncoded());
                    break;
                }
            }
            if (pKey == null)
                throw new ArgumentException("No private key found in keystore !");

            usePSSforRSAkey = _usePSSforRSAkey;
        }


        /// <summary> Signiert das bergebene Byte-Array.
        /// </summary>
        /// <param name="parm1"> die zu signierenden Daten
        /// </param>
        /// <param name="algorithm"> Algorithmus
        /// </param>
        /// <returns> die Signatur
        /// </returns>
        public override byte[] sign(byte[] data, String algorithm)
        {
            String digAlg = null;
            if (algorithm.Contains("sha256"))
                digAlg = "SHA-256";
            else if (algorithm.Contains("sha512"))
                digAlg = "SHA-512";
            else if (algorithm.Contains("sha1"))
                digAlg = "SHA-1";
            else if (algorithm.Contains("ripemd160"))
                digAlg = "RIPEMD160";

            if (algorithm.Contains("ecdsa"))
            {
                ECDsaSigner signer = new ECDsaSigner();
                signer.Init(true, pKey);
                byte[] digest = DigestUtilities.CalculateDigest(digAlg, data);
                BigInteger[] sigBigInts = signer.GenerateSignature(digest);
                int len = (((ECPrivateKeyParameters)pKey).Parameters.N.BitLength + 7) / 8;
                byte[] s = new byte[len * 2];
                byte[] s0 = sigBigInts[0].ToByteArray();
                Array.Copy(s0, s0.Length - len, s, 0, len);
                byte[] s1 = sigBigInts[1].ToByteArray();
                Array.Copy(s1, s1.Length - len, s, len, len);
                return s;
            }
            else
            {
                IDigest id = DigestUtilities.GetDigest(digAlg);
                ISigner rsads;
                if (algorithm.Contains("MGF"))
                    rsads = new PssSigner(new RsaEngine(), id);
                else
                    rsads = new RsaDigestSigner(id);
                rsads.Init(true, pKey);
                rsads.BlockUpdate(data, 0, data.Length);
                return rsads.GenerateSignature();
            }
        }

        public override String getAlgorithm()
        {
            String algo = DialogHandler.SignatureAlgorithm;

            if (pKey is ECPrivateKeyParameters)
            {
                if (algo.Contains("sha256"))
                    algo = Constants.SIGNATURE_ALGORITHM_ECDSA_SHA256;
                else
                    algo = Constants.SIGNATURE_ALGORITHM_ECDSA_SHA512;
            }
            else if (pKey is RsaPrivateCrtKeyParameters)
            {
                if (usePSSforRSAkey)
                {
                    if (algo.Contains("sha256"))
                        algo = Constants.SIGNATURE_ALGORITHM_RSA_SHA256_PSS;
                    else if (algo.Contains("sha512"))
                        algo = Constants.SIGNATURE_ALGORITHM_RSA_SHA512_PSS;
                }
                else
                {
                    if (algo.Contains("sha256"))
                        algo = Constants.SIGNATURE_ALGORITHM_RSA_SHA256;
                    else if (algo.Contains("sha512"))
                        algo = Constants.SIGNATURE_ALGORITHM_RSA_SHA512;
                }
            }
            else
                throw new ArgumentException("Unsupported algorithm found, key is of type " + pKey.GetType().ToString());

            return algo;
        }
    }
}